@@ -1,7 +1,7 @@
 /*
-   Copyright (C) CFEngine AS
+   Copyright 2017 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by CFEngine AS.
+   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -29,6 +29,7 @@
 #include <files_interfaces.h>
 #include <string_lib.h>
 #include <bootstrap.h>
+#include <policy_server.h>
 #include <dbm_api.h>
 #include <dbm_priv.h>
 #include <tokyo_check.h>
@@ -169,7 +170,7 @@ static AgentDiagnosticsResult AgentDiagnosticsCheckDB(ARG_UNUSED const char *wor
                 }
             }
             return AgentDiagnosticsResultNew(true, xstrdup("OK"));
-            
+
         }
     }
 }
