@@ -1,7 +1,7 @@
 /*
-   Copyright (C) CFEngine AS
+   Copyright 2017 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by CFEngine AS.
+   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -73,6 +73,7 @@
 #include <syslog_client.h>
 #include <man.h>
 #include <bootstrap.h>
+#include <policy_server.h>
 #include <misc_lib.h>
 #include <buffer.h>
 #include <loading.h>
@@ -186,6 +187,8 @@ static const struct option OPTIONS[] =
     {"color", optional_argument, 0, 'C'},
     {"no-extensions", no_argument, 0, 'E'},
     {"timestamp", no_argument, 0, 'l'},
+    /* Only long option for the rest */
+    {"log-modules", required_argument, 0, 0},
     {NULL, 0, 0, '\0'}
 };
 
@@ -210,6 +213,7 @@ static const char *const HINTS[] =
     "Enable colorized output. Possible values: 'always', 'auto', 'never'. If option is used, the default value is 'auto'",
     "Disable extension loading (used while upgrading)",
     "Log timestamps on each line of log output",
+    "Enable even more detailed debug logging for specific areas of the implementation. Use together with '-d'. Use --log-modules=help for a list of available modules",
     NULL
 };
 
@@ -234,7 +238,7 @@ int main(int argc, char *argv[])
     GenericAgentDiscoverContext(ctx, config);
 
     Policy *policy = SelectAndLoadPolicy(config, ctx, ALWAYS_VALIDATE, true);
-    
+
     if (!policy)
     {
         Log(LOG_LEVEL_ERR, "Error reading CFEngine policy. Exiting...");
@@ -264,7 +268,7 @@ int main(int argc, char *argv[])
 
     PolicyDestroy(policy); /* Can we safely do this earlier ? */
     int ret = 0;
-    if (config->agent_specific.agent.bootstrap_policy_server && !VerifyBootstrap())
+    if (config->agent_specific.agent.bootstrap_argument && !VerifyBootstrap())
     {
         RemovePolicyServerFile(GetWorkDir());
         WriteAmPolicyHubFile(false);
@@ -292,7 +296,7 @@ static GenericAgentConfig *CheckOpts(int argc, char **argv)
 {
     extern char *optarg;
     int c;
-    
+
     GenericAgentConfig *config = GenericAgentConfigNewDefault(AGENT_TYPE_AGENT, GetTTYInteractive());
     bool option_trust_server = false;
 ;
@@ -306,8 +310,9 @@ static GenericAgentConfig *CheckOpts(int argc, char **argv)
     char **argv_new = TranslateOldBootstrapOptionsConcatenated(argc_new, argv_tmp);
     FreeFixedStringArray(argc_new, argv_tmp);
 
+    int longopt_idx;
     while ((c = getopt_long(argc_new, argv_new, "tdvnKIf:w:D:N:VxMB:b:hC::ElT::",
-                            OPTIONS, NULL))
+                            OPTIONS, &longopt_idx))
            != -1)
     {
         switch (c)
@@ -373,13 +378,17 @@ static GenericAgentConfig *CheckOpts(int argc, char **argv)
                 // temporary assure that network functions are working
                 OpenNetwork();
 
-                char mapped_policy_server[CF_MAX_IP_LEN] = "";
-                if (Hostname2IPString(mapped_policy_server, optarg,
-                                      sizeof(mapped_policy_server)) == -1)
+                config->agent_specific.agent.bootstrap_argument = xstrdup(optarg);
+
+                char *host, *port;
+                ParseHostPort(optarg, &host, &port);
+
+                char ipaddr[CF_MAX_IP_LEN] = "";
+                if (Hostname2IPString(ipaddr, host,sizeof(ipaddr)) == -1)
                 {
                     Log(LOG_LEVEL_ERR,
                         "Could not resolve hostname '%s', unable to bootstrap",
-                        optarg);
+                        host);
                     exit(EXIT_FAILURE);
                 }
 
@@ -387,10 +396,19 @@ static GenericAgentConfig *CheckOpts(int argc, char **argv)
 
                 MINUSF = true;
                 config->ignore_locks = true;
-                GenericAgentConfigSetInputFile(config, GetInputDir(),
-                                               "promises.cf");
-                config->agent_specific.agent.bootstrap_policy_server =
-                    xstrdup(mapped_policy_server);
+                GenericAgentConfigSetInputFile(config, GetInputDir(), "promises.cf");
+
+                config->agent_specific.agent.bootstrap_ip = xstrdup(ipaddr);
+                config->agent_specific.agent.bootstrap_host = xstrdup(host);
+
+                if (port == NULL)
+                {
+                    config->agent_specific.agent.bootstrap_port = NULL;
+                }
+                else
+                {
+                    config->agent_specific.agent.bootstrap_port = xstrdup(port);
+                }
             }
             break;
 
@@ -452,7 +470,7 @@ static GenericAgentConfig *CheckOpts(int argc, char **argv)
         case 'h':
             {
                 Writer *w = FileWriter(stdout);
-                GenericAgentWriteHelp(w, "cf-agent", OPTIONS, HINTS, true);
+                WriterWriteHelp(w, "cf-agent", OPTIONS, HINTS, true);
                 FileWriterDetach(w);
             }
             exit(EXIT_SUCCESS);
@@ -517,10 +535,23 @@ static GenericAgentConfig *CheckOpts(int argc, char **argv)
 
             break;
 
+        /* long options only */
+        case 0:
+
+            if (strcmp(OPTIONS[longopt_idx].name, "log-modules") == 0)
+            {
+                bool ret = LogEnableModulesFromString(optarg);
+                if (!ret)
+                {
+                    exit(EXIT_FAILURE);
+                }
+            }
+            break;
+
         default:
             {
                 Writer *w = FileWriter(stdout);
-                GenericAgentWriteHelp(w, "cf-agent", OPTIONS, HINTS, true);
+                WriterWriteHelp(w, "cf-agent", OPTIONS, HINTS, true);
                 FileWriterDetach(w);
             }
             exit(EXIT_FAILURE);
@@ -535,7 +566,7 @@ static GenericAgentConfig *CheckOpts(int argc, char **argv)
     }
 
     if (option_trust_server &&
-        config->agent_specific.agent.bootstrap_policy_server == NULL)
+        config->agent_specific.agent.bootstrap_argument == NULL)
     {
         Log(LOG_LEVEL_ERR,
             "Option --trust-server can only be used when bootstrapping");
@@ -718,9 +749,12 @@ static void KeepControlPromises(EvalContext *ctx, const Policy *policy, GenericA
             }
 
             VarRef *ref = VarRefParseFromScope(cp->lval, "control_agent");
-            const void *value = EvalContextVariableGet(ctx, ref, NULL);
+            DataType value_type;
+            const void *value = EvalContextVariableGet(ctx, ref, &value_type);
             VarRefDestroy(ref);
-            if (!value)
+
+            /* If var not found, or if it's an empty list. */
+            if (value_type == CF_DATA_TYPE_NONE || value == NULL)
             {
                 Log(LOG_LEVEL_ERR, "Unknown lval '%s' in agent control body", cp->lval);
                 continue;
@@ -1027,10 +1061,10 @@ static void KeepControlPromises(EvalContext *ctx, const Policy *policy, GenericA
 
                 continue;
             }
-            
+
             if (strcmp(cp->lval, CFA_CONTROLBODY[AGENT_CONTROL_SELECT_END_MATCH_EOF].lval) == 0)
             {
-                Log(LOG_LEVEL_VERBOSE, "SET select_end_match_eof %s", value);
+                Log(LOG_LEVEL_VERBOSE, "SET select_end_match_eof %s", (char *) value);
                 EvalContextSetSelectEndMatchEof(ctx, BooleanFromString(value));
             }
 
@@ -1038,7 +1072,7 @@ static void KeepControlPromises(EvalContext *ctx, const Policy *policy, GenericA
             {
                 config->agent_specific.agent.report_class_log = BooleanFromString(value);
 
-                Log(LOG_LEVEL_VERBOSE, "Setting report_class_log to %s", 
+                Log(LOG_LEVEL_VERBOSE, "Setting report_class_log to %s",
                     config->agent_specific.agent.report_class_log? "true" : "false");
                 continue;
             }
@@ -1089,7 +1123,7 @@ static void KeepControlPromises(EvalContext *ctx, const Policy *policy, GenericA
     Nova_Initialize(ctx);
 
     // If not have been enabled above then should be disabled.
-    // By default it's enabled to catch all set classes on startup stage 
+    // By default it's enabled to catch all set classes on startup stage
     // before this part of the policy is processed.
     if (!config->agent_specific.agent.report_class_log)
     {
@@ -1130,11 +1164,6 @@ static void KeepPromiseBundles(EvalContext *ctx, const Policy *policy, GenericAg
         {
         case RVAL_TYPE_SCALAR:
             name = RlistScalarValue(rp);
-            if (strcmp(name, CF_NULL_VALUE) == 0)
-            {
-                continue;
-            }
-
             break;
         case RVAL_TYPE_FNCALL:
             name = RlistFnCallValue(rp)->name;
@@ -1198,10 +1227,6 @@ static void KeepPromiseBundles(EvalContext *ctx, const Policy *policy, GenericAg
             args = NULL;
             break;
         }
-        if (!strcmp(name, CF_NULL_VALUE))
-        {
-            continue;
-        }
 
         EvalContextSetBundleArgs(ctx, args);
 
@@ -1459,7 +1484,7 @@ static PromiseResult DefaultVarPromise(EvalContext *ctx, const Promise *pp)
         VarRefDestroy(ref);
     }
 
-    return VerifyVarPromise(ctx, pp, true);
+    return VerifyVarPromise(ctx, pp, NULL);
 }
 
 static void LogVariableValue(const EvalContext *ctx, const Promise *pp)
@@ -1533,10 +1558,11 @@ static PromiseResult KeepAgentPromise(EvalContext *ctx, const Promise *pp, ARG_U
     if (strcmp("meta", pp->parent_promise_type->name) == 0 ||
         strcmp("vars", pp->parent_promise_type->name) == 0)
     {
-        result = VerifyVarPromise(ctx, pp, true);
+        Log(LOG_LEVEL_VERBOSE, "V:     Computing value of '%s'", pp->promiser);
+
+        result = VerifyVarPromise(ctx, pp, NULL);
         if (result != PROMISE_RESULT_FAIL)
         {
-            Log(LOG_LEVEL_VERBOSE, "V:     Computing value of \"%s\"", pp->promiser);
             if (LogGetGlobalLevel() >= LOG_LEVEL_DEBUG)
             {
                 LogVariableValue(ctx, pp);
@@ -1682,7 +1708,9 @@ static void BannerStatus(PromiseResult status, char *type, char *name)
         break;
     }
 
-    Log(LOG_LEVEL_VERBOSE, "P: END %s promise (%.30s...)\n", type, name);
+    Log(LOG_LEVEL_VERBOSE, "P: END %s promise (%.30s%s)",
+        type, name,
+        (strlen(name) > 30) ? "..." : "");
 }
 
 /*********************************************************************/
@@ -1825,7 +1853,8 @@ static PromiseResult ParallelFindAndVerifyFilesPromises(EvalContext *ctx, const
 
 static bool VerifyBootstrap(void)
 {
-    if (NULL_OR_EMPTY(POLICY_SERVER))
+    const char *policy_server = PolicyServerGet();
+    if (NULL_OR_EMPTY(policy_server))
     {
         Log(LOG_LEVEL_ERR, "Bootstrapping failed, no policy server is specified");
         return false;
@@ -1855,7 +1884,8 @@ static bool VerifyBootstrap(void)
         return false;
     }
 
-    Log(LOG_LEVEL_NOTICE, "Bootstrap to '%s' completed successfully!", POLICY_SERVER);
+
+    Log(LOG_LEVEL_NOTICE, "Bootstrap to '%s' completed successfully!", policy_server);
     return true;
 }
 
@@ -1867,12 +1897,11 @@ static int NoteBundleCompliance(const Bundle *bundle, int save_pr_kept, int save
 {
     double delta_pr_kept, delta_pr_repaired, delta_pr_notkept;
     double bundle_compliance = 0.0;
-        
+
     delta_pr_kept = (double) (PR_KEPT - save_pr_kept);
     delta_pr_notkept = (double) (PR_NOTKEPT - save_pr_notkept);
     delta_pr_repaired = (double) (PR_REPAIRED - save_pr_repaired);
 
-    Log(LOG_LEVEL_VERBOSE, "\n");
     Log(LOG_LEVEL_VERBOSE, "A: ...................................................");
     Log(LOG_LEVEL_VERBOSE, "A: Bundle Accounting Summary for '%s' in namespace %s", bundle->name, bundle->ns);
 
@@ -1880,7 +1909,6 @@ static int NoteBundleCompliance(const Bundle *bundle, int save_pr_kept, int save
     {
         Log(LOG_LEVEL_VERBOSE, "A: Zero promises executed for bundle '%s'", bundle->name);
         Log(LOG_LEVEL_VERBOSE, "A: ...................................................");
-        Log(LOG_LEVEL_VERBOSE, "\n");
         return PROMISE_RESULT_NOOP;
     }
     else
@@ -1888,7 +1916,7 @@ static int NoteBundleCompliance(const Bundle *bundle, int save_pr_kept, int save
         Log(LOG_LEVEL_VERBOSE, "A: Promises kept in '%s' = %.0lf", bundle->name, delta_pr_kept);
         Log(LOG_LEVEL_VERBOSE, "A: Promises not kept in '%s' = %.0lf", bundle->name, delta_pr_notkept);
         Log(LOG_LEVEL_VERBOSE, "A: Promises repaired in '%s' = %.0lf", bundle->name, delta_pr_repaired);
-    
+
         bundle_compliance = (delta_pr_kept + delta_pr_repaired) / (delta_pr_kept + delta_pr_notkept + delta_pr_repaired);
 
         Log(LOG_LEVEL_VERBOSE, "A: Aggregate compliance (promises kept/repaired) for bundle '%s' = %.1lf%%",
@@ -1905,11 +1933,10 @@ static int NoteBundleCompliance(const Bundle *bundle, int save_pr_kept, int save
             EndMeasure(NULL, start);
         }
         Log(LOG_LEVEL_VERBOSE, "A: ...................................................");
-        Log(LOG_LEVEL_VERBOSE, "\n");
     }
 
     // return the worst case for the bundle status
-    
+
     if (delta_pr_notkept > 0)
     {
         return PROMISE_RESULT_FAIL;
@@ -1954,9 +1981,12 @@ static int AutomaticBootstrap(GenericAgentConfig *config)
         Log(LOG_LEVEL_NOTICE, "Autodiscovered hub installed on hostname '%s', IP address '%s'",
             hostname, ipaddr);
 
-        if (strlen(ipaddr) < sizeof(POLICY_SERVER))
+        // TODO: This is a very bad way to check for valid IP(?)
+        if (strlen(ipaddr) < CF_MAX_IP_LEN)
         {
-            config->agent_specific.agent.bootstrap_policy_server = xstrdup(ipaddr);
+            config->agent_specific.agent.bootstrap_argument = xstrdup(ipaddr);
+            config->agent_specific.agent.bootstrap_ip       = xstrdup(ipaddr);
+            config->agent_specific.agent.bootstrap_host     = xstrdup(ipaddr);
             ret = 0;
         }
         else
