@@ -1,7 +1,7 @@
 /*
-   Copyright (C) CFEngine AS
+   Copyright 2017 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by CFEngine AS.
+   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -83,11 +83,11 @@ PackageModuleWrapper *NewPackageModuleWrapper(PackageModuleBody *package_module)
         DeletePackageModuleWrapper(wrapper);
         return NULL;
     }
-    
+
     Log(LOG_LEVEL_DEBUG,
         "Successfully created package module wrapper for '%s' package module.",
         package_module->name);
-    
+
     return wrapper;
 }
 
@@ -95,29 +95,23 @@ PackageModuleWrapper *NewPackageModuleWrapper(PackageModuleBody *package_module)
 void UpdatePackagesCache(EvalContext *ctx, bool force_update)
 {
     Log(LOG_LEVEL_DEBUG, "Updating package cache.");
-    
-    PackagePromiseGlobalLock package_lock = 
+
+    PackagePromiseGlobalLock package_lock =
             AcquireGlobalPackagePromiseLock(ctx);
-                        
+
     if (package_lock.g_lock.lock == NULL)
     {
         Log(LOG_LEVEL_INFO, "Can not acquire global lock for package promise. "
             "Skipping updating cache.");
         return;
     }
-                        
+
     Rlist *default_inventory = GetDefaultInventoryFromContext(ctx);
-    
+
     for (const Rlist *rp = default_inventory; rp != NULL; rp = rp->next)
     {
         const char *pm_name =  RlistScalarValue(rp);
-        
-        /* We don't want inventory to be reported. */
-        if (StringSafeEqual(pm_name, "cf_null"))
-        {
-            break;
-        }
-        
+
         PackageModuleBody *module = GetPackageModuleFromContext(ctx, pm_name);
         if (!module)
         {
@@ -1079,7 +1073,7 @@ PromiseResult RepoInstall(EvalContext *ctx,
         {
             Log(LOG_LEVEL_VERBOSE,
                 "Package '%s' is already in the latest version. "
-                "Skipping instalation.", package_info->name);
+                "Skipping installation.", package_info->name);
 
             return PROMISE_RESULT_NOOP;
         }
@@ -1121,7 +1115,7 @@ PromiseResult RepoInstall(EvalContext *ctx,
             {
                 Log(LOG_LEVEL_VERBOSE,
                     "Package version from updates matches one installed. "
-                    "Skipping package instalation.");
+                    "Skipping package installation.");
                 res = PromiseResultUpdate(res, PROMISE_RESULT_NOOP);
                 continue;
             }
