@@ -1,7 +1,7 @@
 /*
-   Copyright (C) CFEngine AS
+   Copyright 2017 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by CFEngine AS.
+   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -419,7 +419,7 @@ static PromiseResult VerifyVirtNetwork(EvalContext *ctx, char *uri, enum cfhyper
         break;
 
     default:
-        Log(LOG_LEVEL_INFO, "No recogized state specified for this network environment");
+        Log(LOG_LEVEL_INFO, "No recognized state specified for this network environment");
         break;
     }
 
