@@ -1,7 +1,7 @@
 /*
-   Copyright (C) CFEngine AS
+   Copyright 2017 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by CFEngine AS.
+   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -99,13 +99,6 @@ PromiseResult VerifyMethod(EvalContext *ctx, const Rval call, Attributes a, cons
         while (args)
         {
            ++arg_index;
-           if (strcmp(args->val.item, CF_NULL_VALUE) == 0)
-           {
-               Log(LOG_LEVEL_DEBUG, "Skipping invokation of method '%s' due to null-values in argument '%d'",
-                   fp->name, arg_index);
-               BufferDestroy(method_name);
-               return PROMISE_RESULT_SKIPPED;
-           }
            args = args->next;
         }
         args = fp->args;
