@@ -1,7 +1,7 @@
 /*
-   Copyright (C) CFEngine AS
+   Copyright 2017 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by CFEngine AS.
+   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -69,31 +69,6 @@ static int ServicesSanityChecks(Attributes a, const Promise *pp)
 {
     Rlist *dep;
 
-    switch (a.service.service_policy)
-    {
-    case SERVICE_POLICY_START:
-        break;
-
-    case SERVICE_POLICY_STOP:
-    case SERVICE_POLICY_DISABLE:
-    case SERVICE_POLICY_RESTART:
-    case SERVICE_POLICY_RELOAD:
-        if (strcmp(a.service.service_autostart_policy, "none") != 0)
-        {
-            Log(LOG_LEVEL_ERR,
-                "!! Autostart policy of service promiser '%s' needs to be 'none' when service policy is not 'start', but is '%s'",
-                pp->promiser, a.service.service_autostart_policy);
-            PromiseRef(LOG_LEVEL_ERR, pp);
-            return false;
-        }
-        break;
-
-    default:
-        Log(LOG_LEVEL_ERR, "Invalid service policy for service '%s'", pp->promiser);
-        PromiseRef(LOG_LEVEL_ERR, pp);
-        return false;
-    }
-
     for (dep = a.service.service_depend; dep != NULL; dep = dep->next)
     {
         if (strcmp(pp->promiser, RlistScalarValue(dep)) == 0)
@@ -130,6 +105,14 @@ static int ServicesSanityChecks(Attributes a, const Promise *pp)
 
 static void SetServiceDefaults(Attributes *a)
 {
+    if ((a->service.service_policy == NULL) ||
+        strcmp(a->service.service_policy, "") == 0)
+    {
+        Log(LOG_LEVEL_VERBOSE,
+            "Attribute 'service_policy' was not set, defaulting to: 'start'");
+        a->service.service_policy = "start";
+    }
+
     if (a->service.service_autostart_policy == NULL)
     {
         a->service.service_autostart_policy = "none";
@@ -193,35 +176,13 @@ static PromiseResult VerifyServices(EvalContext *ctx, Attributes a, const Promis
 /* Level                                                                     */
 /*****************************************************************************/
 
-static FnCall *DefaultServiceBundleCall(const Promise *pp, ServicePolicy service_policy)
+static FnCall *DefaultServiceBundleCall(const Promise *pp, const char *service_policy)
 {
     Rlist *args = NULL;
     FnCall *call = NULL;
 
-    switch (service_policy)
-    {
-    case SERVICE_POLICY_START:
-        RlistAppend(&args, pp->promiser, RVAL_TYPE_SCALAR);
-        RlistAppend(&args, "start", RVAL_TYPE_SCALAR);
-        break;
-
-    case SERVICE_POLICY_RESTART:
-        RlistAppend(&args, pp->promiser, RVAL_TYPE_SCALAR);
-        RlistAppend(&args, "restart", RVAL_TYPE_SCALAR);
-        break;
-
-    case SERVICE_POLICY_RELOAD:
-        RlistAppend(&args, pp->promiser, RVAL_TYPE_SCALAR);
-        RlistAppend(&args, "restart", RVAL_TYPE_SCALAR);
-        break;
-
-    case SERVICE_POLICY_STOP:
-    case SERVICE_POLICY_DISABLE:
-    default:
-        RlistAppend(&args, pp->promiser, RVAL_TYPE_SCALAR);
-        RlistAppend(&args, "stop", RVAL_TYPE_SCALAR);
-        break;
-    }
+    RlistAppend(&args, pp->promiser, RVAL_TYPE_SCALAR);
+    RlistAppend(&args, service_policy, RVAL_TYPE_SCALAR);
 
     Rval name = DefaultBundleConstraint(pp, "service");
 
@@ -254,26 +215,7 @@ static PromiseResult DoVerifyServices(EvalContext *ctx, Attributes a, const Prom
     }
     a.havebundle = true;
 
-    switch (a.service.service_policy)
-    {
-    case SERVICE_POLICY_START:
-        EvalContextVariablePutSpecial(ctx, SPECIAL_SCOPE_THIS, "service_policy", "start", CF_DATA_TYPE_STRING, "source=promise");
-        break;
-
-    case SERVICE_POLICY_RESTART:
-        EvalContextVariablePutSpecial(ctx, SPECIAL_SCOPE_THIS, "service_policy", "restart", CF_DATA_TYPE_STRING, "source=promise");
-        break;
-
-    case SERVICE_POLICY_RELOAD:
-        EvalContextVariablePutSpecial(ctx, SPECIAL_SCOPE_THIS, "service_policy", "reload", CF_DATA_TYPE_STRING, "source=promise");
-        break;
-
-    case SERVICE_POLICY_STOP:
-    case SERVICE_POLICY_DISABLE:
-    default:
-        EvalContextVariablePutSpecial(ctx, SPECIAL_SCOPE_THIS, "service_policy", "stop", CF_DATA_TYPE_STRING, "source=promise");
-        break;
-    }
+    EvalContextVariablePutSpecial(ctx, SPECIAL_SCOPE_THIS, "service_policy", a.service.service_policy, CF_DATA_TYPE_STRING, "source=promise");
 
     PromiseResult result = PROMISE_RESULT_NOOP;
     result = PromiseResultUpdate(result, VerifyMethod(ctx, call, a, pp));  // Send list of classes to set privately?
