@@ -1,7 +1,7 @@
 /*
-   Copyright (C) CFEngine AS
+   Copyright 2017 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by CFEngine AS.
+   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -33,3 +33,13 @@ void VerifyOneUsersPromise (ARG_UNUSED const char *puser, ARG_UNUSED User u,
 {
     Log(LOG_LEVEL_ERR, "Users promise type is not supported on this OS");
 }
+
+bool IsAccountLocked(const char *puser, const void *passwd_info)
+{
+    return false;
+}
+
+bool GetPasswordHash(const char *puser, const void *passwd_info, const char **result)
+{
+    return false;
+}
