@@ -1,7 +1,7 @@
 /*
-   Copyright (C) CFEngine AS
+   Copyright 2017 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by CFEngine AS.
+   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -40,6 +40,7 @@
 #include <crypto.h>
 #include <known_dirs.h>
 #include <bootstrap.h>
+#include <policy_server.h>
 #include <files_hashes.h>
 #include <item_lib.h>
 
@@ -184,11 +185,6 @@ void LocalExec(const ExecConfig *config)
     if (strlen(config->exec_command) > 0)
     {
         strlcpy(cmd, config->exec_command, CF_BUFSIZE);
-
-        if (!strstr(cmd, "-Dfrom_cfexecd"))
-        {
-            strcat(cmd, " -Dfrom_cfexecd");
-        }
     }
     else
     {
@@ -765,7 +761,7 @@ static void MailResult(const ExecConfig *config, const char *file)
             snprintf(line, line_size,
                      "\r\n[Mail truncated by CFEngine. File is at %s on %s]\r\n",
                      file, config->fq_name);
-            if (send(sd, line, strlen(line), 0))
+            if (send(sd, line, strlen(line), 0) == -1)
             {
                 Log(LOG_LEVEL_ERR, "Error while sending mail to mailserver "
                     "'%s'. (send: '%s')", config->mail_server, GetErrorStr());
@@ -839,4 +835,3 @@ static int Dialogue(int sd, const char *s)
 
     return ((f == '2') || (f == '3'));  /* return code 200 or 300 from smtp */
 }
-
