@@ -1,7 +1,7 @@
 /*
-   Copyright (C) CFEngine AS
+   Copyright 2017 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by CFEngine AS.
+   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -25,6 +25,8 @@
 #ifndef CF_EXECD_H
 #define CF_EXECD_H
 
+#include <cf3.defs.h>
+
 #include <eval_context.h>
 #include <policy.h>
 #include <generic_agent.h>
