@@ -1,7 +1,7 @@
 /*
-   Copyright (C) CFEngine AS
+   Copyright 2017 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by CFEngine AS.
+   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -25,10 +25,11 @@
 #include <cf-serverd-functions.h>
 #include <cf-serverd-enterprise-stubs.h>
 
-#include "server_access.h"
+#include <server_access.h>
 #include <client_code.h>
 #include <server_transform.h>
 #include <bootstrap.h>
+#include <policy_server.h>
 #include <scope.h>
 #include <signals.h>
 #include <systype.h>
@@ -226,7 +227,7 @@ GenericAgentConfig *CheckOpts(int argc, char **argv)
         case 'h':
             {
                 Writer *w = FileWriter(stdout);
-                GenericAgentWriteHelp(w, "cf-serverd", OPTIONS, HINTS, true);
+                WriterWriteHelp(w, "cf-serverd", OPTIONS, HINTS, true);
                 FileWriterDetach(w);
             }
             exit(EXIT_SUCCESS);
@@ -275,7 +276,7 @@ GenericAgentConfig *CheckOpts(int argc, char **argv)
         default:
             {
                 Writer *w = FileWriter(stdout);
-                GenericAgentWriteHelp(w, "cf-serverd", OPTIONS, HINTS, true);
+                WriterWriteHelp(w, "cf-serverd", OPTIONS, HINTS, true);
                 FileWriterDetach(w);
             }
             exit(EXIT_FAILURE);
@@ -441,14 +442,13 @@ static void CheckFileChanges(EvalContext *ctx, Policy **policy, GenericAgentConf
              *   CollectCallStart();  // both
              */
 
-            char *existing_policy_server = ReadPolicyServerFile(GetWorkDir());
-            SetPolicyServer(ctx, existing_policy_server);
-            free(existing_policy_server);
+            SetPolicyServerFromFile(ctx, GetWorkDir());
 
             UpdateLastPolicyUpdateTime(ctx);
 
             DetectEnvironment(ctx);
             KeepHardClasses(ctx);
+            LoadAugments(ctx, config);
 
             /* During startup this is done in GenericAgentDiscoverContext(). */
             EvalContextClassPutHard(ctx, CF_AGENTTYPES[AGENT_TYPE_SERVER], "cfe_internal,source=agent");
@@ -752,7 +752,7 @@ static void CollectCallIfDue(EvalContext *ctx)
 
             ConnectionInfoSetSocket(info, new_client);
             info->is_call_collect = true; /* Mark processed when done. */
-            ServerEntryPoint(ctx, POLICY_SERVER, info);
+            ServerEntryPoint(ctx, PolicyServerGetIP(), info);
         }
     }
 }
