@@ -1,7 +1,7 @@
 /*
-   Copyright (C) CFEngine AS
+   Copyright 2017 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by CFEngine AS.
+   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -793,15 +793,8 @@ int StatFile(ServerConnectionState *conn, char *sendbuffer, char *ofilename)
         cfst.cf_nlink = statbuf.st_nlink;
     }
 
-#if !defined(__MINGW32__)
-    if (statbuf.st_size > statbuf.st_blocks * DEV_BSIZE)
-#else
-# ifdef HAVE_ST_BLOCKS
-    if (statbuf.st_size > statbuf.st_blocks * DEV_BSIZE)
-# else
-    if (statbuf.st_size > ST_NBLOCKS(statbuf) * DEV_BSIZE)
-# endif
-#endif
+    /* Is file sparse? */
+    if (statbuf.st_size > ST_NBYTES(statbuf))
     {
         cfst.cf_makeholes = 1;  /* must have a hole to get checksum right */
     }
@@ -891,7 +884,9 @@ void GetServerLiteral(EvalContext *ctx, ServerConnectionState *conn, char *sendb
 
     if (encrypted)
     {
-        cipherlen = EncryptString(conn->encryption_type, sendbuffer, out, conn->session_key, strlen(sendbuffer) + 1);
+        cipherlen = EncryptString(out, sizeof(out),
+                                  sendbuffer, strlen(sendbuffer) + 1,
+                                  conn->encryption_type, conn->session_key);
         SendTransaction(conn->conn_info, out, cipherlen, CF_DONE);
     }
     else
@@ -931,8 +926,9 @@ void ReplyServerContext(ServerConnectionState *conn, int encrypted, Item *classe
     if (encrypted)
     {
         char out[CF_BUFSIZE];
-        int cipherlen = EncryptString(conn->encryption_type, sendbuffer, out,
-                                      conn->session_key, strlen(sendbuffer) + 1);
+        int cipherlen = EncryptString(out, sizeof(out),
+                                      sendbuffer, strlen(sendbuffer) + 1,
+                                      conn->encryption_type, conn->session_key);
         SendTransaction(conn->conn_info, out, cipherlen, CF_DONE);
     }
     else
@@ -1008,7 +1004,9 @@ int CfSecOpenDirectory(ServerConnectionState *conn, char *sendbuffer, char *dirn
     if (!IsAbsoluteFileName(dirname))
     {
         strcpy(sendbuffer, "BAD: request to access a non-absolute filename");
-        cipherlen = EncryptString(conn->encryption_type, sendbuffer, out, conn->session_key, strlen(sendbuffer) + 1);
+        cipherlen = EncryptString(out, sizeof(out),
+                                  sendbuffer, strlen(sendbuffer) + 1,
+                                  conn->encryption_type, conn->session_key);
         SendTransaction(conn->conn_info, out, cipherlen, CF_DONE);
         return -1;
     }
@@ -1017,7 +1015,9 @@ int CfSecOpenDirectory(ServerConnectionState *conn, char *sendbuffer, char *dirn
     {
         Log(LOG_LEVEL_VERBOSE, "Couldn't open dir %s", dirname);
         snprintf(sendbuffer, CF_BUFSIZE, "BAD: cfengine, couldn't open dir %s", dirname);
-        cipherlen = EncryptString(conn->encryption_type, sendbuffer, out, conn->session_key, strlen(sendbuffer) + 1);
+        cipherlen = EncryptString(out, sizeof(out),
+                                  sendbuffer, strlen(sendbuffer) + 1,
+                                  conn->encryption_type, conn->session_key);
         SendTransaction(conn->conn_info, out, cipherlen, CF_DONE);
         return -1;
     }
@@ -1032,7 +1032,9 @@ int CfSecOpenDirectory(ServerConnectionState *conn, char *sendbuffer, char *dirn
     {
         if (strlen(dirp->d_name) + 1 + offset >= CF_BUFSIZE - CF_MAXLINKSIZE)
         {
-            cipherlen = EncryptString(conn->encryption_type, sendbuffer, out, conn->session_key, offset + 1);
+            cipherlen = EncryptString(out, sizeof(out),
+                                      sendbuffer, offset + 1,
+                                      conn->encryption_type, conn->session_key);
             SendTransaction(conn->conn_info, out, cipherlen, CF_MORE);
             offset = 0;
             memset(sendbuffer, 0, CF_BUFSIZE);
@@ -1047,7 +1049,9 @@ int CfSecOpenDirectory(ServerConnectionState *conn, char *sendbuffer, char *dirn
     strcpy(sendbuffer + offset, CFD_TERMINATOR);
 
     cipherlen =
-        EncryptString(conn->encryption_type, sendbuffer, out, conn->session_key, offset + 2 + strlen(CFD_TERMINATOR));
+        EncryptString(out, sizeof(out),
+                      sendbuffer, offset + 2 + strlen(CFD_TERMINATOR),
+                      conn->encryption_type, conn->session_key);
     SendTransaction(conn->conn_info, out, cipherlen, CF_DONE);
     DirClose(dirh);
     return 0;
@@ -1606,7 +1610,7 @@ static bool AuthorizeDelimitedArgs(const ServerConnectionState *conn,
 /**
  * @return #true if the connection should remain open for next requests, or
  *         #false if the server should actively close it - for example when
- *         protocol errors have occured.
+ *         protocol errors have occurred.
  */
 bool DoExec2(const EvalContext *ctx,
              ServerConnectionState *conn,
