@@ -1,7 +1,7 @@
 /*
-   Copyright (C) CFEngine AS
+   Copyright 2017 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by CFEngine AS.
+   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -243,6 +243,36 @@ void KeepPromises(EvalContext *ctx, const Policy *policy, GenericAgentConfig *co
 
 /*******************************************************************/
 
+static bool SetMaxOpenFiles(int n)
+#ifdef HAVE_SYS_RESOURCE_H
+{
+    const struct rlimit lim = {
+        .rlim_cur = n,
+        .rlim_max = n,
+    };
+    int ret = setrlimit(RLIMIT_NOFILE, &lim);
+    if (ret == -1)
+    {
+        Log(LOG_LEVEL_INFO, "Failed setting max open files limit"
+            " (setrlimit(NOFILE, %d): %s)", n, GetErrorStr());
+        Log(LOG_LEVEL_INFO,
+            "Please ensure that 'nofile' ulimit is at least 2x maxconnections");
+        return false;
+    }
+    else
+    {
+        Log(LOG_LEVEL_VERBOSE, "Setting max open files rlimit to %d", n);
+        return true;
+    }
+}
+#else  /* MinGW */
+{
+    Log(LOG_LEVEL_VERBOSE,
+        "Platform does not support setrlimit(NOFILE), max open files not set");
+    return false;
+}
+#endif  /* HAVE_SYS_RESOURCE_H */
+
 static void KeepControlPromises(EvalContext *ctx, const Policy *policy, GenericAgentConfig *config)
 {
     CFD_MAXPROCESSES = 30;
@@ -260,12 +290,14 @@ static void KeepControlPromises(EvalContext *ctx, const Policy *policy, GenericA
     /* Now expand */
 
     Seq *constraints = ControlBodyConstraints(policy, AGENT_TYPE_SERVER);
+
+#define IsControlBody(e) (strcmp(cp->lval, CFS_CONTROLBODY[e].lval) == 0)
+
     if (constraints)
     {
         for (size_t i = 0; i < SeqLength(constraints); i++)
         {
             Constraint *cp = SeqAt(constraints, i);
-#define IsControlBody(e) (strcmp(cp->lval, CFS_CONTROLBODY[e].lval) == 0)
 
             if (!IsDefinedClass(ctx, cp->classes))
             {
@@ -276,13 +308,7 @@ static void KeepControlPromises(EvalContext *ctx, const Policy *policy, GenericA
             const void *value = EvalContextVariableGet(ctx, ref, NULL);
             VarRefDestroy(ref);
 
-            if (!value)
-            {
-                Log(LOG_LEVEL_ERR,
-                    "Unknown lval '%s' in server control body",
-                    cp->lval);
-            }
-            else if (IsControlBody(SERVER_CONTROL_SERVER_FACILITY))
+            if (IsControlBody(SERVER_CONTROL_SERVER_FACILITY))
             {
                 SetFacility(value);
             }
@@ -308,10 +334,10 @@ static void KeepControlPromises(EvalContext *ctx, const Policy *policy, GenericA
             else if (IsControlBody(SERVER_CONTROL_MAX_CONNECTIONS))
             {
                 CFD_MAXPROCESSES = (int) IntFromString(value);
+
+                /* Ease apoptosis limits. */
                 MAXTRIES = CFD_MAXPROCESSES / 3;
-                Log(LOG_LEVEL_VERBOSE,
-                    "Setting maxconnections to %d",
-                    CFD_MAXPROCESSES);
+
                 /* The handling of max_readers in LMDB is not ideal, but
                  * here is how it is right now: We know that both cf-serverd and
                  * cf-hub will access the lastseen database. Worst case every
@@ -320,9 +346,13 @@ static void KeepControlPromises(EvalContext *ctx, const Policy *policy, GenericA
                  * those two values together to provide a safe ceiling. In
                  * addition, cf-agent can access the database occasionally as
                  * well, so add a few extra for that too. */
+                Log(LOG_LEVEL_VERBOSE,
+                    "Setting maxconnections to %d", CFD_MAXPROCESSES);
                 DBSetMaximumConcurrentTransactions(CFD_MAXPROCESSES
                                                    + EnterpriseGetMaxCfHubProcesses() + 10);
-                continue;
+
+                /* Set RLIMIT_NOFILE to be enough for all threads. */
+                SetMaxOpenFiles(CFD_MAXPROCESSES * 2 + 10);
             }
             else if (IsControlBody(SERVER_CONTROL_CALL_COLLECT_INTERVAL))
             {
@@ -414,7 +444,7 @@ static void KeepControlPromises(EvalContext *ctx, const Policy *policy, GenericA
             }
             else if (IsControlBody(SERVER_CONTROL_TRUST_KEYS_FROM))
             {
-                Log(LOG_LEVEL_VERBOSE, "Setting trust keys from ...");
+                Log(LOG_LEVEL_VERBOSE, "Setting 'trustkeysfrom' ...");
 
                 for (const Rlist *rp = value; rp != NULL; rp = rp->next)
                 {
@@ -426,7 +456,7 @@ static void KeepControlPromises(EvalContext *ctx, const Policy *policy, GenericA
             }
             else if (IsControlBody(SERVER_CONTROL_ALLOWLEGACYCONNECTS))
             {
-                Log(LOG_LEVEL_VERBOSE, "Setting allowing legacy connections from ...");
+                Log(LOG_LEVEL_VERBOSE, "Setting 'allowlegacyconnects' ...");
 
                 for (const Rlist *rp = value; rp != NULL; rp = rp->next)
                 {
@@ -463,8 +493,10 @@ static void KeepControlPromises(EvalContext *ctx, const Policy *policy, GenericA
                 Log(LOG_LEVEL_VERBOSE, "Setting allowtlsversion to: %s",
                     SV.allowtlsversion);
             }
-#undef IsControlBody
         }
+
+#undef IsControlBody
+
     }
 
     const void *value = EvalContextVariableControlCommonGet(ctx, COMMON_CONTROL_SYSLOG_HOST);
@@ -649,7 +681,7 @@ static PromiseResult KeepServerPromise(EvalContext *ctx, const Promise *pp, ARG_
     {
         return VerifyVarPromise(ctx, pp, NULL);
     }
-    
+
     if (strcmp(pp->parent_promise_type->name, "classes") == 0)
     {
         return VerifyClassPromise(ctx, pp, NULL);
@@ -786,7 +818,8 @@ static size_t StrList_AppendRegexHostname(StrList **sl, const char *host)
             return (size_t) -1;
         }
 
-        if (name > host || NULL != strchr(host, '\\'))
+        if (name > host ||
+            strchr(host, '\\') != NULL)
         {
             /* 2: leading '.' and final '\0' */
             char copy[2 + strlen(name)], *c = copy;
