@@ -1,7 +1,7 @@
 /*
-   Copyright (C) CFEngine AS
+   Copyright 2017 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by CFEngine AS.
+   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -319,10 +319,18 @@ Item *RemoteDirList(const char *dirname, bool encrypt, AgentConnection *conn)
         }
 
         snprintf(in, CF_BUFSIZE, "OPENDIR %s", dirname);
-        cipherlen = EncryptString(conn->encryption_type, in, out, conn->session_key, strlen(in) + 1);
+        cipherlen = EncryptString(out, sizeof(out), in, strlen(in) + 1, conn->encryption_type, conn->session_key);
+
+        tosend = cipherlen + CF_PROTO_OFFSET;
+
+        if(tosend > sizeof(sendbuffer))
+        {
+            ProgrammingError("RemoteDirList: tosend (%d) > sendbuffer (%ld)",
+                             tosend, sizeof(sendbuffer));
+        }
+
         snprintf(sendbuffer, CF_BUFSIZE - 1, "SOPENDIR %d", cipherlen);
         memcpy(sendbuffer + CF_PROTO_OFFSET, out, cipherlen);
-        tosend = cipherlen + CF_PROTO_OFFSET;
     }
     else
     {
@@ -335,7 +343,7 @@ Item *RemoteDirList(const char *dirname, bool encrypt, AgentConnection *conn)
         return NULL;
     }
 
-    Item *start = NULL, *end = NULL;                  /* NULL == empty list */
+    Item *start = NULL, *end = NULL;                  /* NULL is empty list */
     while (true)
     {
         /* TODO check the CF_MORE flag, no need for CFD_TERMINATOR. */
@@ -351,8 +359,8 @@ Item *RemoteDirList(const char *dirname, bool encrypt, AgentConnection *conn)
         if (encrypt)
         {
             memcpy(in, recvbuffer, nbytes);
-            DecryptString(conn->encryption_type, in, recvbuffer,
-                          conn->session_key, nbytes);
+            DecryptString(recvbuffer, sizeof(recvbuffer), in, nbytes,
+                          conn->encryption_type, conn->session_key);
         }
 
         if (recvbuffer[0] == '\0')
@@ -441,11 +449,20 @@ int CompareHashNet(const char *file1, const char *file2, bool encrypt, AgentConn
         }
 
         cipherlen =
-            EncryptString(conn->encryption_type, in, out, conn->session_key,
-                          strlen(in) + CF_SMALL_OFFSET + CF_DEFAULT_DIGEST_LEN);
+            EncryptString(out, sizeof(out), in,
+                          strlen(in) + CF_SMALL_OFFSET + CF_DEFAULT_DIGEST_LEN,
+                          conn->encryption_type, conn->session_key);
+
+        tosend = cipherlen + CF_PROTO_OFFSET;
+
+        if(tosend > sizeof(sendbuffer))
+        {
+            ProgrammingError("CompareHashNet: tosend (%d) > sendbuffer (%ld)",
+                             tosend, sizeof(sendbuffer));
+        }
+
         snprintf(sendbuffer, CF_BUFSIZE, "SMD5 %d", cipherlen);
         memcpy(sendbuffer + CF_PROTO_OFFSET, out, cipherlen);
-        tosend = cipherlen + CF_PROTO_OFFSET;
     }
     else
     {
@@ -490,53 +507,13 @@ int CompareHashNet(const char *file1, const char *file2, bool encrypt, AgentConn
 /*********************************************************************/
 
 
-static int FSWrite(const char *destination, int dd, const char *buf, size_t n_write)
-{
-    const void *cur = buf;
-    const void *end = buf + n_write;
-
-    while (cur < end)
-    {
-        const void *skip_span = MemSpan(cur, 0, end - cur);
-        if (skip_span > cur)
-        {
-            if (lseek(dd, skip_span - cur, SEEK_CUR) < 0)
-            {
-                Log(LOG_LEVEL_ERR,
-                    "Copy failed (no space?) while copying to '%s' from network '%s'",
-                    destination, GetErrorStr());
-                return false;
-            }
-
-            cur = skip_span;
-        }
-
-        const void *copy_span = MemSpanInverse(cur, 0, end - cur);
-        if (copy_span > cur)
-        {
-            if (FullWrite(dd, cur, copy_span - cur) < 0)
-            {
-                Log(LOG_LEVEL_ERR,
-                    "Copy failed (no space?) while copying to '%s' from network '%s'",
-                    destination, GetErrorStr());
-                return false;
-            }
-
-            cur = copy_span;
-        }
-    }
-
-    return true;
-}
-
 int EncryptCopyRegularFileNet(const char *source, const char *dest, off_t size, AgentConnection *conn)
 {
-    int dd, blocksize = 2048, n_read = 0, towrite, plainlen, more = true, finlen, cnt = 0;
+    int dd, blocksize = 2048, n_read = 0, plainlen, more = true, finlen, cnt = 0;
     int tosend, cipherlen = 0;
     char *buf, in[CF_BUFSIZE], out[CF_BUFSIZE], workbuf[CF_BUFSIZE], cfchangedstr[265];
     unsigned char iv[32] =
         { 1, 2, 3, 4, 5, 6, 7, 8, 1, 2, 3, 4, 5, 6, 7, 8, 1, 2, 3, 4, 5, 6, 7, 8, 1, 2, 3, 4, 5, 6, 7, 8 };
-    long n_read_total = 0;
     EVP_CIPHER_CTX crypto_ctx;
 
     snprintf(cfchangedstr, 255, "%s%s", CF_CHANGEDSTR1, CF_CHANGEDSTR2);
@@ -569,10 +546,18 @@ int EncryptCopyRegularFileNet(const char *source, const char *dest, off_t size,
     EVP_CIPHER_CTX_init(&crypto_ctx);
 
     snprintf(in, CF_BUFSIZE - CF_PROTO_OFFSET, "GET dummykey %s", source);
-    cipherlen = EncryptString(conn->encryption_type, in, out, conn->session_key, strlen(in) + 1);
+    cipherlen = EncryptString(out, sizeof(out), in, strlen(in) + 1, conn->encryption_type, conn->session_key);
+
+    tosend = cipherlen + CF_PROTO_OFFSET;
+
+    if(tosend > sizeof(workbuf))
+    {
+        ProgrammingError("EncryptCopyRegularFileNet: tosend (%d) > workbuf (%ld)",
+                         tosend, sizeof(workbuf));
+    }
+
     snprintf(workbuf, CF_BUFSIZE, "SGET %4d %4d", cipherlen, blocksize);
     memcpy(workbuf + CF_PROTO_OFFSET, out, cipherlen);
-    tosend = cipherlen + CF_PROTO_OFFSET;
 
 /* Send proposition C0 - query */
 
@@ -585,7 +570,8 @@ int EncryptCopyRegularFileNet(const char *source, const char *dest, off_t size,
 
     buf = xmalloc(CF_BUFSIZE + sizeof(int));
 
-    n_read_total = 0;
+    bool   last_write_made_hole = false;
+    size_t n_wrote_total        = 0;
 
     while (more)
     {
@@ -599,7 +585,8 @@ int EncryptCopyRegularFileNet(const char *source, const char *dest, off_t size,
 
         /* If the first thing we get is an error message, break. */
 
-        if ((n_read_total == 0) && (strncmp(buf + CF_INBAND_OFFSET, CF_FAILEDSTR, strlen(CF_FAILEDSTR)) == 0))
+        if (n_wrote_total == 0 &&
+            strncmp(buf + CF_INBAND_OFFSET, CF_FAILEDSTR, strlen(CF_FAILEDSTR)) == 0)
         {
             Log(LOG_LEVEL_INFO, "Network access to '%s:%s' denied", conn->this_server, source);
             close(dd);
@@ -631,43 +618,38 @@ int EncryptCopyRegularFileNet(const char *source, const char *dest, off_t size,
             return false;
         }
 
-        towrite = n_read = plainlen + finlen;
+        n_read = plainlen + finlen;
 
-        n_read_total += n_read;
-
-        if (!FSWrite(dest, dd, workbuf, towrite))
+        bool w_ok = FileSparseWrite(dd, workbuf, n_read,
+                                    &last_write_made_hole);
+        if (!w_ok)
         {
-            Log(LOG_LEVEL_ERR, "Local disk write failed copying '%s:%s' to '%s:%s'",
-                conn->this_server, source, dest, GetErrorStr());
-            if (conn)
-            {
-                conn->error = true;
-            }
+            Log(LOG_LEVEL_ERR,
+                "Local disk write failed copying '%s:%s' to '%s'",
+                conn->this_server, source, dest);
             free(buf);
             unlink(dest);
             close(dd);
+            conn->error = true;
             EVP_CIPHER_CTX_cleanup(&crypto_ctx);
             return false;
         }
+
+        n_wrote_total += n_read;
     }
 
-    /* If the file ends with a `hole', something needs to be written at
-       the end.  Otherwise the kernel would truncate the file at the end
-       of the last write operation. Write a null character and truncate
-       it again.  */
+    const bool do_sync = false;
 
-    if (ftruncate(dd, n_read_total) < 0)
+    bool ret = FileSparseClose(dd, dest, do_sync,
+                               n_wrote_total, last_write_made_hole);
+    if (!ret)
     {
-        Log(LOG_LEVEL_ERR, "Copy failed (no space?) while copying '%s' from network '%s'",
-            dest, GetErrorStr());
-        free(buf);
         unlink(dest);
-        close(dd);
+        free(buf);
         EVP_CIPHER_CTX_cleanup(&crypto_ctx);
         return false;
     }
 
-    close(dd);
     free(buf);
     EVP_CIPHER_CTX_cleanup(&crypto_ctx);
     return true;
@@ -694,9 +676,6 @@ int CopyRegularFileNet(const char *source, const char *dest, off_t size,
     char *buf, workbuf[CF_BUFSIZE], cfchangedstr[265];
     const int buf_size = 2048;
 
-    off_t n_read_total = 0;
-    EVP_CIPHER_CTX crypto_ctx;
-
     /* We encrypt only for CLASSIC protocol. The TLS protocol is always over
      * encrypted layer, so it does not support encrypted (S*) commands. */
     encrypt = encrypt && conn->conn_info->protocol == CF_PROTOCOL_CLASSIC;
@@ -750,10 +729,11 @@ int CopyRegularFileNet(const char *source, const char *dest, off_t size,
     Log(LOG_LEVEL_VERBOSE, "Copying remote file '%s:%s', expecting %jd bytes",
           conn->this_server, source, (intmax_t)size);
 
-    n_read_total = 0;
-    while (n_read_total < size)
+    size_t n_wrote_total        = 0;
+    bool   last_write_made_hole = false;
+    while (n_wrote_total < size)
     {
-        int toget = MIN(size - n_read_total, buf_size);
+        int toget = MIN(size - n_wrote_total, buf_size);
 
         assert(toget > 0);
 
@@ -791,7 +771,8 @@ int CopyRegularFileNet(const char *source, const char *dest, off_t size,
 
         /* If the first thing we get is an error message, break. */
 
-        if ((n_read_total == 0) && (strncmp(buf, CF_FAILEDSTR, strlen(CF_FAILEDSTR)) == 0))
+        if (n_wrote_total == 0
+            && strncmp(buf, CF_FAILEDSTR, strlen(CF_FAILEDSTR) == 0))
         {
             Log(LOG_LEVEL_INFO, "Network access to '%s:%s' denied",
                 conn->this_server, source);
@@ -809,7 +790,6 @@ int CopyRegularFileNet(const char *source, const char *dest, off_t size,
             return false;
         }
 
-
         /* Check for mismatch between encryption here and on server. */
 
         int value = -1;
@@ -824,43 +804,36 @@ int CopyRegularFileNet(const char *source, const char *dest, off_t size,
             return false;
         }
 
-        if (!FSWrite(dest, dd, buf, n_read))
+        bool w_ok = FileSparseWrite(dd, buf, n_read,
+                                    &last_write_made_hole);
+        if (!w_ok)
         {
             Log(LOG_LEVEL_ERR,
-                "Local disk write failed copying '%s:%s' to '%s'. (FSWrite: %s)",
-                conn->this_server, source, dest, GetErrorStr());
-            if (conn)
-            {
-                conn->error = true;
-            }
+                "Local disk write failed copying '%s:%s' to '%s'",
+                conn->this_server, source, dest);
             free(buf);
             unlink(dest);
             close(dd);
-            FlushFileStream(conn->conn_info->sd, size - n_read_total);
-            EVP_CIPHER_CTX_cleanup(&crypto_ctx);
+            FlushFileStream(conn->conn_info->sd, size - n_wrote_total - n_read);
+            conn->error = true;
             return false;
         }
 
-        n_read_total += n_read;
+        n_wrote_total += n_read;
     }
 
-    /* If the file ends with a `hole', something needs to be written at
-       the end.  Otherwise the kernel would truncate the file at the end
-       of the last write operation. Write a null character and truncate
-       it again.  */
+    const bool do_sync = false;
 
-    if (ftruncate(dd, n_read_total) < 0)
+    bool ret = FileSparseClose(dd, dest, do_sync,
+                               n_wrote_total, last_write_made_hole);
+    if (!ret)
     {
-        Log(LOG_LEVEL_ERR, "Copy failed (no space?) while copying '%s' from network '%s'",
-            dest, GetErrorStr());
-        free(buf);
         unlink(dest);
-        close(dd);
-        FlushFileStream(conn->conn_info->sd, size - n_read_total);
+        free(buf);
+        FlushFileStream(conn->conn_info->sd, size - n_wrote_total);
         return false;
     }
 
-    close(dd);
     free(buf);
     return true;
 }
