@@ -1,7 +1,7 @@
 /*
-   Copyright (C) CFEngine AS
+   Copyright 2017 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by CFEngine AS.
+   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -106,7 +106,7 @@ AgentConnection *ConnCache_FindIdleMarkBusy(const char *server,
 
 
         if (strcmp(server,  svp->conn->this_server) == 0 &&
-            ConnectionFlagsEqual(&flags, &svp->conn->flags) == 0 &&
+            ConnectionFlagsEqual(&flags, &svp->conn->flags) &&
             (port == svp->conn->this_port
              ||
              (port != NULL && svp->conn->this_port != NULL &&
