@@ -1,7 +1,7 @@
 /*
-   Copyright (C) CFEngine AS
+   Copyright 2017 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by CFEngine AS.
+   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -460,7 +460,7 @@ X509 *TLSGenerateCertFromPrivKey(RSA *privkey)
     const EVP_MD *md = EVP_get_digestbyname("sha384");
     if (md == NULL)
     {
-        Log(LOG_LEVEL_ERR, "OpenSSL: Uknown digest algorithm %s",
+        Log(LOG_LEVEL_ERR, "OpenSSL: Unknown digest algorithm %s",
             "sha384");
         goto err3;
     }
