@@ -1,7 +1,7 @@
 /*
-   Copyright (C) CFEngine AS
+   Copyright 2017 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by CFEngine AS.
+   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -74,7 +74,6 @@ LineSelect GetInsertSelectConstraints(const EvalContext *ctx, const Promise *pp)
 Measurement GetMeasurementConstraint(const EvalContext *ctx, const Promise *pp);
 Packages GetPackageConstraints(const EvalContext *ctx, const Promise *pp);
 NewPackages GetNewPackageConstraints(const EvalContext *ctx, const Promise *pp);
-User GetUserConstraints(const EvalContext *ctx, const Promise *pp);
 ProcessCount GetMatchesConstraints(const EvalContext *ctx, const Promise *pp);
 ProcessSelect GetProcessFilterConstraints(const EvalContext *ctx, const Promise *pp);
 DirectoryRecursion GetRecursionConstraints(const EvalContext *ctx, const Promise *pp);
