@@ -1,7 +1,7 @@
 /*
-   Copyright (C) CFEngine AS
+   Copyright 2017 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by CFEngine AS.
+   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -27,69 +27,28 @@
 
 #include <cf3.defs.h>
 
-/**
- * @brief Sets sys.policy_hub and the POLICY_SERVER global.
- * @param ctx EvalContext is used to set related variables
- * @param new_policy_server IP of new policy server
- */
-void SetPolicyServer(EvalContext *ctx, const char *new_policy_server);
 
-/**
- * @brief Updates sys.last_policy_update variable from $(sys.masterdir)/cf_promises_validated
- * @param ctx EvalContext to put variable into
- */
-void UpdateLastPolicyUpdateTime(EvalContext *ctx);
+void EvalContextSetPolicyServer(EvalContext *ctx, const char *new_policy_server);
 
-/**
- * @return The contents of policy_server.dat, or NULL if file is not found. Return value must be freed.
- */
-char *ReadPolicyServerFile(const char *workdir);
-
-/**
- * @brief Write new_policy_server to the policy_server.dat file.
- * @return True if successful
- */
+// POLICY SERVER FILE FUNCTIONS:
+char* ReadPolicyServerFile(const char *workdir);
+void SetPolicyServerFromFile(EvalContext *ctx, const char *workdir);
+bool ParsePolicyServerFile(const char *workdir, char **host, char **port);
+bool LookUpPolicyServerFile(const char *workdir, char **ipaddr, char **port);
 bool WritePolicyServerFile(const char *workdir, const char *new_policy_server);
-
-/**
- * @brief Remove the policy_server.dat file
- * @return True if successful
- */
 bool RemovePolicyServerFile(const char *workdir);
 
-/**
- * @return True if the file STATEDIR/am_policy_hub exists
- */
+// POLICY HUB FUNCTIONS:
+void UpdateLastPolicyUpdateTime(EvalContext *ctx);
 bool GetAmPolicyHub(void);
-
-/**
- * @brief Set the STATEDIR/am_policy_hub marker file.
- * @param am_policy_hub If true, create marker file. If false, delete it.
- * @return True if successful
- */
 bool WriteAmPolicyHubFile(bool am_policy_hub);
 
-/**
- * @brief Write the builtin failsafe policy to the default location
- * @return True if successful
- */
+// FAILSAFE FUNCTIONS:
 bool WriteBuiltinFailsafePolicy(const char *workdir);
-
-/**
- * @brief Exposed for testing. Use WriteBuiltinFailsafePolicy.
- */
 bool WriteBuiltinFailsafePolicyToPath(const char *filename);
 
-/**
- * @brief Removes all files in $(sys.inputdir)
- * @param inputdir
- * @return True if successful
- */
+// POLICY FILE FUNCTIONS:
 bool RemoveAllExistingPolicyInInputs(const char *inputdir);
-
-/**
- * @return True if the file $(sys.masterdir)/promises.cf exists
- */
 bool MasterfileExists(const char *masterdir);
 
 #endif
