@@ -1,7 +1,7 @@
 /*
-   Copyright (C) CFEngine AS
+   Copyright 2017 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by CFEngine AS.
+   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -84,8 +84,8 @@ int NovaWin_NameToSid(char *name, SID *sid, DWORD sidSz);
 int NovaWin_SidToName(SID *sid, char *name, int nameSz);
 int NovaWin_StringToSid(char *stringSid, SID *sid, int sidSz);
 
-FnCallResult FnCallUserExists(EvalContext *ctx, FnCall *fp, Rlist *finalargs);
-FnCallResult FnCallGroupExists(EvalContext *ctx, FnCall *fp, Rlist *finalargs);
+FnCallResult FnCallUserExists(EvalContext *ctx, const Policy *policy, const FnCall *fp, const Rlist *finalargs);
+FnCallResult FnCallGroupExists(EvalContext *ctx, const Policy *policy, const FnCall *fp, const Rlist *finalargs);
 
 /* win_wmi.c */
 
