@@ -1,7 +1,7 @@
 /*
-   Copyright (C) CFEngine AS
+   Copyright 2017 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by CFEngine AS.
+   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -25,7 +25,6 @@
 #ifndef CFENGINE_CF3_DEFS_H
 #define CFENGINE_CF3_DEFS_H
 
-
 /* ALWAYS INCLUDE EITHER THIS FILE OR platform.h FIRST */
 
 
@@ -122,6 +121,12 @@
 
 #define CF_NS ':'   // namespace character separator
 
+/* Mangled namespace and scope characters, in order for the iteration engine
+ * to VariablePut() to THIS scope single elements of namespaced iterables
+ * (slists/containers). See expand.c and iteration.c. */
+#define CF_MANGLED_NS    '*'
+#define CF_MANGLED_SCOPE '#'
+
 /*****************************************************************************/
 
 /* Auditing key */
@@ -546,7 +551,6 @@ typedef enum
 #define CF_INTLISTRANGE  "[-0-9_$(){}\\[\\].]+"
 #define CF_REALRANGE "-9.99999E100,9.99999E100"
 #define CF_CHARRANGE "^.$"
-#define CF_NULL_VALUE "cf_null"
 
 #define CF_MODERANGE   "[0-7augorwxst,+-=]+"
 #define CF_BSDFLAGRANGE "[+-]*[(arch|archived|nodump|opaque|sappnd|sappend|schg|schange|simmutable|sunlnk|sunlink|uappnd|uappend|uchg|uchange|uimmutable|uunlnk|uunlink)]+"
@@ -1305,7 +1309,7 @@ typedef struct
     char *package_version_equal_command;
 
     int package_noverify_returncode;
-    
+
     bool has_package_method;
     bool is_empty;
 } Packages;
@@ -1323,10 +1327,10 @@ typedef struct
 
 typedef struct
 {
-    Rlist *control_package_inventory; /* list of all inventory used package managers 
+    Rlist *control_package_inventory; /* list of all inventory used package managers
                                        * names taken from common control */
     char *control_package_module;    /* policy default package manager name */
-    Seq *package_modules_bodies; /* list of all discovered in policy PackageManagerBody 
+    Seq *package_modules_bodies; /* list of all discovered in policy PackageManagerBody
                                    * bodies taken from common control */
 } PackagePromiseContext;
 
@@ -1339,7 +1343,7 @@ typedef struct
     char *package_version;
     char *package_architecture;
     Rlist *package_options;
-    
+
     bool is_empty;
 } NewPackages;
 
@@ -1407,28 +1411,19 @@ typedef struct
     char *description;
     char *group_primary;
     Rlist *groups_secondary;
+    bool   groups_secondary_given;
     char *home_dir;
     char *shell;
 } User;
 
 /*************************************************************************/
 
-typedef enum
-{
-    SERVICE_POLICY_START,
-    SERVICE_POLICY_STOP,
-    SERVICE_POLICY_DISABLE,
-    SERVICE_POLICY_RESTART,
-    SERVICE_POLICY_RELOAD,
-    SERVICE_POLICY_NONE
-} ServicePolicy;
-
 typedef struct
 {
     Rlist *service_depend;
     char *service_type;
     char *service_args;
-    ServicePolicy service_policy;
+    char *service_policy;
     char *service_autostart_policy;
     char *service_depend_chain;
     FnCall *service_method;
@@ -1583,13 +1578,12 @@ typedef struct
     Rlist *insert_match;
 } Attributes;
 
+
 /*************************************************************************/
 /* common macros                                                         */
 /*************************************************************************/
 
-#define NULL_OR_EMPTY(str) ((str == NULL) || (str[0] == '\0'))
-#define BEGINSWITH(str,start) (strncmp(str,start,strlen(start)) == 0)
-
+#include <rlist.h>
 #include <dbm_api.h>
 #include <sequence.h>
 #include <prototypes3.h>
@@ -1610,4 +1604,3 @@ extern const ConstraintSyntax CFEX_CONTROLBODY[];
 typedef struct ServerConnectionState_ ServerConnectionState;
 
 #endif
-
