@@ -1,7 +1,7 @@
 /*
-   Copyright (C) CFEngine AS
+   Copyright 2017 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by CFEngine AS.
+   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -58,7 +58,6 @@ extern int VEXPIREAFTER;
 extern const char *const OBS[CF_OBSERVABLES][2];
 
 extern bool FIPS_MODE;
-extern char POLICY_SERVER[CF_MAX_IP_LEN];
 extern HashMethod CF_DEFAULT_DIGEST;
 extern int CF_DEFAULT_DIGEST_LEN;
 
@@ -68,8 +67,7 @@ extern const char *const CF_AGENTTYPES[];
 
 extern int CFA_MAXTHREADS;
 extern AgentType THIS_AGENT_TYPE;
-extern int SHOWREPORTS;
-extern int LASTSEENEXPIREAFTER;
+extern long LASTSEENEXPIREAFTER;
 extern const char *DEFAULT_COPYTYPE;
 
 extern const char *const DAY_TEXT[];
