@@ -1,7 +1,7 @@
 /*
-   Copyright (C) CFEngine AS
+   Copyright 2017 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by CFEngine AS.
+   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -48,7 +48,6 @@ AclMethod AclMethodFromString(const char *string);
 AclType AclTypeFromString(const char *string);
 AclDefault AclDefaultFromString(const char *string);
 AclInherit AclInheritFromString(const char *string);
-ServicePolicy ServicePolicyFromString(const char *string);
 int SignalFromString(const char *s);
 int SyslogPriorityFromString(const char *s);
 ShellType ShellTypeFromString(const char *s);
@@ -73,6 +72,8 @@ const char *CommandArg0(const char *execstr);
 size_t CommandArg0_bound(char *dst, const char *src, size_t dst_size);
 void CommandPrefix(char *execstr, char *comm);
 const char *DataTypeShortToType(char *short_type);
+bool DataTypeIsIterable(DataType t);
+
 int CoarseLaterThan(const char *key, const char *from);
 int FindTypeInArray(const char *const haystack[], const char *needle, int default_value, int null_value);
 
