@@ -1,7 +1,7 @@
 /*
-   Copyright (C) CFEngine AS
+   Copyright 2017 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by CFEngine AS.
+   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -205,7 +205,7 @@ bool LoadSecretKeys(void)
         }
 
         PUBKEY = PEM_read_RSAPublicKey(fp, NULL, NULL, (void*) priv_passphrase);
-        if (NULL == PUBKEY)
+        if (PUBKEY == NULL)
         {
             Log(LOG_LEVEL_ERR,
                 "Error reading public key at '%s'. (PEM_read_RSAPublicKey: %s)",
@@ -220,7 +220,7 @@ bool LoadSecretKeys(void)
         fclose(fp);
     }
 
-    if (NULL != PUBKEY
+    if (PUBKEY != NULL
         && ((BN_num_bits(PUBKEY->e) < 2) || (!BN_is_odd(PUBKEY->e))))
     {
         Log(LOG_LEVEL_ERR, "The public key RSA exponent is too small or not odd");
@@ -232,7 +232,7 @@ bool LoadSecretKeys(void)
 
 void PolicyHubUpdateKeys(const char *policy_server)
 {
-    if (GetAmPolicyHub() && NULL != PUBKEY)
+    if (GetAmPolicyHub() && PUBKEY != NULL)
     {
         unsigned char digest[EVP_MAX_MD_SIZE + 1];
         const char* const workdir = GetWorkDir();
@@ -432,7 +432,8 @@ bool SavePublicKey(const char *user, const char *digest, const RSA *key)
     return true;
 }
 
-int EncryptString(char type, const char *in, char *out, unsigned char *key, int plainlen)
+int EncryptString(char *out, size_t out_size, const char *in, int plainlen,
+                  char type, unsigned char *key)
 {
     int cipherlen = 0, tmplen;
     unsigned char iv[32] =
@@ -442,6 +443,14 @@ int EncryptString(char type, const char *in, char *out, unsigned char *key, int
     if (key == NULL)
         ProgrammingError("EncryptString: session key == NULL");
 
+    size_t max_ciphertext_size = CipherTextSizeMax(CfengineCipher(type), plainlen);
+
+    if(max_ciphertext_size > out_size)
+    {
+        ProgrammingError("EncryptString: output buffer too small: max_ciphertext_size (%ld) > out_size (%ld)",
+                          max_ciphertext_size, out_size);
+    }
+
     EVP_CIPHER_CTX_init(&ctx);
     EVP_EncryptInit_ex(&ctx, CfengineCipher(type), NULL, key, iv);
 
@@ -458,13 +467,56 @@ int EncryptString(char type, const char *in, char *out, unsigned char *key, int
     }
 
     cipherlen += tmplen;
+
+    if(cipherlen > max_ciphertext_size)
+    {
+        ProgrammingError("EncryptString: too large ciphertext written: cipherlen (%d) > max_ciphertext_size (%ld)",
+                          cipherlen, max_ciphertext_size);
+    }
+
     EVP_CIPHER_CTX_cleanup(&ctx);
     return cipherlen;
 }
 
+size_t CipherBlockSizeBytes(const EVP_CIPHER *cipher)
+{
+    return EVP_CIPHER_block_size(cipher);
+}
+
+size_t CipherTextSizeMax(const EVP_CIPHER* cipher, size_t plaintext_size)
+{
+    // see man EVP_DecryptUpdate() and EVP_DecryptFinal_ex()
+    size_t padding_size = (CipherBlockSizeBytes(cipher) * 2) - 1;
+
+    // check for potential integer overflow, leave some buffer
+    if(plaintext_size > SIZE_MAX - padding_size)
+    {
+        ProgrammingError("CipherTextSizeMax: plaintext_size is too large (%zu)",
+                         plaintext_size);
+    }
+
+    return plaintext_size + padding_size;
+}
+
+size_t PlainTextSizeMax(const EVP_CIPHER* cipher, size_t ciphertext_size)
+{
+    // see man EVP_DecryptUpdate() and EVP_DecryptFinal_ex()
+    size_t padding_size = (CipherBlockSizeBytes(cipher) * 2);
+
+    // check for potential integer overflow, leave some buffer
+    if(ciphertext_size > SIZE_MAX - padding_size)
+    {
+        ProgrammingError("PlainTextSizeMax: ciphertext_size is too large (%zu)",
+                         ciphertext_size);
+    }
+
+    return ciphertext_size + padding_size;
+}
+
 /*********************************************************************/
 
-int DecryptString(char type, const char *in, char *out, unsigned char *key, int cipherlen)
+int DecryptString(char *out, size_t out_size, const char *in, int cipherlen,
+                  char type, unsigned char *key)
 {
     int plainlen = 0, tmplen;
     unsigned char iv[32] =
@@ -474,6 +526,14 @@ int DecryptString(char type, const char *in, char *out, unsigned char *key, int
     if (key == NULL)
         ProgrammingError("DecryptString: session key == NULL");
 
+    size_t max_plaintext_size = PlainTextSizeMax(CfengineCipher(type), cipherlen);
+
+    if(max_plaintext_size > out_size)
+    {
+        ProgrammingError("DecryptString: output buffer too small: max_plaintext_size (%ld) > out_size (%ld)",
+                          max_plaintext_size, out_size);
+    }
+
     EVP_CIPHER_CTX_init(&ctx);
     EVP_DecryptInit_ex(&ctx, CfengineCipher(type), NULL, key, iv);
 
@@ -495,6 +555,12 @@ int DecryptString(char type, const char *in, char *out, unsigned char *key, int
 
     plainlen += tmplen;
 
+    if(plainlen > max_plaintext_size)
+    {
+        ProgrammingError("DecryptString: too large plaintext written: plainlen (%d) > max_plaintext_size (%ld)",
+                          plainlen, max_plaintext_size);
+    }
+
     EVP_CIPHER_CTX_cleanup(&ctx);
 
     return plainlen;
