@@ -1,7 +1,7 @@
 /*
-   Copyright (C) CFEngine AS
+   Copyright 2017 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by CFEngine AS.
+   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -39,8 +39,13 @@ const char *CryptoLastErrorString(void);
 void DebugBinOut(char *buffer, int len, char *com);
 bool LoadSecretKeys(void);
 void PolicyHubUpdateKeys(const char *policy_server);
-int EncryptString(char type, const char *in, char *out, unsigned char *key, int len);
-int DecryptString(char type, const char *in, char *out, unsigned char *key, int len);
+int EncryptString(char *out, size_t out_size, const char *in, int plainlen,
+                  char type, unsigned char *key);
+size_t CipherBlockSizeBytes(const EVP_CIPHER *cipher);
+size_t CipherTextSizeMax(const EVP_CIPHER* cipher, size_t plaintext_size);
+size_t PlainTextSizeMax(const EVP_CIPHER* cipher, size_t ciphertext_size);
+int DecryptString(char *out, size_t out_size, const char *in, int cipherlen,
+                  char type, unsigned char *key);
 RSA *HavePublicKey(const char *username, const char *ipaddress, const char *digest);
 RSA *HavePublicKeyByIP(const char *username, const char *ipaddress);
 bool SavePublicKey(const char *username, const char *digest, const RSA *key);
