@@ -1,7 +1,7 @@
 /*
-   Copyright (C) CFEngine AS
+   Copyright 2017 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by CFEngine AS.
+   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -43,7 +43,7 @@ struct DBHandle_
 {
     /* Filename of database file */
     char *filename;
-    
+
     /* Name of specific sub-db */
     char *subname;
 
@@ -148,7 +148,7 @@ char *DBIdToSubPath(dbid id, const char *subdb_name)
         ProgrammingError("Unable to construct sub database filename for file"
                 "%s_%s", DB_PATHS_STATEDIR[id], subdb_name);
     }
-    
+
     char *native_filename = MapNameCopy(filename);
     free(filename);
 
@@ -189,15 +189,21 @@ char *DBIdToPath(dbid id)
 static
 bool IsSubHandle(DBHandle *handle, dbid id, const char *name)
 {
-    return StringSafeEqual(handle->filename, DBIdToSubPath(id, name));
+    char *sub_path = DBIdToSubPath(id, name);
+    bool result = StringSafeEqual(handle->filename, sub_path);
+    free(sub_path);
+    return result;
 }
 
 static DBHandle *DBHandleGetSubDB(dbid id, const char *name)
 {
-    ThreadLock(&db_handles_lock);
-    
+    if (!ThreadLock(&db_handles_lock))
+    {
+        return NULL;
+    }
+
     DynamicDBHandles *handles_list = db_dynamic_handles;
-    
+
     while (handles_list)
     {
         if (IsSubHandle(handles_list->handle, id, name))
@@ -207,26 +213,26 @@ static DBHandle *DBHandleGetSubDB(dbid id, const char *name)
         }
         handles_list = handles_list->next;
     }
-    
+
     DBHandle *handle = xcalloc(1, sizeof(DBHandle));
     handle->filename = DBIdToSubPath(id, name);
     handle->subname = SafeStringDuplicate(name);
-    
+
     /* Initialize mutexes as error-checking ones. */
     pthread_mutexattr_t attr;
     pthread_mutexattr_init(&attr);
     pthread_mutexattr_settype(&attr, PTHREAD_MUTEX_ERRORCHECK);
     pthread_mutex_init(&handle->lock, &attr);
     pthread_mutexattr_destroy(&attr);
-    
+
     /* Prepend handle to global list. */
     handles_list = xcalloc(1, sizeof(DynamicDBHandles));
     handles_list->handle = handle;
     handles_list->next = db_dynamic_handles;
     db_dynamic_handles = handles_list;
-    
+
     ThreadUnlock(&db_handles_lock);
-    
+
     return handle;
 }
 
@@ -234,23 +240,28 @@ static DBHandle *DBHandleGet(int id)
 {
     assert(id >= 0 && id < dbid_max);
 
-    ThreadLock(&db_handles_lock);
-
-    if (db_handles[id].filename == NULL)
+    if (ThreadLock(&db_handles_lock))
     {
-        db_handles[id].filename = DBIdToPath(id);
+        if (db_handles[id].filename == NULL)
+        {
+            db_handles[id].filename = DBIdToPath(id);
 
-        /* Initialize mutexes as error-checking ones. */
-        pthread_mutexattr_t attr;
-        pthread_mutexattr_init(&attr);
-        pthread_mutexattr_settype(&attr, PTHREAD_MUTEX_ERRORCHECK);
-        pthread_mutex_init(&db_handles[id].lock, &attr);
-        pthread_mutexattr_destroy(&attr);
+            /* Initialize mutexes as error-checking ones. */
+            pthread_mutexattr_t attr;
+            pthread_mutexattr_init(&attr);
+            pthread_mutexattr_settype(&attr, PTHREAD_MUTEX_ERRORCHECK);
+            pthread_mutex_init(&db_handles[id].lock, &attr);
+            pthread_mutexattr_destroy(&attr);
+        }
+
+        ThreadUnlock(&db_handles_lock);
+
+        return &db_handles[id];
+    }
+    else
+    {
+        return NULL;
     }
-
-    ThreadUnlock(&db_handles_lock);
-
-    return &db_handles[id];
 }
 
 static inline
@@ -311,7 +322,7 @@ void CloseAllDBExit()
             CloseDBInstance(&db_handles[i]);
         }
     }
-    
+
     DynamicDBHandles *db_dynamic_handles_list = db_dynamic_handles;
     while (db_dynamic_handles_list)
     {
@@ -320,6 +331,7 @@ void CloseAllDBExit()
         db_dynamic_handles_list = db_dynamic_handles_list->next;
         free(handle);
     }
+    free(db_dynamic_handles);
     db_dynamic_handles = NULL;
 }
 
@@ -344,59 +356,63 @@ void DBSetMaximumConcurrentTransactions(int max_txn)
 static inline
 bool OpenDBInstance(DBHandle **dbp, dbid id, DBHandle *handle)
 {
-    ThreadLock(&handle->lock);
-
-    if (handle->refcount == 0)
+    if (ThreadLock(&handle->lock))
     {
-        int lock_fd = DBPathLock(handle->filename);
-
-        if(lock_fd != -1)
+        if (handle->refcount == 0)
         {
-            handle->priv = DBPrivOpenDB(handle->filename, id);
+            int lock_fd = DBPathLock(handle->filename);
 
-            if (handle->priv == DB_PRIV_DATABASE_BROKEN)
+            if(lock_fd != -1)
             {
-                DBPathMoveBroken(handle->filename);
                 handle->priv = DBPrivOpenDB(handle->filename, id);
+
                 if (handle->priv == DB_PRIV_DATABASE_BROKEN)
                 {
+                    DBPathMoveBroken(handle->filename);
+                    handle->priv = DBPrivOpenDB(handle->filename, id);
+                    if (handle->priv == DB_PRIV_DATABASE_BROKEN)
+                    {
+                        handle->priv = NULL;
+                    }
+                }
+
+                DBPathUnLock(lock_fd);
+            }
+
+            if (handle->priv)
+            {
+                if (!DBMigrate(handle, id))
+                {
+                    DBPrivCloseDB(handle->priv);
                     handle->priv = NULL;
                 }
             }
-
-            DBPathUnLock(lock_fd);
         }
 
         if (handle->priv)
         {
-            if (!DBMigrate(handle, id))
-            {
-                DBPrivCloseDB(handle->priv);
-                handle->priv = NULL;
-            }
+            handle->refcount++;
+            *dbp = handle;
+
+            /* Only register shutdown handler if any database was opened
+             * correctly. Otherwise this shutdown caller may be called too early,
+             * and shutdown handler installed by the database library may end up
+             * being called before CloseAllDB function */
+
+            pthread_once(&db_shutdown_once, RegisterShutdownHandler);
+        }
+        else
+        {
+            *dbp = NULL;
         }
-    }
 
-    if (handle->priv)
-    {
-        handle->refcount++;
-        *dbp = handle;
-
-        /* Only register shutdown handler if any database was opened
-         * correctly. Otherwise this shutdown caller may be called too early,
-         * and shutdown handler installed by the database library may end up
-         * being called before CloseAllDB function */
-
-        pthread_once(&db_shutdown_once, RegisterShutdownHandler);
+        ThreadUnlock(&handle->lock);
+        return *dbp != NULL;
     }
     else
     {
-        *dbp = NULL;
+        return false;
     }
-
-    ThreadUnlock(&handle->lock);
-
-    return *dbp != NULL;
 }
 
 bool OpenSubDB(DBHandle **dbp, dbid id, const char *sub_name)
@@ -413,30 +429,40 @@ bool OpenDB(DBHandle **dbp, dbid id)
 
 void CloseDB(DBHandle *handle)
 {
-    ThreadLock(&handle->lock);
-
-    DBPrivCommit(handle->priv);
-
-    if (handle->refcount < 1)
+    /* Skip in case of nested locking, for example signal handler.
+     * DB behaviour becomes erratic otherwise (CFE-1996). */
+    if (ThreadLock(&handle->lock))
     {
-        Log(LOG_LEVEL_ERR, "Trying to close database %s which is not open", handle->filename);
-    }
-    else if (--handle->refcount == 0)
-    {
-        DBPrivCloseDB(handle->priv);
-    }
+        DBPrivCommit(handle->priv);
 
-    ThreadUnlock(&handle->lock);
+        if (handle->refcount < 1)
+        {
+            Log(LOG_LEVEL_ERR,
+                "Trying to close database which is not open: %s",
+                handle->filename);
+        }
+        else
+        {
+            handle->refcount--;
+            if (handle->refcount == 0)
+            {
+                DBPrivCloseDB(handle->priv);
+            }
+        }
+
+        ThreadUnlock(&handle->lock);
+    }
 }
 
 bool CleanDB(DBHandle *handle)
 {
-    ThreadLock(&handle->lock);
-    
-    bool ret = DBPrivClean(handle->priv);
-    
-    ThreadUnlock(&handle->lock);
-    
+    bool ret = false;
+    if (ThreadLock(&handle->lock))
+    {
+        ret = DBPrivClean(handle->priv);
+
+        ThreadUnlock(&handle->lock);
+    }
     return ret;
 }
 
@@ -620,4 +646,3 @@ StringMap *LoadDatabaseToStringMap(dbid database_id)
 
     return db_map;
 }
-
