@@ -1,7 +1,7 @@
 /*
-   Copyright (C) CFEngine AS
+   Copyright 2017 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by CFEngine AS.
+   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -29,7 +29,7 @@
 #include <syntax.h>
 #include <item_lib.h>
 #include <ornaments.h>
-#include <expand.h>
+#include <expand.h>                                    /* ExpandPrivateRval */
 #include <matching.h>
 #include <string_lib.h>
 #include <misc_lib.h>
@@ -49,6 +49,7 @@
 #include <printsize.h>
 #include <regex.h>
 #include <map.h>
+#include <conversion.h>                               /* DataTypeIsIterable */
 
 
 /**
@@ -190,11 +191,11 @@ void AddPackageModuleToContext(const EvalContext *ctx, PackageModuleBody *pm)
 PackageModuleBody *GetPackageModuleFromContext(const EvalContext *ctx,
         const char *name)
 {
-    if (name == NULL || StringSafeEqual("cf_null", name))
+    if (name == NULL)
     {
         return NULL;
     }
-    
+
     for (int i = 0;
          i < SeqLength(ctx->package_promise_context->package_modules_bodies);
          i++)
@@ -487,12 +488,12 @@ bool IsDefinedClass(const EvalContext *ctx, const char *context)
         return true;
     }
 
-    if (NULL == context_expression_whitespace_rx)
+    if (context_expression_whitespace_rx == NULL)
     {
         context_expression_whitespace_rx = CompileRegex(CFENGINE_REGEX_WHITESPACE_IN_CONTEXTS);
     }
 
-    if (NULL == context_expression_whitespace_rx)
+    if (context_expression_whitespace_rx == NULL)
     {
         Log(LOG_LEVEL_ERR, "The context expression whitespace regular expression could not be compiled, aborting.");
         return false;
@@ -896,6 +897,7 @@ void FreePackagePromiseContext(PackagePromiseContext *pp_ctx)
 {
     SeqDestroy(pp_ctx->package_modules_bodies);
     RlistDestroy(pp_ctx->control_package_inventory);
+    free(pp_ctx->control_package_module);
     free(pp_ctx);
 }
 
@@ -1116,7 +1118,8 @@ JsonElement *EvalContextGetPromiseCallers(EvalContext *ctx) {
             JsonObjectAppendString(f, "promise_type", frame->data.promise.owner->parent_promise_type->name);
             JsonObjectAppendString(f, "promiser", frame->data.promise.owner->promiser);
             JsonObjectAppendString(f, "promise_classes", frame->data.promise.owner->classes);
-            JsonObjectAppendString(f, "promise_comment", NULL == frame->data.promise.owner->comment ? "" : frame->data.promise.owner->comment);
+            JsonObjectAppendString(f, "promise_comment",
+                                   (frame->data.promise.owner->comment == NULL) ? "" : frame->data.promise.owner->comment);
             break;
 
         case STACK_FRAME_TYPE_PROMISE_TYPE:
@@ -1206,13 +1209,12 @@ static StackFrame *StackFrameNewPromise(const Promise *owner)
     return frame;
 }
 
-static StackFrame *StackFrameNewPromiseIteration(Promise *owner, const PromiseIterator *iter_ctx, unsigned index)
+static StackFrame *StackFrameNewPromiseIteration(Promise *owner, const PromiseIterator *iter_ctx)
 {
     StackFrame *frame = StackFrameNew(STACK_FRAME_TYPE_PROMISE_ITERATION, true);
 
     frame->data.promise_iteration.owner = owner;
     frame->data.promise_iteration.iter_ctx = iter_ctx;
-    frame->data.promise_iteration.index = index;
     frame->data.promise_iteration.log_messages = RingBufferNew(5, NULL, free);
 
     return frame;
@@ -1241,6 +1243,8 @@ static void EvalContextStackPushFrame(EvalContext *ctx, StackFrame *frame)
 
     assert(!frame->path);
     frame->path = EvalContextStackPath(ctx);
+
+    LogDebug(LOG_MOD_EVALCTX, "PUSHED FRAME (type %d)", frame->type);
 }
 
 void EvalContextStackPushBundleFrame(EvalContext *ctx, const Bundle *owner, const Rlist *args, bool inherits_previous)
@@ -1379,24 +1383,24 @@ void EvalContextStackPushPromiseFrame(EvalContext *ctx, const Promise *owner, bo
     EvalContextVariablePutSpecial(ctx, SPECIAL_SCOPE_THIS, "namespace", PromiseGetNamespace(owner), CF_DATA_TYPE_STRING, "source=promise");
 }
 
-Promise *EvalContextStackPushPromiseIterationFrame(EvalContext *ctx, size_t iteration_index, const PromiseIterator *iter_ctx)
+Promise *EvalContextStackPushPromiseIterationFrame(EvalContext *ctx, const PromiseIterator *iter_ctx)
 {
-    assert(LastStackFrame(ctx, 0) && LastStackFrame(ctx, 0)->type == STACK_FRAME_TYPE_PROMISE);
+    const StackFrame *last_frame = LastStackFrame(ctx, 0);
 
-    if (iter_ctx)
-    {
-        PromiseIteratorUpdateVariable(ctx, iter_ctx);
-    }
+    assert(last_frame       != NULL);
+    assert(last_frame->type == STACK_FRAME_TYPE_PROMISE);
 
-    bool excluded = false;
-    Promise *pexp = ExpandDeRefPromise(ctx, LastStackFrame(ctx, 0)->data.promise.owner, &excluded);
+    /* Evaluate all constraints by calling functions etc. */
+    bool excluded;
+    Promise *pexp = ExpandDeRefPromise(ctx, last_frame->data.promise.owner,
+                                       &excluded);
     if (excluded || !pexp)
     {
         PromiseDestroy(pexp);
         return NULL;
     }
 
-    EvalContextStackPushFrame(ctx, StackFrameNewPromiseIteration(pexp, iter_ctx, iteration_index));
+    EvalContextStackPushFrame(ctx, StackFrameNewPromiseIteration(pexp, iter_ctx));
 
     LoggingPrivSetLevels(CalculateLogLevel(pexp), CalculateReportLevel(pexp));
 
@@ -1441,6 +1445,8 @@ void EvalContextStackPopFrame(EvalContext *ctx)
             LoggingPrivSetLevels(CalculateLogLevel(pp), CalculateReportLevel(pp));
         }
     }
+
+    LogDebug(LOG_MOD_EVALCTX, "POPPED FRAME (type %d)", last_frame_type);
 }
 
 bool EvalContextClassRemove(EvalContext *ctx, const char *ns, const char *name)
@@ -1504,9 +1510,9 @@ static bool EvalContextClassPut(EvalContext *ctx, const char *ns, const char *na
                 "is equal or longer than %zu", name, sizeof(canonified_context));
             return false;
         }
-        
+
         strlcpy(canonified_context, name, sizeof(canonified_context));
-        
+
         if (Chop(canonified_context, CF_EXPANDSIZE) == -1)
         {
             Log(LOG_LEVEL_ERR, "Chop was called on a string that seemed to have no terminator");
@@ -1664,6 +1670,73 @@ const RingBuffer *EvalContextStackCurrentMessages(const EvalContext *ctx)
     return frame ? frame->data.promise_iteration.log_messages : NULL;
 }
 
+
+
+/**
+ * @brief Concatenate string #str to #buf, replacing mangling
+ *        characters '*' and '#' with their visible counterparts.
+ */
+static void BufferAppendPromiseStr(Buffer *buf, const char *str)
+{
+    for (const char *ch = str; *ch != '\0'; ch++)
+    {
+        switch (*ch)
+        {
+        case CF_MANGLED_NS:
+            BufferAppendChar(buf, ':');
+            break;
+
+        case CF_MANGLED_SCOPE:
+            BufferAppendChar(buf, '.');
+            break;
+
+        default:
+            BufferAppendChar(buf, *ch);
+            break;
+        }
+    }
+}
+
+/**
+ * @brief Like @c BufferAppendPromiseStr, but if @c str contains newlines
+ *   and is longer than 2*N+3, then only copy an abbreviated version
+ *   consisting of the first and last N characters, separated by @c `...`
+ *
+ * @param buffer Buffer to be used.
+ * @param promiser Constant string to append
+ * @param N      Max. length of initial/final segment of @c promiser to keep
+ * @note 2*N+3 is the maximum length of the appended string (excl. terminating NULL)
+ *
+ */
+static void BufferAppendAbbreviatedStr(Buffer *buf,
+                                       const char *promiser, const int N)
+{
+    /* check if `promiser` contains a new line (may happen for "insert_lines") */
+    const char *const nl = strchr(promiser, '\n');
+    if (NULL == nl)
+    {
+        BufferAppendPromiseStr(buf, promiser);
+    }
+    else
+    {
+        /* `promiser` contains a newline: abbreviate it by taking the first and last few characters */
+        static const char sep[] = "...";
+        char abbr[sizeof(sep) + 2 * N];
+        const int head = (nl > promiser + N) ? N : (nl - promiser);
+        const char * last_line = strrchr(promiser, '\n') + 1;
+        assert(last_line); /* not NULL, we know we have at least one '\n' */
+        const int tail = strlen(last_line);
+        if (tail > N)
+        {
+            last_line += tail - N;
+        }
+        memcpy(abbr, promiser, head);
+        strcpy(abbr + head, sep);
+        strcat(abbr, last_line);
+        BufferAppendPromiseStr(buf, abbr);
+    }
+}
+
 char *EvalContextStackPath(const EvalContext *ctx)
 {
     Buffer *path = BufferNew();
@@ -1697,9 +1770,13 @@ char *EvalContextStackPath(const EvalContext *ctx)
             BufferAppendChar(path, '\'');
             BufferAppendAbbreviatedStr(path, frame->data.promise_iteration.owner->promiser, CF_MAXFRAGMENT);
             BufferAppendChar(path, '\'');
-            if (i == SeqLength(ctx->stack) - 1)
+            if (i == SeqLength(ctx->stack) - 1  &&
+                /* For some reason verify_packages.c is adding NULL iteration
+                 * frames all over the place; TODO fix. */
+                frame->data.promise_iteration.iter_ctx != NULL)
             {
-                BufferAppendF(path, "[%zd]", frame->data.promise_iteration.index);
+                BufferAppendF(path, "[%zu]",
+                              PromiseIteratorIndex(frame->data.promise_iteration.iter_ctx));
             }
             break;
         }
@@ -1943,11 +2020,10 @@ bool EvalContextVariablePut(EvalContext *ctx,
     assert(type != CF_DATA_TYPE_NONE);
     assert(ref);
     assert(ref->lval);
-    assert(value);
-    if (!value)
-    {
-        return false;
-    }
+
+    /* The only possible way to get a NULL value is if it's an empty linked
+     * list (Rlist usually). */
+    assert(value != NULL || DataTypeIsIterable(type));
 
     if (strlen(ref->lval) > CF_MAXVARSIZE)
     {
@@ -2003,7 +2079,7 @@ static Variable *VariableResolve(const EvalContext *ctx, const VarRef *ref)
         {
             return var;
         }
-        else if (ref->num_indices > 0)
+        else if (ref->num_indices > 0)                 /* why? TODO comment */
         {
             VarRef *base_ref = VarRefCopyIndexless(ref);
             var = VariableTableGet(table, base_ref);
@@ -2019,14 +2095,23 @@ static Variable *VariableResolve(const EvalContext *ctx, const VarRef *ref)
     return NULL;
 }
 
+/**
+ *
+ * @NOTE NULL is a valid return value if #type_out is of list type and the
+ *       list is empty. To check if the variable didn't resolve, check if
+ *       #type_out was set to CF_DATA_TYPE_NONE.
+ */
 const void *EvalContextVariableGet(const EvalContext *ctx, const VarRef *ref, DataType *type_out)
 {
     Variable *var = VariableResolve(ctx, ref);
     if (var)
     {
-        if (var->ref->num_indices == 0 && ref->num_indices > 0 && var->type == CF_DATA_TYPE_CONTAINER)
+        if (var->ref->num_indices == 0    &&
+                 ref->num_indices > 0     &&
+            var->type == CF_DATA_TYPE_CONTAINER)
         {
-            JsonElement *child = JsonSelect(RvalContainerValue(var->rval), ref->num_indices, ref->indices);
+            JsonElement *child = JsonSelect(RvalContainerValue(var->rval),
+                                            ref->num_indices, ref->indices);
             if (child)
             {
                 if (type_out)
@@ -2048,12 +2133,13 @@ const void *EvalContextVariableGet(const EvalContext *ctx, const VarRef *ref, Da
     else if (!VarRefIsQualified(ref))
     {
         /*
-         * FALLBACK: Because VariableResolve currently does not walk the stack (rather, it looks at
-         * only the top frame), we do an explicit retry here to qualify an unqualified reference to
-         * the current bundle.
+         * FALLBACK: Because VariableResolve currently does not walk the stack
+         * (rather, it looks at only the top frame), we do an explicit retry
+         * here to qualify an unqualified reference to the current bundle.
          *
-         * This is overly complicated, and will go away as soon as VariableResolve can walk the stack
-         * (which is pending rework in variable iteration).
+         * This is overly complicated, and will go away as soon as
+         * VariableResolve can walk the stack (which is pending rework in
+         * variable iteration).
          */
         const Bundle *bp = EvalContextStackCurrentBundle(ctx);
         if (bp)
@@ -2223,7 +2309,7 @@ void EvalContextAppendBodyParentsAndArgs(const EvalContext *ctx, const Policy *p
 
             // We don't do a more detailed check for circular
             // inheritance because the depth check above will catch it
-            if (0 == strcmp(parent_ref.name, bp->name))
+            if (strcmp(parent_ref.name, bp->name) == 0)
             {
                 Log(LOG_LEVEL_ERR, "EvalContextAppendBodyParentsAndArgs: self body inheritance in %s->%s, aborting", bp->name, parent_ref.name);
                 exit(EXIT_FAILURE);
@@ -2350,12 +2436,6 @@ static bool IsPromiseValuableForLogging(const Promise *pp)
 static void AddAllClasses(EvalContext *ctx, const Rlist *list, unsigned int persistence_ttl,
                           PersistentClassPolicy policy, ContextScope context_scope)
 {
-
-    if (list)
-    {
-        Log(LOG_LEVEL_VERBOSE, "\n");
-    }
-
     for (const Rlist *rp = list; rp != NULL; rp = rp->next)
     {
         char *classname = xstrdup(RlistScalarValue(rp));
@@ -2405,12 +2485,6 @@ static void AddAllClasses(EvalContext *ctx, const Rlist *list, unsigned int pers
         }
         free(classname);
     }
-
-    if (list)
-    {
-        Log(LOG_LEVEL_VERBOSE, "\n");
-    }
-
 }
 
 static void DeleteAllClasses(EvalContext *ctx, const Rlist *list)
@@ -2707,7 +2781,8 @@ bool GetChecksumUpdatesDefault(const EvalContext *ctx)
 
 void EvalContextAddIpAddress(EvalContext *ctx, const char *ip_address, const char *iface)
 {
-    AppendItem(&ctx->ip_addresses, ip_address, (NULL == iface ? "" : iface));
+    AppendItem(&ctx->ip_addresses, ip_address,
+               (iface == NULL) ? "" : iface);
 }
 
 void EvalContextDeleteIpAddresses(EvalContext *ctx)
