@@ -1,7 +1,7 @@
 /*
-   Copyright (C) CFEngine AS
+   Copyright 2017 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by CFEngine AS.
+   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -234,7 +234,7 @@ char *SkipHashType(char *hash)
 {
     char *str = hash;
 
-    if(BEGINSWITH(hash, "MD5=") || BEGINSWITH(hash, "SHA="))
+    if(STARTSWITH(hash, "MD5=") || STARTSWITH(hash, "SHA="))
     {
         str = hash + 4;
     }
