@@ -1,7 +1,7 @@
 /*
-   Copyright (C) CFEngine AS
+   Copyright 2017 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by CFEngine AS.
+   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -30,10 +30,4 @@
 
 int cf_lstat(const char *file, struct stat *buf, FileCopy fc, AgentConnection *conn);
 
-/**
- * @brief Works exactly like posix 'getline', EXCEPT it does not include carriage return at the end.
- * @return -1 on error OR EOF, so check. Or bytes in buff without null terminator.
- */
-ssize_t CfReadLine(char **buff, size_t *size, FILE *fp);
-
 #endif
