@@ -1,7 +1,7 @@
 /*
-   Copyright (C) CFEngine AS
+   Copyright 2017 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by CFEngine AS.
+   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -149,27 +149,24 @@ Returns true if so, false otherwise.
 
 /*********************************************************************/
 
-/**
- * @TODO fix the dangerous path lengths
- */
-char *JoinSuffix(char *path, const char *leaf)
+char *JoinSuffix(char *path, size_t path_size, const char *leaf)
 {
     int len = strlen(leaf);
 
-    if (Chop(path, CF_EXPANDSIZE) == -1)
+    if (Chop(path, path_size) == -1)
     {
         Log(LOG_LEVEL_ERR, "Chop was called on a string that seemed to have no terminator");
     }
     DeleteSlash(path);
 
-    if ((strlen(path) + len) > (CF_BUFSIZE - CF_BUFFERMARGIN))
+    if (strlen(path) + len + 1 > path_size)
     {
-        Log(LOG_LEVEL_ERR, "Internal limit 2: Buffer ran out of space constructing string. Tried to add %s to %s",
+        Log(LOG_LEVEL_ERR, "JoinSuffix: Internal limit reached. Tried to add %s to %s",
               leaf, path);
         return NULL;
     }
 
-    strcat(path, leaf);
+    strlcat(path, leaf, path_size);
     return path;
 }
 
