@@ -1,7 +1,7 @@
 /*
-   Copyright (C) CFEngine AS
+   Copyright 2017 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by CFEngine AS.
+   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -57,8 +57,14 @@ leads to Hash Association (lval,rval) => (user,"$(person)")
 
 /******************************************************************/
 
-static Rlist *NewExpArgs(EvalContext *ctx, const Policy *policy, const FnCall *fp)
+Rlist *NewExpArgs(EvalContext *ctx, const Policy *policy, const FnCall *fp, const FnCallType *fp_type)
 {
+    // Functions with delayed evaluation will call this themselves later
+    if (fp_type && fp_type->options & FNCALL_OPTION_DELAYED_EVALUATION)
+    {
+        return RlistCopy(fp->args);
+    }
+
     const FnCallType *fn = FnCallTypeGet(fp->name);
     {
         int len = RlistLen(fp->args);
@@ -85,10 +91,7 @@ static Rlist *NewExpArgs(EvalContext *ctx, const Policy *policy, const FnCall *f
         case RVAL_TYPE_FNCALL:
             {
                 FnCall *subfp = RlistFnCallValue(rp);
-
                 rval = FnCallEvaluate(ctx, policy, subfp, fp->caller).rval;
-
-                assert(rval.item);
             }
             break;
         default:
@@ -332,7 +335,7 @@ FnCallResult FnCallEvaluate(EvalContext *ctx, const Policy *policy, FnCall *fp,
         return (FnCallResult) { FNCALL_FAILURE, { FnCallCopy(fp), RVAL_TYPE_FNCALL } };
     }
 
-    Rlist *expargs = NewExpArgs(ctx, policy, fp);
+    Rlist *expargs = NewExpArgs(ctx, policy, fp, fp_type);
 
     Writer *fncall_writer = NULL;
     const char *fncall_string = "";
@@ -343,13 +346,15 @@ FnCallResult FnCallEvaluate(EvalContext *ctx, const Policy *policy, FnCall *fp,
         fncall_string = StringWriterData(fncall_writer);
     }
 
-    if (RlistIsUnresolved(expargs))
+    // Check if arguments are resolved, except for delayed evaluation functions
+    if ( ! (fp_type->options & FNCALL_OPTION_DELAYED_EVALUATION) &&
+         RlistIsUnresolved(expargs))
     {
         // Special case: ifelse(isvariable("x"), $(x), "default")
         // (the first argument will come down expanded as "!any")
-        if (0 == strcmp(fp->name, "ifelse") &&
+        if (strcmp(fp->name, "ifelse") == 0 &&
             RlistLen(expargs) == 3 &&
-            0 == strcmp("!any", RlistScalarValueSafe(expargs)) &&
+            strcmp("!any", RlistScalarValueSafe(expargs)) == 0 &&
             !RlistIsUnresolved(expargs->next->next))
         {
                 Log(LOG_LEVEL_DEBUG, "Allowing ifelse() function evaluation even"
@@ -402,13 +407,6 @@ FnCallResult FnCallEvaluate(EvalContext *ctx, const Policy *policy, FnCall *fp,
         RlistDestroy(expargs);
         return (FnCallResult) { FNCALL_FAILURE, { FnCallCopy(fp), RVAL_TYPE_FNCALL } };
     }
-    else if (result.rval.type == RVAL_TYPE_LIST && !result.rval.item)
-    {
-        Rlist *seq = NULL;
-        // don't pass NULL items to evaluator
-        RlistPrepend(&seq, CF_NULL_VALUE, RVAL_TYPE_SCALAR);
-        result.rval.item = seq;
-    }
 
     if (fp_type->options & FNCALL_OPTION_CACHED)
     {
