@@ -1,7 +1,7 @@
 /*
-   Copyright (C) CFEngine AS
+   Copyright 2017 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by CFEngine AS.
+   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -26,6 +26,7 @@
 #include <generic_agent.h>
 
 #include <bootstrap.h>
+#include <policy_server.h>
 #include <sysinfo.h>
 #include <known_dirs.h>
 #include <eval_context.h>
@@ -63,8 +64,8 @@
 #include <ornaments.h>
 #include <cf-windows-functions.h>
 #include <loading.h>
-#include <iteration.h>
 #include <signals.h>
+#include <addr_lib.h>
 
 static pthread_once_t pid_cleanup_once = PTHREAD_ONCE_INIT; /* GLOBAL_T */
 
@@ -100,8 +101,8 @@ static JsonElement *ReadJsonFile(const char *filename, LogLevel log_level)
     // 5 MB should be enough for most reasonable def.json data
     JsonParseError err = JsonParseFile(filename, 5 * 1024 * 1024, &doc);
 
-    if (err != JSON_PARSE_OK
-        || NULL == doc)
+    if (err != JSON_PARSE_OK ||
+        doc == NULL)
     {
         Log(log_level, "Could not parse JSON file %s: %s", filename, JsonParseErrorToString(err));
     }
@@ -143,7 +144,7 @@ void MarkAsPolicyServer(EvalContext *ctx)
 Policy *SelectAndLoadPolicy(GenericAgentConfig *config, EvalContext *ctx, bool validate_policy, bool write_validated_file)
 {
     Policy *policy = NULL;
-    
+
     if (GenericAgentCheckPolicy(config, validate_policy, write_validated_file))
     {
         policy = LoadPolicy(ctx, config);
@@ -157,7 +158,7 @@ Policy *SelectAndLoadPolicy(GenericAgentConfig *config, EvalContext *ctx, bool v
     {
         Log(LOG_LEVEL_ERR, "CFEngine was not able to get confirmation of promises from cf-promises, so going to failsafe");
         EvalContextClassPutHard(ctx, "failsafe_fallback", "attribute_name=Errors,source=agent");
-        
+
         if (CheckAndGenerateFailsafe(GetInputDir(), "failsafe.cf"))
         {
             GenericAgentConfigSetInputFile(config, GetInputDir(), "failsafe.cf");
@@ -211,12 +212,12 @@ void LoadAugmentsData(EvalContext *ctx, const Buffer* filename_buffer, const Jso
         const char *key;
         while ((key = JsonIteratorNextKey(&iter)))
         {
-            if (0 == strcmp("vars", key))
+            if (strcmp("vars", key) == 0)
             {
                 // load variables
                 JsonElement* vars = JsonExpandElement(ctx, JsonObjectGet(augment, key));
 
-                if (NULL == vars ||
+                if (vars == NULL ||
                     JsonGetElementType(vars) != JSON_ELEMENT_TYPE_CONTAINER ||
                     JsonGetContainerType(vars) != JSON_CONTAINER_TYPE_OBJECT)
                 {
@@ -245,7 +246,7 @@ void LoadAugmentsData(EvalContext *ctx, const Buffer* filename_buffer, const Jso
                         Log(LOG_LEVEL_VERBOSE, "Installing augments slist variable '%s.%s' from file '%s'",
                             SpecialScopeToString(SPECIAL_SCOPE_DEF), vkey, BufferData(filename_buffer));
                         EvalContextVariablePutSpecial(ctx, SPECIAL_SCOPE_DEF,
-                                                      vkey, ContainerToRlist(data),
+                                                      vkey, RlistFromContainer(data),
                                                       CF_DATA_TYPE_STRING_LIST,
                                                       "source=augments_file");
                     }
@@ -263,7 +264,7 @@ void LoadAugmentsData(EvalContext *ctx, const Buffer* filename_buffer, const Jso
               vars_cleanup:
                 JsonDestroy(vars);
             }
-            else if (0 == strcmp("classes", key))
+            else if (strcmp("classes", key) == 0)
             {
                 // load classes
                 JsonElement* classes = JsonExpandElement(ctx, JsonObjectGet(augment, key));
@@ -325,7 +326,7 @@ void LoadAugmentsData(EvalContext *ctx, const Buffer* filename_buffer, const Jso
               classes_cleanup:
                 JsonDestroy(classes);
             }
-            else if (0 == strcmp("inputs", key))
+            else if (strcmp("inputs", key) == 0)
             {
                 // load inputs
                 JsonElement* inputs = JsonExpandElement(ctx, JsonObjectGet(augment, key));
@@ -336,7 +337,7 @@ void LoadAugmentsData(EvalContext *ctx, const Buffer* filename_buffer, const Jso
                 {
                     Log(LOG_LEVEL_VERBOSE, "Installing augments def.augments_inputs from file '%s'",
                         BufferData(filename_buffer));
-                    Rlist *rlist = ContainerToRlist(inputs);
+                    Rlist *rlist = RlistFromContainer(inputs);
                     EvalContextVariablePutSpecial(ctx, SPECIAL_SCOPE_DEF,
                                                   "augments_inputs", rlist,
                                                   CF_DATA_TYPE_STRING_LIST,
@@ -377,7 +378,7 @@ void LoadAugmentsFiles(EvalContext *ctx, const char* filename)
         if (FileCanOpen(BufferData(expbuf), "r"))
         {
             JsonElement* augment = ReadJsonFile(BufferData(expbuf), LOG_LEVEL_ERR);
-            if (NULL != augment )
+            if (augment != NULL)
             {
                 LoadAugmentsData(ctx, expbuf, augment);
                 JsonDestroy(augment);
@@ -436,7 +437,9 @@ void GenericAgentDiscoverContext(EvalContext *ctx, GenericAgentConfig *config)
     LoadAugments(ctx, config);
 
     const char *bootstrap_arg =
-        config->agent_specific.agent.bootstrap_policy_server;
+        config->agent_specific.agent.bootstrap_argument;
+    const char *bootstrap_ip =
+        config->agent_specific.agent.bootstrap_ip;
 
     /* Are we bootstrapping the agent? */
     if (config->agent_type == AGENT_TYPE_AGENT && bootstrap_arg != NULL)
@@ -458,8 +461,8 @@ void GenericAgentDiscoverContext(EvalContext *ctx, GenericAgentConfig *config)
         }
         GenericAgentConfigSetInputFile(config, GetInputDir(), "failsafe.cf");
 
-        char canonified_ipaddr[strlen(bootstrap_arg) + 1];
-        StringCanonify(canonified_ipaddr, bootstrap_arg);
+        char canonified_ipaddr[strlen(bootstrap_ip) + 1];
+        StringCanonify(canonified_ipaddr, bootstrap_ip);
 
         bool am_policy_server =
             EvalContextClassGet(ctx, NULL, canonified_ipaddr) != NULL;
@@ -496,7 +499,7 @@ void GenericAgentDiscoverContext(EvalContext *ctx, GenericAgentConfig *config)
         WriteAmPolicyHubFile(am_policy_server);
 
         WritePolicyServerFile(GetWorkDir(), bootstrap_arg);
-        SetPolicyServer(ctx, bootstrap_arg);
+        EvalContextSetPolicyServer(ctx, bootstrap_arg);
 
         /* FIXME: Why it is called here? Can't we move both invocations to before if? */
         UpdateLastPolicyUpdateTime(ctx);
@@ -508,7 +511,7 @@ void GenericAgentDiscoverContext(EvalContext *ctx, GenericAgentConfig *config)
         {
             Log(LOG_LEVEL_VERBOSE, "This agent is bootstrapped to: %s",
                 existing_policy_server);
-            SetPolicyServer(ctx, existing_policy_server);
+            EvalContextSetPolicyServer(ctx, existing_policy_server);
             free(existing_policy_server);
             UpdateLastPolicyUpdateTime(ctx);
         }
@@ -577,7 +580,7 @@ bool GenericAgentCheckPolicy(GenericAgentConfig *config, bool force_validation,
                                                 GetAmPolicyHub()); // write release ID?
             }
 
-            if (config->agent_specific.agent.bootstrap_policy_server && !policy_check_ok)
+            if (config->agent_specific.agent.bootstrap_argument && !policy_check_ok)
             {
                 Log(LOG_LEVEL_VERBOSE, "Policy is not valid, but proceeding with bootstrap");
                 return true;
@@ -604,7 +607,7 @@ static JsonElement *ReadPolicyValidatedFile(const char *filename)
     }
 
     JsonElement *validated_doc = ReadJsonFile(filename, LOG_LEVEL_DEBUG);
-    if (NULL == validated_doc)
+    if (validated_doc == NULL)
     {
         Log(missing ? LOG_LEVEL_DEBUG : LOG_LEVEL_VERBOSE, "Could not parse policy_validated JSON file '%s', using dummy data", filename);
         validated_doc = JsonObjectCreate(2);
@@ -671,7 +674,7 @@ static bool WritePolicyValidatedFile(ARG_UNUSED const GenericAgentConfig *config
 bool GenericAgentTagReleaseDirectory(const GenericAgentConfig *config, const char *dirname, bool write_validated, bool write_release)
 {
     char local_dirname[PATH_MAX + 1];
-    if (NULL == dirname)
+    if (dirname == NULL)
     {
         GetAutotagDir(local_dirname, PATH_MAX, NULL);
         dirname = local_dirname;
@@ -691,10 +694,11 @@ bool GenericAgentTagReleaseDirectory(const GenericAgentConfig *config, const cha
         // first, tag the release ID
         GetReleaseIdFile(dirname, filename, sizeof(filename));
         char *id = ReadReleaseIdFromReleaseIdFileMasterfiles(dirname);
-        if (NULL == id
-            || (have_git_checksum && 0 != strcmp(id, git_checksum)))
+        if (id == NULL
+            || (have_git_checksum &&
+                strcmp(id, git_checksum) != 0))
         {
-            if (NULL == id)
+            if (id == NULL)
             {
                 Log(LOG_LEVEL_DEBUG, "The release_id of %s was missing", dirname);
             }
@@ -851,7 +855,7 @@ bool GenericAgentArePromisesValid(const GenericAgentConfig *config)
 #if !defined(__MINGW32__)
 static void OpenLog(int facility)
 {
-    openlog(VPREFIX, LOG_PID | LOG_NOWAIT | LOG_ODELAY, facility);
+    openlog(NULL, LOG_PID | LOG_NOWAIT | LOG_ODELAY, facility);
 }
 #endif
 
@@ -880,11 +884,22 @@ void GenericAgentInitialize(EvalContext *ctx, GenericAgentConfig *config)
     InitializeWindows();
 #endif
 
+    /* Set output to line-buffered to avoid truncated debug logs. */
+
     /* Bug on HP-UX: Buffered output is discarded if you switch buffering mode
        without flushing the buffered output first. This will happen anyway when
        switching modes, so no performance is lost. */
     fflush(stdout);
+
+#ifndef SUNOS_5
     setlinebuf(stdout);
+#else
+    /* CFE-2527: On Solaris we avoid calling setlinebuf, since fprintf() on
+       Solaris 10 and 11 truncates output under certain conditions. We fully
+       disable buffering to avoid truncated debug logs; performance impact
+       should be minimal because we mostly write full lines anyway. */
+    setvbuf(stdout, NULL, _IONBF, 0);
+#endif
 
     DetermineCfenginePort();
 
@@ -908,7 +923,7 @@ void GenericAgentInitialize(EvalContext *ctx, GenericAgentConfig *config)
 
     snprintf(vbuff, CF_BUFSIZE, "%s%cupdate.conf", GetInputDir(), FILE_SEPARATOR);
     MakeParentDirectory(vbuff, force);
-    snprintf(vbuff, CF_BUFSIZE, "%s%cbin%ccf-agent -D from_cfexecd", workdir, FILE_SEPARATOR, FILE_SEPARATOR);
+    snprintf(vbuff, CF_BUFSIZE, "%s%cbin%ccf-agent", workdir, FILE_SEPARATOR, FILE_SEPARATOR);
     MakeParentDirectory(vbuff, force);
     snprintf(vbuff, CF_BUFSIZE, "%s%coutputs%cspooled_reports", workdir, FILE_SEPARATOR, FILE_SEPARATOR);
     MakeParentDirectory(vbuff, force);
@@ -1001,9 +1016,11 @@ void GenericAgentInitialize(EvalContext *ctx, GenericAgentConfig *config)
     if (config->agent_type != AGENT_TYPE_KEYGEN)
     {
         LoadSecretKeys();
-        char *bootstrapped_policy_server = ReadPolicyServerFile(workdir);
-        PolicyHubUpdateKeys(bootstrapped_policy_server);
-        free(bootstrapped_policy_server);
+        char *ipaddr = NULL, *port = NULL;
+        LookUpPolicyServerFile(workdir, &ipaddr, &port);
+        PolicyHubUpdateKeys(ipaddr);
+        free(ipaddr);
+        free(port);
     }
 
     size_t cwd_size = PATH_MAX;
@@ -1184,7 +1201,7 @@ static void GetPromisesValidatedFile(char *filename, size_t max_size, const Gene
     /* TODO overflow error checking! */
     GetAutotagDir(dirname, max_size, maybe_dirname);
 
-    if (NULL == maybe_dirname && MINUSF)
+    if (maybe_dirname == NULL && MINUSF)
     {
         snprintf(filename, max_size, "%s/validated_%s", dirname, CanonifyName(config->original_input_file));
     }
@@ -1201,7 +1218,7 @@ static void GetPromisesValidatedFile(char *filename, size_t max_size, const Gene
  */
 static void GetAutotagDir(char *dirname, size_t max_size, const char *maybe_dirname)
 {
-    if (NULL != maybe_dirname)
+    if (maybe_dirname != NULL)
     {
         strlcpy(dirname, maybe_dirname, max_size);
     }
@@ -1230,11 +1247,11 @@ static JsonElement *ReadReleaseIdFileFromMasterfiles(const char *maybe_dirname)
 {
     char filename[CF_MAXVARSIZE];
 
-    GetReleaseIdFile(NULL == maybe_dirname ? GetMasterDir() : maybe_dirname,
+    GetReleaseIdFile((maybe_dirname == NULL) ? GetMasterDir() : maybe_dirname,
                      filename, sizeof(filename));
 
     JsonElement *doc = ReadJsonFile(filename, LOG_LEVEL_DEBUG);
-    if (NULL == doc)
+    if (doc == NULL)
     {
         Log(LOG_LEVEL_VERBOSE, "Could not parse release_id JSON file %s", filename);
     }
@@ -1587,39 +1604,6 @@ const char *GenericAgentResolveInputPath(const GenericAgentConfig *config, const
     return MapName(input_path);
 }
 
-void GenericAgentWriteHelp(Writer *w, const char *component, const struct option options[], const char *const hints[], bool accepts_file_argument)
-{
-    WriterWriteF(w, "Usage: %s [OPTION]...%s\n", component, accepts_file_argument ? " [FILE]" : "");
-
-    WriterWriteF(w, "\nOptions:\n");
-
-    for (int i = 0; options[i].name != NULL; i++)
-    {
-        char short_option[] = ", -*";
-        if (options[i].val < 128)
-        {
-            // Within ASCII range, means there is a short option.
-            short_option[3] = options[i].val;
-        }
-        else
-        {
-            // No short option.
-            short_option[0] = '\0';
-        }
-        if (options[i].has_arg)
-        {
-            WriterWriteF(w, "  --%-12s%s value - %s\n", options[i].name, short_option, hints[i]);
-        }
-        else
-        {
-            WriterWriteF(w, "  --%-12s%-10s - %s\n", options[i].name, short_option, hints[i]);
-        }
-    }
-
-    WriterWriteF(w, "\nWebsite: http://www.cfengine.com\n");
-    WriterWriteF(w, "This software is Copyright (C) 2008,2010-present CFEngine AS.\n");
-}
-
 ENTERPRISE_VOID_FUNC_1ARG_DEFINE_STUB(void, GenericAgentWriteVersion, Writer *, w)
 {
     WriterWriteF(w, "%s\n", NameVersion());
@@ -1784,7 +1768,7 @@ GenericAgentConfig *GenericAgentConfigNewDefault(AgentType agent_type, bool tty_
     config->tty_interactive = tty_interactive;
 
     const char *color_env = getenv("CFENGINE_COLOR");
-    config->color = (color_env && 0 == strcmp(color_env, "1"));
+    config->color = (color_env && strcmp(color_env, "1") == 0);
 
     config->bundlesequence = NULL;
 
@@ -1803,7 +1787,10 @@ GenericAgentConfig *GenericAgentConfigNewDefault(AgentType agent_type, bool tty_
 
     config->protocol_version = CF_PROTOCOL_UNDEFINED;
 
-    config->agent_specific.agent.bootstrap_policy_server = NULL;
+    config->agent_specific.agent.bootstrap_argument = NULL;
+    config->agent_specific.agent.bootstrap_ip = NULL;
+    config->agent_specific.agent.bootstrap_port = NULL;
+    config->agent_specific.agent.bootstrap_host = NULL;
 
     /* By default we trust the network when bootstrapping. */
     config->agent_specific.agent.bootstrap_trust_server = true;
@@ -1840,6 +1827,10 @@ void GenericAgentConfigDestroy(GenericAgentConfig *config)
         free(config->original_input_file);
         free(config->input_file);
         free(config->input_dir);
+        free(config->agent_specific.agent.bootstrap_argument);
+        free(config->agent_specific.agent.bootstrap_host);
+        free(config->agent_specific.agent.bootstrap_ip);
+        free(config->agent_specific.agent.bootstrap_port);
         free(config);
     }
 }
@@ -1906,20 +1897,20 @@ void GenericAgentConfigApply(EvalContext *ctx, const GenericAgentConfig *config)
 bool CheckAndGenerateFailsafe(const char *inputdir, const char *input_file)
 {
     char failsafe_path[CF_BUFSIZE];
-    
+
     if (strlen(inputdir) + strlen(input_file) > sizeof(failsafe_path) - 2)
     {
         Log(LOG_LEVEL_ERR,
             "Unable to generate path for %s/%s file. Path too long.",
             inputdir, input_file);
-        /* We could create dynamically allocated buffer able to hold the 
+        /* We could create dynamically allocated buffer able to hold the
            whole content of the path but this should be unlikely that we
            will end up here. */
         return false;
     }
     snprintf(failsafe_path, CF_BUFSIZE - 1, "%s/%s", inputdir, input_file);
     MapName(failsafe_path);
-    
+
     if (access(failsafe_path, R_OK) != 0)
     {
         return WriteBuiltinFailsafePolicyToPath(failsafe_path);
