@@ -1,7 +1,7 @@
 /*
-   Copyright (C) CFEngine AS
+   Copyright 2017 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by CFEngine AS.
+   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -642,11 +642,16 @@ char *ItemList2CSV(const Item *list)
 {
     /* After each entry, we need space for either a ',' (before the
      * next entry) or a final '\0'. */
-    int len = ItemListSize(list) + ListLen(list);
-    char *s = xmalloc(len);
+    int s_size = ItemListSize(list) + ListLen(list);
+    if (s_size == 0)
+    {
+        s_size = 1;
+    }
+
+    char *s = xmalloc(s_size);
     *s = '\0';
 
-    /* No point cycle-checking; done while computing len. */
+    /* No point cycle-checking; done while computing s_size. */
     for (const Item *ip = list; ip != NULL; ip = ip->next)
     {
         if (ip->name)
@@ -659,7 +664,8 @@ char *ItemList2CSV(const Item *list)
             strcat(s, ",");
         }
     }
-    assert(strlen(s) + 1 == len);
+
+    assert(strlen(s) + 1 == s_size);
 
     return s;
 }
@@ -1074,3 +1080,19 @@ Item *RawLoadItemList(const char *filename)
 
     return ReverseItemList(list);
 }
+
+bool IsInterfaceAddress(const Item *ip_addresses, const char *adr)
+ /* Does this address belong to a local interface */
+{
+    for (const Item *ip = ip_addresses; ip != NULL; ip = ip->next)
+    {
+        if (strncasecmp(adr, ip->name, strlen(adr)) == 0)
+        {
+            Log(LOG_LEVEL_DEBUG, "Identifying '%s' as one of my interfaces", adr);
+            return true;
+        }
+    }
+
+    Log(LOG_LEVEL_DEBUG, "'%s' is not one of my interfaces", adr);
+    return false;
+}
