@@ -1,7 +1,7 @@
 /*
-   Copyright (C) CFEngine AS
+   Copyright 2017 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by CFEngine AS.
+   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -25,22 +25,21 @@
 #ifndef CFENGINE_ITERATION_H
 #define CFENGINE_ITERATION_H
 
+
 #include <cf3.defs.h>
 
-#include <rlist.h>
-#include <variable.h>
 
 typedef struct PromiseIterator_ PromiseIterator;
 
-Rlist *ContainerToRlist(const JsonElement *container);
-PromiseIterator *PromiseIteratorNew(EvalContext *ctx, const Promise *pp, const Rlist *lists, const Rlist *containers);
-void PromiseIteratorDestroy(PromiseIterator *iter_ctx);
 
-bool PromiseIteratorHasMore(const PromiseIterator *iter_ctx);
-bool PromiseIteratorNext(PromiseIterator *iter_ctx);
+PromiseIterator *PromiseIteratorNew(const Promise *pp);
+void PromiseIteratorDestroy(PromiseIterator *iterctx);
+void PromiseIteratorPrepare(PromiseIterator *iterctx,
+                            const EvalContext *evalctx,
+                            char *s);
+bool PromiseIteratorNext(PromiseIterator *iterctx,
+                         EvalContext *evalctx);
+size_t PromiseIteratorIndex(const PromiseIterator *iter_ctx);
 
-bool NullIterators(const PromiseIterator *iter_ctx);
-
-void PromiseIteratorUpdateVariable(EvalContext *ctx, const PromiseIterator *iter_ctx);
 
 #endif
