@@ -1,7 +1,7 @@
 /*
-   Copyright (C) CFEngine AS
+   Copyright 2017 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by CFEngine AS.
+   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -43,8 +43,8 @@ bool Address2Hostkey(char *dst, size_t dst_size, const char *address);
 void LastSaw1(const char *ipaddress, const char *hashstr, LastSeenRole role);
 void LastSaw(const char *ipaddress, const char *digest, LastSeenRole role);
 
-bool DeleteIpFromLastSeen(const char *ip, char *digest);
-bool DeleteDigestFromLastSeen(const char *key, char *ip);
+bool DeleteIpFromLastSeen(const char *ip, char *digest, size_t digest_size);
+bool DeleteDigestFromLastSeen(const char *key, char *ip, size_t ip_size);
 
 /*
  * Return false in order to stop iteration
@@ -57,6 +57,6 @@ bool ScanLastSeenQuality(LastSeenQualityCallback callback, void *ctx);
 int LastSeenHostKeyCount(void);
 bool IsLastSeenCoherent(void);
 int RemoveKeysFromLastSeen(const char *input, bool must_be_coherent,
-                           char *equivalent);
+                           char *equivalent, size_t equivalent_size);
 
 #endif
