@@ -1,7 +1,7 @@
 /*
-   Copyright (C) CFEngine AS
+   Copyright 2017 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by CFEngine AS.
+   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -37,8 +37,8 @@
 #include <ornaments.h>
 
 // TODO: remove
-#include <vars.h>
-#include <audit.h>
+#include <vars.h>                                         /* IsCf3VarString */
+#include <audit.h>                                        /* FatalError */
 
 
 static Policy *LoadPolicyFile(EvalContext *ctx, GenericAgentConfig *config, const char *policy_file,
@@ -51,7 +51,7 @@ static Policy *LoadPolicyFile(EvalContext *ctx, GenericAgentConfig *config, cons
  * The difference between filename and input_input file is that the latter is the file specified by -f or
  * equivalently the file containing body common control. This will hopefully be squashed in later refactoring.
  */
-static Policy *Cf3ParseFile(const GenericAgentConfig *config, const char *input_path)
+Policy *Cf3ParseFile(const GenericAgentConfig *config, const char *input_path)
 {
     struct stat statbuf;
 
@@ -84,7 +84,7 @@ static Policy *Cf3ParseFile(const GenericAgentConfig *config, const char *input_
     }
 #endif
 
-    Log(LOG_LEVEL_VERBOSE, "Begin Parsing file '%s'", input_path);
+    Log(LOG_LEVEL_VERBOSE, "BEGIN parsing file: %s", input_path);
 
     if (!FileCanOpen(input_path, "r"))
     {
@@ -127,7 +127,7 @@ static Policy *Cf3ParseFile(const GenericAgentConfig *config, const char *input_
         }
     }
 
-    Log(LOG_LEVEL_VERBOSE, "End Parsing file '%s'", input_path);
+    Log(LOG_LEVEL_VERBOSE, "END   parsing file: %s", input_path);
     return policy;
 }
 
@@ -144,12 +144,6 @@ static Policy *LoadPolicyInputFiles(EvalContext *ctx, GenericAgentConfig *config
         }
 
         const char *unresolved_input = RlistScalarValue(rp);
-
-        if (strcmp(CF_NULL_VALUE, unresolved_input) == 0)
-        {
-            continue;
-        }
-
         if (IsExpandable(unresolved_input))
         {
             PolicyResolve(ctx, policy, config);
@@ -387,11 +381,6 @@ static bool VerifyBundleSequence(EvalContext *ctx, const Policy *policy, const G
             continue;
         }
 
-        if (strcmp(name, CF_NULL_VALUE) == 0)
-        {
-            continue;
-        }
-
         if (!config->ignore_missing_bundles && !PolicyGetBundle(policy, NULL, NULL, name))
         {
             Log(LOG_LEVEL_ERR, "Bundle '%s' listed in the bundlesequence is not a defined bundle", name);
@@ -504,7 +493,14 @@ Policy *LoadPolicy(EvalContext *ctx, GenericAgentConfig *config)
                 for (size_t ppi = 0; ppi < SeqLength(sp->promises); ppi++)
                 {
                     Promise *pp = SeqAt(sp->promises, ppi);
-                    ExpandPromise(ctx, pp, CommonEvalPromise, NULL);
+
+                    /* Skip constraint syntax verification through all
+                     * slist/container iterations for cf-promises! cf-agent
+                     * still checks though. */
+                    if (config->agent_type != AGENT_TYPE_COMMON)
+                    {
+                        ExpandPromise(ctx, pp, CommonEvalPromise, NULL);
+                    }
                 }
 
                 EvalContextStackPopFrame(ctx);
