@@ -1,7 +1,7 @@
 /*
-   Copyright (C) CFEngine AS
+   Copyright 2017 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by CFEngine AS.
+   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -29,5 +29,6 @@
 #include <generic_agent.h>
 
 Policy *LoadPolicy(EvalContext *ctx, GenericAgentConfig *config);
+Policy *Cf3ParseFile(const GenericAgentConfig *config, const char *input_path);
 
 #endif
