@@ -1,7 +1,7 @@
 /*
-   Copyright (C) CFEngine AS
+   Copyright 2017 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by CFEngine AS.
+   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -95,19 +95,19 @@ static inline void log_lock(const char *op,
     {
         if (lock_data)
         {
-            Log(LOG_LEVEL_DEBUG, "%s lock operation in '%s()'. "
-                "lock_id = '%s', lock_checksum = '%s', "
-                "lock.pid = '%d', lock.time = '%d', "
-                "lock.process_start_time = '%d'",
-                op, function, lock, lock_sum,
-                (int)lock_data->pid, (int)lock_data->time,
-                (int)lock_data->process_start_time);
+            LogDebug(LOG_MOD_LOCKS, "%s lock operation in '%s()': "
+                     "lock_id = '%s', lock_checksum = '%s', "
+                     "lock.pid = '%d', lock.time = '%d', "
+                     "lock.process_start_time = '%d'",
+                     op, function, lock, lock_sum,
+                     (int)lock_data->pid, (int)lock_data->time,
+                     (int)lock_data->process_start_time);
         }
         else
         {
-            Log(LOG_LEVEL_DEBUG, "%s lock operation in '%s()'. "
-                "lock_id = '%s', lock_checksum = '%s'",
-                op, function, lock, lock_sum);
+            LogDebug(LOG_MOD_LOCKS, "%s lock operation in '%s()'. "
+                     "lock_id = '%s', lock_checksum = '%s'",
+                     op, function, lock, lock_sum);
         }
     }
 }
@@ -115,7 +115,7 @@ static inline void log_lock(const char *op,
 static void GenerateMd5Hash(const char *istring, char *ohash)
 {
     if (!strcmp(istring, "CF_CRITICAL_SECTION"))
-    { 
+    {
         strcpy(ohash, istring);
         return;
     }
@@ -410,62 +410,63 @@ static void RegisterLockCleanup(void)
     RegisterAtExitFunction(&LocksCleanup);
 }
 
-static char *BodyName(const Promise *pp)
+/**
+ * Return a type template for the promise for lock-type
+ * identification. WARNING: instead of truncation, it does not include any
+ * parts (i.e. constraints or promise type) that don't fit.
+ *
+ * @WARNING currently it only prints up to the 5 first constraints (WHY?)
+ */
+static void PromiseTypeString(char *dst, size_t dst_size, const Promise *pp)
 {
-    char *name, *sp;
-    int size = 0;
+    char *sp       = pp->parent_promise_type->name;
+    size_t sp_len  = strlen(sp);
 
-/* Return a type template for the promise body for lock-type identification */
+    dst[0]         = '\0';
+    size_t dst_len = 0;
 
-    name = xmalloc(CF_MAXVARSIZE);
-
-    sp = pp->parent_promise_type->name;
-
-    if (size + strlen(sp) < CF_MAXVARSIZE - CF_BUFFERMARGIN)
+    if (sp_len + 1 < dst_size)
     {
-        strcpy(name, sp);
-        strcat(name, ".");
-        size += strlen(sp);
+        strcpy(dst, sp);
+        strcat(dst, ".");
+        dst_len += sp_len + 1;
     }
 
-    if (pp->conlist)
+    if (pp->conlist != NULL)
     {
-        for (size_t i = 0; (i < 5) && i < SeqLength(pp->conlist); i++)
+        /* Number of constraints (attributes) of that promise. */
+        size_t cons_num = SeqLength(pp->conlist);
+        for (size_t i = 0;  (i < 5) && (i < cons_num);  i++)
         {
-            Constraint *cp = SeqAt(pp->conlist, i);
+            Constraint *cp  = SeqAt(pp->conlist, i);
+            const char *con = cp->lval;                    /* the constraint */
 
-            if (strcmp(cp->lval, "args") == 0)      /* Exception for args, by symmetry, for locking */
+            /* Exception for args (promise type commands),
+               by symmetry, for locking. */
+            if (strcmp(con, "args") == 0)
             {
                 continue;
             }
 
-            if (strcmp(cp->lval, "arglist") == 0)      /* Exception for arglist, by symmetry, for locking */
+            /* Exception for arglist (promise type commands),
+               by symmetry, for locking. */
+            if (strcmp(con, "arglist") == 0)
             {
                 continue;
             }
 
-            if (size + strlen(cp->lval) < CF_MAXVARSIZE - CF_BUFFERMARGIN)
+            size_t con_len = strlen(con);
+            if (dst_len + con_len + 1 < dst_size)
             {
-                strcat(name, cp->lval);
-                strcat(name, ".");
-                size += strlen(cp->lval);
+                strcat(dst, con);
+                strcat(dst, ".");
+                dst_len += con_len + 1;
             }
         }
     }
-
-    return name;
 }
 
-#ifdef __MINGW32__
 
-static bool KillLockHolder(ARG_UNUSED const char *lock)
-{
-    Log(LOG_LEVEL_VERBOSE,
-          "Process is not running - ignoring lock (Windows does not support graceful processes termination)");
-    return true;
-}
-
-#else
 
 static bool KillLockHolder(const char *lock)
 {
@@ -504,8 +505,6 @@ static bool KillLockHolder(const char *lock)
     return GracefulTerminate(lock_data.pid, lock_data.process_start_time);
 }
 
-#endif
-
 void PromiseRuntimeHash(const Promise *pp, const char *salt, unsigned char digest[EVP_MAX_MD_SIZE + 1], HashMethod type)
 {
     static const char PACK_UPIFELAPSED_SALT[] = "packageuplist";
@@ -656,15 +655,17 @@ CfLock AcquireLock(EvalContext *ctx, const char *operand, const char *host, time
         return CfLockNull();
     }
 
-    unsigned char digest[EVP_MAX_MD_SIZE + 1];
-    PromiseRuntimeHash(pp, operand, digest, CF_DEFAULT_DIGEST);
     char str_digest[CF_HOSTKEY_STRING_SIZE];
-    HashPrintSafe(str_digest, sizeof(str_digest), digest,
-                  CF_DEFAULT_DIGEST, true);
+    {
+        unsigned char digest[EVP_MAX_MD_SIZE + 1];
+        PromiseRuntimeHash(pp, operand, digest, CF_DEFAULT_DIGEST);
+        HashPrintSafe(str_digest, sizeof(str_digest), digest,
+                      CF_DEFAULT_DIGEST, true);
+    }
 
     if (EvalContextPromiseLockCacheContains(ctx, str_digest))
     {
-        Log(LOG_LEVEL_DEBUG, "This promise has already been verified");
+//        Log(LOG_LEVEL_DEBUG, "This promise has already been verified");
         return CfLockNull();
     }
 
@@ -676,17 +677,22 @@ CfLock AcquireLock(EvalContext *ctx, const char *operand, const char *host, time
         return CfLockNew(NULL, "dummy", true);
     }
 
-    char *promise = BodyName(pp);
-    char cc_operator[CF_BUFSIZE], cc_operand[CF_BUFSIZE];
-    snprintf(cc_operator, CF_MAXVARSIZE - 1, "%s-%s", promise, host);
+    char cc_operator[CF_MAXVARSIZE];
+    {
+        char promise[CF_MAXVARSIZE - CF_BUFFERMARGIN];
+        PromiseTypeString(promise, sizeof(promise), pp);
+        snprintf(cc_operator, sizeof(cc_operator), "%s-%s", promise, host);
+    }
+
+    char cc_operand[CF_BUFSIZE];
     strlcpy(cc_operand, operand, CF_BUFSIZE);
     CanonifyNameInPlace(cc_operand);
     RemoveDates(cc_operand);
 
-    free(promise);
 
-    Log(LOG_LEVEL_DEBUG, "AcquireLock(%s,%s), ExpireAfter = %d, IfElapsed = %d", cc_operator, cc_operand, tc.expireafter,
-        tc.ifelapsed);
+    Log(LOG_LEVEL_DEBUG,
+        "AcquireLock(%s,%s), ExpireAfter = %d, IfElapsed = %d",
+        cc_operator, cc_operand, tc.expireafter, tc.ifelapsed);
 
     int sum = 0;
     for (int i = 0; cc_operator[i] != '\0'; i++)
@@ -699,13 +705,18 @@ CfLock AcquireLock(EvalContext *ctx, const char *operand, const char *host, time
         sum = (CF_MACROALPHABET * sum + cc_operand[i]) % CF_HASHTABLESIZE;
     }
 
+    const char *bundle_name = PromiseGetBundle(pp)->name;
+
     char cflock[CF_BUFSIZE] = "";
-    snprintf(cflock, CF_BUFSIZE, "lock.%.100s.%s.%.100s_%d_%s", PromiseGetBundle(pp)->name, cc_operator, cc_operand, sum, str_digest);
+    snprintf(cflock, CF_BUFSIZE, "lock.%.100s.%s.%.100s_%d_%s",
+             bundle_name, cc_operator, cc_operand, sum, str_digest);
 
     char cflast[CF_BUFSIZE] = "";
-    snprintf(cflast, CF_BUFSIZE, "last.%.100s.%s.%.100s_%d_%s", PromiseGetBundle(pp)->name, cc_operator, cc_operand, sum, str_digest);
+    snprintf(cflast, CF_BUFSIZE, "last.%.100s.%s.%.100s_%d_%s",
+             bundle_name, cc_operator, cc_operand, sum, str_digest);
 
-    Log(LOG_LEVEL_DEBUG, "Locking bundle '%s' with lock '%s'", PromiseGetBundle(pp)->name, cflock);
+    Log(LOG_LEVEL_DEBUG, "Locking bundle '%s' with lock '%s'",
+        bundle_name, cflock);
 
     // Now see if we can get exclusivity to edit the locks
     WaitForCriticalSection(CF_CRITIAL_SECTION);
@@ -720,7 +731,8 @@ CfLock AcquireLock(EvalContext *ctx, const char *operand, const char *host, time
     {
         if (elapsedtime < 0)
         {
-            Log(LOG_LEVEL_VERBOSE, "XX Another cf-agent seems to have done this since I started (elapsed=%jd)",
+            Log(LOG_LEVEL_VERBOSE,
+                "XX Another cf-agent seems to have done this since I started (elapsed=%jd)",
                 (intmax_t) elapsedtime);
             ReleaseCriticalSection(CF_CRITIAL_SECTION);
             return CfLockNull();
@@ -728,8 +740,9 @@ CfLock AcquireLock(EvalContext *ctx, const char *operand, const char *host, time
 
         if (elapsedtime < tc.ifelapsed)
         {
-            Log(LOG_LEVEL_VERBOSE, "XX Nothing promised here [%.40s] (%jd/%u minutes elapsed)", cflast,
-                (intmax_t) elapsedtime, tc.ifelapsed);
+            Log(LOG_LEVEL_VERBOSE,
+                "XX Nothing promised here [%.40s] (%jd/%u minutes elapsed)",
+                cflast, (intmax_t) elapsedtime, tc.ifelapsed);
             ReleaseCriticalSection(CF_CRITIAL_SECTION);
             return CfLockNull();
         }
@@ -745,14 +758,16 @@ CfLock AcquireLock(EvalContext *ctx, const char *operand, const char *host, time
         {
             if (elapsedtime >= tc.expireafter)
             {
-                Log(LOG_LEVEL_INFO, "Lock %s expired (after %jd/%u minutes)",
-                    cflock, (intmax_t) elapsedtime, tc.expireafter);
+                Log(LOG_LEVEL_INFO, "Lock expired after %jd/%u minutes: %s",
+                    (intmax_t) elapsedtime, tc.expireafter, cflock);
 
                 pid_t pid = FindLockPid(cflock);
 
                 if (KillLockHolder(cflock))
                 {
-                    Log(LOG_LEVEL_INFO, "Lock expired, process with PID %jd killed", (intmax_t)pid);
+                    Log(LOG_LEVEL_INFO,
+                        "Lock expired, process with PID %jd killed",
+                        (intmax_t) pid);
                     unlink(cflock);
                 }
                 else
@@ -910,23 +925,6 @@ void GetLockName(char *lockname, const char *locktype, const char *base, const R
     }
 }
 
-/**
- * Returns pointer to the first byte in #buf that is not #c. Returns NULL if
- * all of #buf contains only bytes of value #c.
- */
-static void *memcchr(const void *buf, int c, size_t buf_size)
-{
-    const char *cbuf = buf;
-    for (size_t i = 0; i < buf_size; i++)
-    {
-        if (cbuf[i] != c)
-        {
-            return (void *) &cbuf[i];                    /* cast-away const */
-        }
-    }
-    return NULL;
-}
-
 static void CopyLockDatabaseAtomically(const char *from, const char *to,
                                        const char *from_pretty_name, const char *to_pretty_name)
 {
@@ -936,88 +934,52 @@ static void CopyLockDatabaseAtomically(const char *from, const char *to,
     int from_fd = open(from, O_RDONLY | O_BINARY);
     if (from_fd < 0)
     {
-        Log(LOG_LEVEL_WARNING, "Could not open %s. (open: '%s')", from_pretty_name, GetErrorStr());
-        goto cleanup_1;
+        Log(LOG_LEVEL_WARNING,
+            "Could not open '%s' (open: %s)",
+            from_pretty_name, GetErrorStr());
+        goto cleanup;
     }
 
     int to_fd = open(tmp_file_name, O_WRONLY | O_CREAT | O_TRUNC | O_BINARY, 0600);
     if (to_fd < 0)
     {
-        Log(LOG_LEVEL_WARNING, "Could not open %s temporary file. (open: '%s')", to_pretty_name, GetErrorStr());
-        goto cleanup_2;
+        Log(LOG_LEVEL_WARNING,
+            "Could not open '%s' temporary file (open: %s)",
+            to_pretty_name, GetErrorStr());
+        goto cleanup;
     }
 
-    char data[CF_BUFSIZE];
-    while (1)
+    size_t total_bytes_written;
+    bool   last_write_was_hole;
+    bool ok1 = FileSparseCopy(from_fd, from_pretty_name,
+                              to_fd,   to_pretty_name, DEV_BSIZE,
+                              &total_bytes_written, &last_write_was_hole);
+
+    /* Make sure changes are persistent on disk, so database cannot get
+     * corrupted at system crash. */
+    bool do_sync = true;
+    bool ok2 = FileSparseClose(to_fd, to_pretty_name, do_sync,
+                               total_bytes_written, last_write_was_hole);
+
+    if (!ok1 || !ok2)
     {
-        int read_status = read(from_fd, data, sizeof(data));
-        if (read_status < 0)
-        {
-            Log(LOG_LEVEL_WARNING, "Could not read from %s. (read: '%s')", from_pretty_name, GetErrorStr());
-            goto cleanup_4;
-        }
-        else if (read_status == 0)
-        {
-            break;
-        }
-
-        /* Is the file sparse? */
-        if (memcchr(data, '\0', read_status) == NULL)    /* is file sparse? */
-        {
-            /*
-             * TODO seek()ing is not enough to preserve sparse-ness on
-             * windows. TODO fix according to:
-             * https://msdn.microsoft.com/en-us/library/windows/desktop/aa365566%28v=vs.85%29.aspx
-             */
-            off_t seek_status = lseek(to_fd, read_status, SEEK_CUR);
-            if (seek_status == (off_t) -1)
-            {
-                Log(LOG_LEVEL_WARNING,
-                    "Could not seek in file '%s' (lseek: %s)",
-                    to_pretty_name, GetErrorStr());
-                goto cleanup_4;
-            }
-        }
-        else                                          /* file is not sparse */
-        {
-            int write_status = write(to_fd, data, read_status);
-            if (write_status < 0)
-            {
-                Log(LOG_LEVEL_WARNING, "Could not write to %s. (write: '%s')", to_pretty_name, GetErrorStr());
-                goto cleanup_4;
-            }
-            else if (write_status == 0)
-            {
-                Log(LOG_LEVEL_WARNING, "Could not write to %s. (write: 'Unknown error')", to_pretty_name);
-                goto cleanup_4;
-            }
-        }
+        Log(LOG_LEVEL_WARNING,
+            "Error while moving database from '%s' to '%s'",
+            from_pretty_name, to_pretty_name);
     }
 
-    // Make sure changes are persistent on disk, so database cannot get corrupted at system crash.
-    if (fsync(to_fd) != 0)
-    {
-        Log(LOG_LEVEL_WARNING, "Could not sync %s file to disk. (fsync: '%s')", to_pretty_name, GetErrorStr());
-        goto cleanup_4;
-    }
-
-    close(to_fd);
     if (rename(tmp_file_name, to) != 0)
     {
-        Log(LOG_LEVEL_WARNING, "Could not move %s into place. (rename: '%s')", to_pretty_name, GetErrorStr());
-        goto cleanup_3;
+        Log(LOG_LEVEL_WARNING, "Could not move '%s' into place (rename: %s)",
+            to_pretty_name, GetErrorStr());
     }
 
-    // Finished.
-    goto cleanup_3;
-
-cleanup_4:
-    close(to_fd);
-cleanup_3:
+  cleanup:
+    if (from_fd != -1)
+    {
+        close(from_fd);
+    }
     unlink(tmp_file_name);
-cleanup_2:
-    close(from_fd);
-cleanup_1:
     free(tmp_file_name);
 }
 
@@ -1102,8 +1064,7 @@ void PurgeLocks(void)
             continue;
         }
 #else
-        if (strncmp(key, "last.internal_bundle.track_license.handle",
-                    strlen("last.internal_bundle.track_license.handle")) == 0)
+        if (STARTSWITH(key, "last.internal_bundle.track_license.handle"))
         {
             continue;
         }
@@ -1111,20 +1072,19 @@ void PurgeLocks(void)
 
         if (now - entry->time > (time_t) CF_LOCKHORIZON)
         {
-            Log(LOG_LEVEL_VERBOSE, " --> Purging lock (%jd) %s", (intmax_t)(now - entry->time), key);
+            Log(LOG_LEVEL_VERBOSE, "Purging lock (%jd s elapsed): %s",
+                (intmax_t) (now - entry->time), key);
             DBCursorDeleteEntry(dbcp);
         }
     }
 
-    Log(LOG_LEVEL_DEBUG, "Finished purging locks in '%s()'", __FUNCTION__);
+    Log(LOG_LEVEL_DEBUG, "Finished purging locks");
 
     lock_horizon.time = now;
     DeleteDBCursor(dbcp);
 
     WriteDB(dbp, "lock_horizon", &lock_horizon, sizeof(lock_horizon));
     CloseLock(dbp);
-
-    Log(LOG_LEVEL_DEBUG, "Exiting '%s()'", __FUNCTION__);
 }
 
 int WriteLock(const char *name)
