@@ -1,7 +1,7 @@
 /*
-   Copyright (C) CFEngine AS
+   Copyright 2017 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by CFEngine AS.
+   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -250,11 +250,11 @@ ExpressionValue EvalExpression(const Expression *expr,
         {
             return EXPRESSION_VALUE_ERROR;
         }
-        else if (0 == strcmp("true", name))
+        else if (strcmp("true", name) == 0)
         {
             ret =  EXPRESSION_VALUE_TRUE;
         }
-        else if (0 == strcmp("false", name))
+        else if (strcmp("false", name) == 0)
         {
             ret =  EXPRESSION_VALUE_FALSE;
         }
