@@ -1,7 +1,7 @@
 /*
-   Copyright (C) CFEngine AS
+   Copyright 2017 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by CFEngine AS.
+   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -33,6 +33,8 @@
 #include <misc_lib.h>
 #include <rlist.h>
 #include <regex.h>                          /* CompileRegex,StringMatchFull */
+#include <string_lib.h>
+
 
 /* Pure, non-thread-safe */
 static char *FirstBackReference(pcre *rx, const char *teststring)
