@@ -1,7 +1,7 @@
 /*
-   Copyright (C) CFEngine AS
+   Copyright 2017 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by CFEngine AS.
+   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -129,7 +129,7 @@ static pid_t GenericCreatePipeAndFork(IOPipe *pipes)
     {
         return -1;
     }
-    
+
     /* Create second pair of descriptors (if exists) to this process.
      * This will allow full I/O operations. */
     if (pipes[1].type && pipe(pipes[1].pipe_desc) < 0)
@@ -140,13 +140,13 @@ static pid_t GenericCreatePipeAndFork(IOPipe *pipes)
     }
 
     pid_t pid = -1;
-    
+
     if ((pid = fork()) == -1)
     {
         /* One pipe will be always here. */
         close(pipes[0].pipe_desc[0]);
         close(pipes[0].pipe_desc[1]);
-        
+
         /* Second pipe is optional so we have to check existence. */
         if (pipes[1].type)
         {
@@ -156,12 +156,25 @@ static pid_t GenericCreatePipeAndFork(IOPipe *pipes)
         return -1;
     }
 
+    /* WARNING: UNDEFINED BEHAVIOUR if the program is multi-threaded! */
     signal(SIGCHLD, SIG_DFL);
 
-    // Redmine #2971: reset SIGPIPE signal handler to have a sane behavior of piped commands within child
-    if (pid == 0)
+    if (pid == 0)                                               /* child */
     {
+        /* WARNING only call async-signal-safe functions in child. */
+
+        /* The fork()ed child is always single-threaded, but we are only
+         * allowed to call async-signal-safe functions (man 3p fork). */
+
+        // Redmine #2971: reset SIGPIPE signal handler in the child to have a
+        // sane behavior of piped commands within child
         signal(SIGPIPE, SIG_DFL);
+
+        /* The child should always accept all signals after it has exec'd,
+         * else the child might be unkillable! (happened in ENT-3147). */
+        sigset_t sigmask;
+        sigemptyset(&sigmask);
+        sigprocmask(SIG_SETMASK, &sigmask, NULL);
     }
 
     ALARM_PID = (pid != 0 ? pid : -1);
@@ -176,9 +189,9 @@ static pid_t CreatePipeAndFork(const char *type, int *pd)
     IOPipe pipes[2];
     pipes[0].type = type;
     pipes[1].type = NULL; /* We don't want to create this one. */
-    
+
     pid_t pid = GenericCreatePipeAndFork(pipes);
-    
+
     pd[0] = pipes[0].pipe_desc[0];
     pd[1] = pipes[0].pipe_desc[1];
 
@@ -222,16 +235,16 @@ IOData cf_popen_full_duplex(const char *command, bool capture_stderr, bool requi
         Log(LOG_LEVEL_ERR, "Couldn't fork child process: %s", GetErrorStr());
         return (IOData) {-1, -1};
     }
-    
+
     else if (pid > 0) // parent
     {
         close(child_pipe[WRITE]);
         close(parent_pipe[READ]);
-        
+
         IOData io_desc;
         io_desc.write_fd = parent_pipe[WRITE];
         io_desc.read_fd = child_pipe[READ];
-        
+
         SetChildFD(parent_pipe[WRITE], pid);
         SetChildFD(child_pipe[READ], pid);
         return io_desc;
@@ -247,13 +260,13 @@ IOData cf_popen_full_duplex(const char *command, bool capture_stderr, bool requi
             Log(LOG_LEVEL_ERR, "Can not execute dup2: %s", GetErrorStr());
             _exit(EXIT_FAILURE);
         }
-        
+
         if (capture_stderr)
         {
             /* Merge stdout/stderr */
             if(dup2(child_pipe[WRITE], 2) == -1)
             {
-                Log(LOG_LEVEL_ERR, "Can not execute dup2 for merging stderr: %s", 
+                Log(LOG_LEVEL_ERR, "Can not execute dup2 for merging stderr: %s",
                     GetErrorStr());
                 _exit(EXIT_FAILURE);
             }
@@ -262,12 +275,12 @@ IOData cf_popen_full_duplex(const char *command, bool capture_stderr, bool requi
         {
             /* leave stderr open */
         }
-        
+
         close(child_pipe[WRITE]);
         close(parent_pipe[READ]);
 
         CloseChildrenFD();
-        
+
         char **argv  = ArgSplitCommand(command);
         int res = -1;
 
@@ -306,8 +319,10 @@ FILE *cf_popen(const char *command, const char *type, bool capture_stderr)
         return NULL;
     }
 
-    if (pid == 0)
+    if (pid == 0)                                               /* child */
     {
+        /* WARNING only call async-signal-safe functions in child. */
+
         switch (*type)
         {
         case 'r':
@@ -403,8 +418,10 @@ FILE *cf_popensetuid(const char *command, const char *type, uid_t uid, gid_t gid
         return NULL;
     }
 
-    if (pid == 0)
+    if (pid == 0)                                               /* child */
     {
+        /* WARNING only call async-signal-safe functions in child. */
+
         switch (*type)
         {
         case 'r':
@@ -515,8 +532,10 @@ FILE *cf_popen_sh(const char *command, const char *type)
         return NULL;
     }
 
-    if (pid == 0)
+    if (pid == 0)                                               /* child */
     {
+        /* WARNING only call async-signal-safe functions in child. */
+
         switch (*type)
         {
         case 'r':
@@ -594,8 +613,10 @@ FILE *cf_popen_shsetuid(const char *command, const char *type, uid_t uid, gid_t
         return NULL;
     }
 
-    if (pid == 0)
+    if (pid == 0)                                               /* child */
     {
+        /* WARNING only call async-signal-safe functions in child. */
+
         switch (*type)
         {
         case 'r':
@@ -741,7 +762,7 @@ int cf_pclose(FILE *pp)
         CHILDREN[fd] = 0;
         ThreadUnlock(cft_count);
     }
-    
+
     if (fclose(pp) == EOF || pid == 0)
     {
         return -1;
@@ -757,14 +778,14 @@ int cf_pclose_full_duplex_side(int fd)
         close(fd);
         return -1;
     }
-    
+
     if (CHILDREN == NULL)       /* popen hasn't been called */
     {
         ThreadUnlock(cft_count);
         close(fd);
         return -1;
     }
-    
+
     if (fd >= MAX_FD)
     {
         ThreadUnlock(cft_count);
@@ -790,7 +811,7 @@ int cf_pclose_full_duplex(IOData *data)
         {
             close(data->read_fd);
         }
-        
+
         if (data->write_fd >= 0)
         {
             close(data->write_fd);
@@ -805,7 +826,7 @@ int cf_pclose_full_duplex(IOData *data)
         {
             close(data->read_fd);
         }
-        
+
         if (data->write_fd >= 0)
         {
             close(data->write_fd);
@@ -835,7 +856,7 @@ int cf_pclose_full_duplex(IOData *data)
         CHILDREN[data->read_fd] = 0;
         ThreadUnlock(cft_count);
     }
-    
+
     if (close(data->read_fd) != 0 || (data->write_fd >= 0 && close(data->write_fd) != 0) || pid == 0)
     {
         return -1;
