@@ -1,7 +1,7 @@
 /*
-   Copyright (C) CFEngine AS
+   Copyright 2017 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by CFEngine AS.
+   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -83,6 +83,9 @@ static bool PromiseCheck(const Promise *pp, Seq *errors);
 
 /*************************************************************************/
 
+/**
+ * @brief Return a default bundle name for this method/service
+ */
 Rval DefaultBundleConstraint(const Promise *pp, char *promisetype)
 {
     static char name[CF_BUFSIZE];
@@ -262,33 +265,40 @@ StringSet *PolicySourceFiles(const Policy *policy)
 
 /*************************************************************************/
 
-static char *StripNamespace(const char *full_symbol)
+static const char *StripNamespace(const char *full_symbol)
 {
     char *sep = strchr(full_symbol, CF_NS);
-    if (sep)
+
+    if (sep == NULL)
     {
-        return xstrdup(sep + 1);
+        return full_symbol;
     }
     else
     {
-        return xstrdup(full_symbol);
+        return sep + 1;
     }
 }
 
 /*************************************************************************/
 
+/**
+ * @brief Query a policy for a body
+ * @param policy The policy to query
+ * @param ns Namespace filter (optionally NULL)
+ * @param type Body type filter
+ * @param name Body name filter
+ * @return Body child object if found, otherwise NULL
+ */
 Body *PolicyGetBody(const Policy *policy, const char *ns, const char *type, const char *name)
 {
     for (size_t i = 0; i < SeqLength(policy->bodies); i++)
     {
         Body *bp = SeqAt(policy->bodies, i);
+        const char *body_symbol = StripNamespace(bp->name);
 
-        char *body_symbol = StripNamespace(bp->name);
-
-        if (strcmp(bp->type, type) == 0 && strcmp(body_symbol, name) == 0)
+        if (strcmp(bp->type, type)    == 0 &&
+            strcmp(body_symbol, name) == 0)
         {
-            free(body_symbol);
-
             // allow namespace to be optionally matched
             if (ns && strcmp(bp->ns, ns) != 0)
             {
@@ -297,8 +307,6 @@ Body *PolicyGetBody(const Policy *policy, const char *ns, const char *type, cons
 
             return bp;
         }
-
-        free(body_symbol);
     }
 
     return NULL;
@@ -306,18 +314,27 @@ Body *PolicyGetBody(const Policy *policy, const char *ns, const char *type, cons
 
 /*************************************************************************/
 
+/**
+ * @brief Query a policy for a bundle
+ * @param policy The policy to query
+ * @param ns Namespace filter (optionally NULL)
+ * @param type Bundle type filter
+ * @param name Bundle name filter
+ * @return Bundle child object if found, otherwise NULL
+ */
 Bundle *PolicyGetBundle(const Policy *policy, const char *ns, const char *type, const char *name)
 {
+    const char *bundle_symbol = StripNamespace(name);
+
     for (size_t i = 0; i < SeqLength(policy->bundles); i++)
     {
         Bundle *bp = SeqAt(policy->bundles, i);
 
-        char *bundle_symbol = StripNamespace(name);
-
-        if ((!type || strcmp(bp->type, type) == 0) && ((strcmp(bundle_symbol, bp->name) == 0) || (strcmp(bp->name, name) == 0)))
+        if ((type == NULL || strcmp(bp->type, type) == 0)
+            &&
+            ((strcmp(bp->name, bundle_symbol) == 0) ||
+             (strcmp(bp->name, name)          == 0)))
         {
-            free(bundle_symbol);
-
             // allow namespace to be optionally matched
             if (ns && strcmp(bp->ns, ns) != 0)
             {
@@ -326,8 +343,6 @@ Bundle *PolicyGetBundle(const Policy *policy, const char *ns, const char *type,
 
             return bp;
         }
-
-        free(bundle_symbol);
     }
 
     return NULL;
@@ -335,6 +350,11 @@ Bundle *PolicyGetBundle(const Policy *policy, const char *ns, const char *type,
 
 /*************************************************************************/
 
+/**
+ * @brief Check to see if a policy is runnable (contains body common control)
+ * @param policy Policy to check
+ * @return True if policy is runnable
+ */
 bool PolicyIsRunnable(const Policy *policy)
 {
     return PolicyGetBody(policy, NULL, "common", "control") != NULL;
@@ -342,6 +362,12 @@ bool PolicyIsRunnable(const Policy *policy)
 
 /*************************************************************************/
 
+/**
+ * @brief Merge two partial policy objects. The memory for the child objects of the original policies are transferred to the new parent.
+ * @param a
+ * @param b
+ * @return Merged policy
+ */
 Policy *PolicyMerge(Policy *a, Policy *b)
 {
     Policy *result = PolicyNew();
@@ -396,6 +422,11 @@ const char *ConstraintGetNamespace(const Constraint *cp)
 
 /*************************************************************************/
 
+/**
+ * @brief Convenience function to get the policy object associated with a promise
+ * @param promise
+ * @return Policy object
+ */
 const Policy *PolicyFromPromise(const Promise *promise)
 {
     assert(promise);
@@ -717,7 +748,7 @@ static const char *RvalFullSymbol(const Rval *rval)
 }
 
 /**
- * @return A copy of the namespace compoent of an Rval, or NULL. e.g. "foo:bar""foo"
+ * @return A copy of the namespace component of a qualified name, or NULL. e.g. "foo:bar" -> "foo"
  */
 char *QualifiedNameNamespaceComponent(const char *qualified_name)
 {
@@ -735,7 +766,7 @@ char *QualifiedNameNamespaceComponent(const char *qualified_name)
 }
 
 /**
- * @return A copy of the symbol compoent of an Rval, or NULL. e.g. "foo:bar""bar"
+ * @return A copy of the symbol compoent of a qualified name, or NULL. e.g. "foo:bar" -> "bar"
  */
 char *QualifiedNameScopeComponent(const char *qualified_name)
 {
@@ -961,6 +992,13 @@ bool PolicyCheckDuplicateHandles(const Policy *policy, Seq *errors)
     return success;
 }
 
+/**
+ * @brief Check a runnable policy DOM for errors
+ * @param policy Policy to check
+ * @param errors Sequence of PolicyError to append errors to
+ * @param ignore_missing_bundles Whether to ignore missing bundle references
+ * @return True if no new errors are found
+ */
 bool PolicyCheckRunnable(const EvalContext *ctx, const Policy *policy, Seq *errors, bool ignore_missing_bundles)
 {
     bool success = true;
@@ -978,6 +1016,12 @@ bool PolicyCheckRunnable(const EvalContext *ctx, const Policy *policy, Seq *erro
     return success;
 }
 
+/**
+ * @brief Check a partial policy DOM for errors
+ * @param policy Policy to check
+ * @param errors Sequence of PolicyError to append errors to
+ * @return True if no new errors are found
+ */
 bool PolicyCheckPartial(const Policy *policy, Seq *errors)
 {
     bool success = true;
@@ -1338,7 +1382,7 @@ Promise *PromiseTypeAppendPromise(PromiseType *type, const char *promiser, Rval
     pp->conlist = SeqNew(10, ConstraintDestroy);
     pp->org_pp = pp;
 
-    if (NULL != varclasses)
+    if (varclasses != NULL)
     {
         PromiseAppendConstraint(pp, "ifvarclass", RvalNew(varclasses, RVAL_TYPE_SCALAR), true);
     }
@@ -1433,17 +1477,21 @@ Constraint *PromiseAppendConstraint(Promise *pp, const char *lval, Rval rval, bo
                 {
                 case RVAL_TYPE_FNCALL: // case 1: merge FnCall with scalar
                 {
-                    Log(LOG_LEVEL_DEBUG, "PromiseAppendConstraint: merging PREVIOUS %s string context rval %s", old_cp->lval, RvalToString(old_cp->rval));
-                    Log(LOG_LEVEL_DEBUG, "PromiseAppendConstraint: merging NEW %s rval %s", old_cp->lval, RvalToString(old_cp->rval));
+                    char * rval_string = RvalToString(old_cp->rval);
+                    Log(LOG_LEVEL_DEBUG, "PromiseAppendConstraint: merging PREVIOUS %s string context rval %s", old_cp->lval, rval_string);
+                    Log(LOG_LEVEL_DEBUG, "PromiseAppendConstraint: merging NEW %s rval %s", old_cp->lval, rval_string);
+                    free(rval_string);
 
                     Rlist *synthetic_args = NULL;
-                    RlistAppendScalar(&synthetic_args, xstrdup(RvalScalarValue(old_cp->rval)));
+                    RlistAppendScalar(&synthetic_args, RvalScalarValue(old_cp->rval));
 
                     // append the old Rval (a function call) under the arguments of the new one
                     RlistAppend(&synthetic_args, rval.item, RVAL_TYPE_FNCALL);
 
-                    Rval replacement = (Rval) { FnCallNew(xstrdup("and"), synthetic_args), RVAL_TYPE_FNCALL };
-                    Log(LOG_LEVEL_DEBUG, "PromiseAppendConstraint: MERGED %s rval %s", old_cp->lval, RvalToString(replacement));
+                    Rval replacement = (Rval) { FnCallNew("and", synthetic_args), RVAL_TYPE_FNCALL };
+                    rval_string = RvalToString(replacement);
+                    Log(LOG_LEVEL_DEBUG, "PromiseAppendConstraint: MERGED %s rval %s", old_cp->lval, rval_string);
+                    free(rval_string);
 
                     // overwrite the old Constraint rval with its replacement
                     RvalDestroy(cp->rval);
@@ -1525,30 +1573,23 @@ const PromiseType *BundleGetPromiseType(const Bundle *bp, const char *name)
 
 /****************************************************************************/
 
-static char *EscapeQuotes(const char *s, char *out, int outSz)
+static Buffer *EscapeQuotes(const char *raw, Buffer *out)
 {
-    char *spt;
     const char *spf;
-    int i = 0;
 
-    memset(out, 0, outSz);
-
-    for (spf = s, spt = out; (i < outSz - 2) && (*spf != '\0'); spf++, spt++, i++)
+    for (spf = raw; *spf != '\0'; spf++)
     {
         switch (*spf)
         {
         case '\'':
         case '\"':
-            *spt++ = '\\';
-            *spt = *spf;
-            i += 3;
+            BufferAppendChar(out, '\\');
             break;
 
         default:
-            *spt = *spf;
-            i++;
             break;
         }
+        BufferAppendChar(out, *spf);
     }
 
     return out;
@@ -1565,9 +1606,9 @@ static JsonElement *AttributeValueToJson(Rval rval, bool symbolic_reference)
 
     case RVAL_TYPE_SCALAR:
     {
-        char buffer[CF_BUFSIZE];
+        Buffer *buffer = BufferNewWithCapacity(strlen(rval.item));
 
-        EscapeQuotes((const char *) rval.item, buffer, sizeof(buffer));
+        EscapeQuotes((const char *) rval.item, buffer);
 
         JsonElement *json_attribute = JsonObjectCreate(10);
 
@@ -1579,7 +1620,9 @@ static JsonElement *AttributeValueToJson(Rval rval, bool symbolic_reference)
         {
             JsonObjectAppendString(json_attribute, "type", "string");
         }
-        JsonObjectAppendString(json_attribute, "value", buffer);
+        JsonObjectAppendString(json_attribute, "value", BufferData(buffer));
+
+        BufferDestroy(buffer);
 
         return json_attribute;
     }
@@ -1761,6 +1804,11 @@ static JsonElement *BundleContextsToJson(const Seq *promises)
     return json_contexts;
 }
 
+/**
+ * @brief Serialize a bundle as JSON
+ * @param bundle The bundle to serialize
+ * @return A JsonElement representing the input bundle
+ */
 JsonElement *BundleToJson(const Bundle *bundle)
 {
     JsonElement *json_bundle = JsonObjectCreate(10);
@@ -1809,7 +1857,11 @@ JsonElement *BundleToJson(const Bundle *bundle)
     return json_bundle;
 }
 
-
+/**
+ * @brief Serialize a body as JSON
+ * @param body The body to serialize
+ * @return A JsonElement representing the input body
+ */
 JsonElement *BodyToJson(const Body *body)
 {
     JsonElement *json_body = JsonObjectCreate(10);
@@ -1841,6 +1893,11 @@ JsonElement *BodyToJson(const Body *body)
     return json_body;
 }
 
+/**
+ * @brief Serialize a policy as JSON
+ * @param policy The policy to serialize
+ * @return A JsonElement representing the input policy
+ */
 JsonElement *PolicyToJson(const Policy *policy)
 {
     JsonElement *json_policy = JsonObjectCreate(10);
@@ -2016,6 +2073,11 @@ void BundleToString(Writer *writer, Bundle *bundle)
     WriterWrite(writer, "\n}\n");
 }
 
+/**
+ * @brief Pretty-print a policy
+ * @param policy The policy to print
+ * @param writer Writer to write into
+ */
 void PolicyToString(const Policy *policy, Writer *writer)
 {
     for (size_t i = 0; i < SeqLength(policy->bundles); i++)
@@ -2219,7 +2281,11 @@ static Body *PolicyAppendBodyJson(Policy *policy, JsonElement *json_body)
     return body;
 }
 
-
+/**
+ * @brief Deserialize a policy from JSON
+ * @param json_policy JSON to deserialize
+ * @return A policy DOM
+ */
 Policy *PolicyFromJson(JsonElement *json_policy)
 {
     Policy *policy = PolicyNew();
@@ -2245,7 +2311,12 @@ Policy *PolicyFromJson(JsonElement *json_policy)
     return policy;
 }
 
-
+/**
+ * @brief A sequence of constraints matching the l-value.
+ * @param body Body to query
+ * @param lval l-value to match
+ * @return Sequence of pointers to the constraints. Destroying it does not alter the DOM.
+ */
 Seq *BodyGetConstraint(Body *body, const char *lval)
 {
     Seq *matches = SeqNew(5, NULL);
@@ -2276,6 +2347,11 @@ bool BodyHasConstraint(const Body *body, const char *lval)
     return false;
 }
 
+/**
+ * @brief Get the context of the given constraint
+ * @param cp
+ * @return context. never returns NULL.
+ */
 const char *ConstraintContext(const Constraint *cp)
 {
     switch (cp->type)
@@ -2292,6 +2368,11 @@ const char *ConstraintContext(const Constraint *cp)
     }
 }
 
+/**
+ * @brief Returns the first effective constraint from a list of candidates, depending on evaluation state.
+ * @param constraints The list of potential candidates
+ * @return The effective constraint, or NULL if none are found.
+ */
 Constraint *EffectiveConstraint(const EvalContext *ctx, Seq *constraints)
 {
     for (size_t i = 0; i < SeqLength(constraints); i++)
@@ -2322,6 +2403,12 @@ void ConstraintDestroy(Constraint *cp)
 
 /*****************************************************************************/
 
+/**
+ * @brief Get the trinary boolean value of the first effective constraint found matching, from a promise
+ * @param lval
+ * @param list
+ * @return True/false, or CF_UNDEFINED if not found
+ */
 int PromiseGetConstraintAsBoolean(const EvalContext *ctx, const char *lval, const Promise *pp)
 {
     int retval = CF_UNDEFINED;
@@ -2376,6 +2463,12 @@ int PromiseGetConstraintAsBoolean(const EvalContext *ctx, const char *lval, cons
 
 /*****************************************************************************/
 
+/**
+ * @brief Get the trinary boolean value of the first effective constraint found matching
+ * @param lval
+ * @param constraints
+ * @return True/false, or CF_UNDEFINED if not found
+ */
 int ConstraintsGetAsBoolean(const EvalContext *ctx, const char *lval, const Seq *constraints)
 {
     int retval = CF_UNDEFINED;
@@ -2531,6 +2624,9 @@ static void PromiseTypePath(Writer *w, const PromiseType *pt)
     WriterWrite(w, pt->name);
 }
 
+/**
+ * @brief Write a string describing the promise location in policy, e.g. /default/foo/packages/'emacs'
+ */
 void PromisePath(Writer *w, const Promise *pp)
 {
     PromiseTypePath(w, pp->parent_promise_type);
@@ -2540,11 +2636,22 @@ void PromisePath(Writer *w, const Promise *pp)
     WriterWriteChar(w, '\'');
 }
 
+/**
+ * @brief Return handle of the promise.
+ * @param pp
+ * @return Promise handle or NULL if no handle is provided
+ */
 const char *PromiseGetHandle(const Promise *pp)
 {
     return (const char *)PromiseGetImmediateRvalValue("handle", pp, RVAL_TYPE_SCALAR);
 }
 
+/**
+ * @brief Get the int value of the first effective constraint found matching, from a promise
+ * @param lval
+ * @param pp
+ * @return Int value, or CF_NOINT
+ */
 int PromiseGetConstraintAsInt(const EvalContext *ctx, const char *lval, const Promise *pp)
 {
     int retval = CF_NOINT;
@@ -2567,6 +2674,10 @@ int PromiseGetConstraintAsInt(const EvalContext *ctx, const char *lval, const Pr
 
 /*****************************************************************************/
 
+/**
+ * @brief Get the real value of the first effective constraint found matching, from a promise
+ * @return true if value could be extracted
+ */
 bool PromiseGetConstraintAsReal(const EvalContext *ctx, const char *lval, const Promise *pp, double *value_out)
 {
     const Constraint *cp = PromiseGetConstraint(pp, lval);
@@ -2612,6 +2723,12 @@ static bool Str2Mode(const char *s, mode_t *mode_out)
     return true;
 }
 
+/**
+ * @brief Get the octal value of the first effective constraint found matching, from a promise
+ * @param lval
+ * @param list
+ * @return Double value, or 077 if not found
+ */
 mode_t PromiseGetConstraintAsOctal(const EvalContext *ctx, const char *lval, const Promise *pp)
 {
     mode_t retval = 077;
@@ -2650,6 +2767,12 @@ uid_t PromiseGetConstraintAsUid(const EvalContext *ctx, const char *lval, const
 
 #else /* !__MINGW32__ */
 
+/**
+ * @brief Get the uid value of the first effective constraint found matching, from a promise
+ * @param lval
+ * @param pp
+ * @return Uid value, or CF_SAME_OWNER if not found
+ */
 uid_t PromiseGetConstraintAsUid(const EvalContext *ctx, const char *lval, const Promise *pp)
 {
     int retval = CF_SAME_OWNER;
@@ -2686,6 +2809,12 @@ gid_t PromiseGetConstraintAsGid(const EvalContext *ctx, char *lval, const Promis
 
 #else /* !__MINGW32__ */
 
+/**
+ * @brief Get the uid value of the first effective constraint found matching, from a promise
+ * @param lval
+ * @param pp
+ * @return Gid value, or CF_SAME_GROUP if not found
+ */
 gid_t PromiseGetConstraintAsGid(const EvalContext *ctx, char *lval, const Promise *pp)
 {
     int retval = CF_SAME_GROUP;
@@ -2712,6 +2841,12 @@ gid_t PromiseGetConstraintAsGid(const EvalContext *ctx, char *lval, const Promis
 
 /*****************************************************************************/
 
+/**
+ * @brief Get the Rlist value of the first effective constraint found matching, from a promise
+ * @param lval
+ * @param list
+ * @return Rlist or NULL if not found (note: same as empty list)
+ */
 // FIX: promise constrained classed?
 Rlist *PromiseGetConstraintAsList(const EvalContext *ctx, const char *lval, const Promise *pp)
 {
@@ -2731,6 +2866,12 @@ Rlist *PromiseGetConstraintAsList(const EvalContext *ctx, const char *lval, cons
     return NULL;
 }
 
+/**
+ * @brief Get the first effective constraint from the promise, also does some checking
+ * @param promise
+ * @param lval
+ * @return Effective constraint if found, otherwise NULL
+ */
 Constraint *PromiseGetConstraint(const Promise *pp, const char *lval)
 {
     if (!pp)
@@ -2770,7 +2911,15 @@ Constraint *PromiseGetConstraintWithType(const Promise *pp, const char *lval, Rv
     return NULL;
 }
 
-
+/**
+ * @brief Get the first constraint from the promise
+ *
+ * Kill this function with fire once we have separated promise constraints and body constraints.
+ *
+ * @param promise
+ * @param lval
+ * @return Constraint if found, otherwise NULL
+ */
 Constraint *PromiseGetImmediateConstraint(const Promise *pp, const char *lval)
 {
     if (pp == NULL)
@@ -2799,6 +2948,17 @@ Constraint *PromiseGetImmediateConstraint(const Promise *pp, const char *lval)
     return NULL;
 }
 
+/**
+ * @brief Get the Rval value of the first constraint that matches the given
+ * type. Checks that this constraint does not have any contexts attached.
+ *
+ * Kill this function with fire once we have separated body constraints and bundle constraints.
+ *
+ * @param lval
+ * @param promise
+ * @param type
+ * @return Rval value if found, NULL otherwise
+ */
 void *PromiseGetImmediateRvalValue(const char *lval, const Promise *pp, RvalType rtype)
 {
     const Constraint *constraint = PromiseGetImmediateConstraint(pp, lval);
@@ -2815,6 +2975,13 @@ void *PromiseGetImmediateRvalValue(const char *lval, const Promise *pp, RvalType
 
 /*****************************************************************************/
 
+/**
+ * @brief Get the Rval value of the first effective constraint that matches the given type
+ * @param lval
+ * @param promise
+ * @param type
+ * @return Rval value if found, NULL otherwise
+ */
 void *PromiseGetConstraintAsRval(const Promise *pp, const char *lval, RvalType rtype)
 {
     const Constraint *constraint = PromiseGetConstraint(pp, lval);
@@ -2831,52 +2998,66 @@ void *PromiseGetConstraintAsRval(const Promise *pp, const char *lval, RvalType r
 
 /*****************************************************************************/
 
+/*
+ * Check promise constraints while iterating through all slist/containers
+ * combinations, called from ExpandPromiseAndDo() during PRE-EVAL.  This is
+ * currently running also in cf-promises, but it's enough if such errors are
+ * detected in the agent run.
+ *
+ * TODO remove CommonEvalPromise() actuator, that all it does is call this
+ * function, and call this one at the beginning of the main actuators, like
+ * KeepAgentPromise().
+ */
 void PromiseRecheckAllConstraints(const EvalContext *ctx, const Promise *pp)
 {
-    static Item *EDIT_ANCHORS = NULL; /* GLOBAL_X */
-
     for (size_t i = 0; i < SeqLength(pp->conlist); i++)
     {
         Constraint *cp = SeqAt(pp->conlist, i);
         SyntaxTypeMatch err = ConstraintCheckType(cp);
         if (err != SYNTAX_TYPE_MATCH_OK && err != SYNTAX_TYPE_MATCH_ERROR_UNEXPANDED)
         {
-            PolicyError *error = PolicyErrorNew(POLICY_ELEMENT_TYPE_CONSTRAINT, cp, "In attribute '%s', %s", cp->lval, SyntaxTypeMatchToString(err));
+            PolicyError *error =
+                PolicyErrorNew(POLICY_ELEMENT_TYPE_CONSTRAINT, cp,
+                               "In attribute '%s', %s",
+                               cp->lval, SyntaxTypeMatchToString(err));
             char *error_str = PolicyErrorToString(error);
-            PolicyErrorDestroy(error);
 
             Log(LOG_LEVEL_ERR, "%s", error_str);
+            PolicyErrorDestroy(error);
             free(error_str);
 
             FatalError(ctx, "Cannot continue");
         }
     }
 
+    /* Check and warn for non-convergence, see commits
+       4f8c19b84327b8f3c2e269173196282ccedfdad9 and
+       30c109d22e170a781a647b04b4b0a4a2f7244871. */
     if (strcmp(pp->parent_promise_type->name, "insert_lines") == 0)
     {
-        /* Multiple additions with same criterion will not be convergent -- but ignore for empty file baseline */
+        /* TODO without static var, actually remove this check from here
+         * completely, do it at the end of PRE-EVAL promise iterations
+         * (ExpandPromiseAndDo() after the main loop). */
+        static Item *EDIT_ANCHORS = NULL; /* GLOBAL_X */
 
-        const char *sp = PromiseGetConstraintAsRval(pp, "select_line_matching", RVAL_TYPE_SCALAR);
-        if (sp)
+        const char *sp = PromiseGetConstraintAsRval(pp, "select_line_matching",
+                                                    RVAL_TYPE_SCALAR);
+        if (sp != NULL && !IsExpandable(sp))
         {
-            if (!IsExpandable(sp))
+            /* Avoid adding twice the same select_line_matching anchor. */
+            const char *bun = PromiseGetBundle(pp)->name;
+            const Item *ptr = ReturnItemInClass(EDIT_ANCHORS, sp, bun);
+            if (ptr != NULL)
             {
-                const Item *ptr = NULL;
-                if ((ptr = ReturnItemIn(EDIT_ANCHORS, sp)))
-                {
-                    if (strcmp(ptr->classes, PromiseGetBundle(pp)->name) == 0)
-                    {
-                        Log(LOG_LEVEL_INFO,
-                            "insert_lines promise uses the same select_line_matching anchor '%s' as another promise. This will lead to non-convergent behaviour unless 'empty_file_before_editing' is set",
-                            sp);
-                        PromiseRef(LOG_LEVEL_INFO, pp);
-                    }
-                }
-                else
-                {
-                    PrependItem(&EDIT_ANCHORS, sp, PromiseGetBundle(pp)->name);
-                }
+                Log(LOG_LEVEL_INFO,
+                    "insert_lines promise uses the same select_line_matching anchor '%s' as another promise."
+                    " This will lead to non-convergent behaviour unless 'empty_file_before_editing' is set",
+                    sp);
+                PromiseRef(LOG_LEVEL_INFO, pp);
+                return;
             }
+
+            PrependItem(&EDIT_ANCHORS, sp, bun);
         }
     }
 }
@@ -2967,6 +3148,9 @@ static SyntaxTypeMatch ConstraintCheckType(const Constraint *cp)
     return SYNTAX_TYPE_MATCH_OK;
 }
 
+/**
+ * @brief Check whether the promise type is allowed one
+ */
 /* FIXME: need to be done automatically */
 bool BundleTypeCheck(const char *name)
 {
