@@ -1,7 +1,7 @@
 /*
-   Copyright (C) CFEngine AS
+   Copyright 2017 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by CFEngine AS.
+   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -25,15 +25,19 @@
 #ifndef CFENGINE_PROMISES_H
 #define CFENGINE_PROMISES_H
 
+
 #include <cf3.defs.h>
 
 #include <logging.h>
 #include <sequence.h>
 
+
 Promise *DeRefCopyPromise(EvalContext *ctx, const Promise *pp);
 Promise *ExpandDeRefPromise(EvalContext *ctx, const Promise *pp, bool *excluded);
 void PromiseRef(LogLevel level, const Promise *pp);
 void CopyBodyConstraintsToPromise(EvalContext *ctx, Promise *pp,
                                   const Body *bp);
+const char *PromiseID(const Promise *pp);
+
 
 #endif
