@@ -1,7 +1,7 @@
 /*
-   Copyright (C) CFEngine AS
+   Copyright 2017 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by CFEngine AS.
+   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -32,7 +32,6 @@
 #include <fncall.h>
 #include <string_lib.h>                                       /* StringHash */
 #include <regex.h>          /* StringMatchWithPrecompiledRegex,CompileRegex */
-#include <mutex.h>
 #include <misc_lib.h>
 #include <assoc.h>
 #include <eval_context.h>
@@ -77,7 +76,7 @@ RvalType DataTypeToRvalType(DataType datatype)
 
 bool RlistValueIsType(const Rlist *rlist, RvalType type)
 {
-    return (NULL != rlist &&
+    return (rlist != NULL &&
             rlist->val.type == type);
 }
 
@@ -260,23 +259,9 @@ bool RlistMatchesRegex(const Rlist *list, const char *regex)
     return false;
 }
 
-// TODO: cf_null lists should not be needed
 bool RlistIsNullList(const Rlist *list)
 {
-    for (const Rlist *rp = list; rp; rp = rp->next)
-    {
-        if (rp->val.type != RVAL_TYPE_SCALAR)
-        {
-            return false;
-        }
-
-        if (strcmp(CF_NULL_VALUE, RlistScalarValue(rp)) != 0)
-        {
-            return false;
-        }
-    }
-
-    return true;
+    return (list == NULL);
 }
 
 bool RlistIsInListOfRegex(const Rlist *list, const char *str)
@@ -316,6 +301,9 @@ Rlist *RlistAppendRval(Rlist **start, Rval rval)
 {
     Rlist *rp = xmalloc(sizeof(Rlist));
 
+    rp->val  = rval;
+    rp->next = NULL;
+
     if (*start == NULL)
     {
         *start = rp;
@@ -331,23 +319,26 @@ Rlist *RlistAppendRval(Rlist **start, Rval rval)
         lp->next = rp;
     }
 
-    rp->val = rval;
-
-    ThreadLock(cft_lock);
-
-    rp->next = NULL;
-
-    ThreadUnlock(cft_lock);
-
     return rp;
 }
 
+/* Inserts an Rlist node with value #rval, right after the rlist node #node. */
+void RlistInsertAfter(Rlist *node, Rval rval)
+{
+    assert(node != NULL);
+
+    Rlist new_node = { .val  = rval,
+                       .next = node->next };
+
+    node->next = xmemdup(&new_node, sizeof(new_node));
+}
+
 Rval RvalNewRewriter(const void *item, RvalType type, JsonElement *map)
 {
     switch (type)
     {
     case RVAL_TYPE_SCALAR:
-        if (NULL != map && JsonLength(map) > 0 &&       // do we have a rewrite map?
+        if (map != NULL && JsonLength(map) > 0 &&       // do we have a rewrite map?
             (strstr(item, "$(") || strstr(item, "${"))) // are there unresolved variable references?
         {
             // TODO: replace with BufferSearchAndReplace when the
@@ -587,6 +578,9 @@ Rlist *RlistAppendAllTypes(Rlist **start, const void *item, RvalType type, bool
 
     Rlist *rp = xmalloc(sizeof(Rlist));
 
+    rp->val  = RvalNew(item, type);
+    rp->next = NULL;
+
     if (*start == NULL)
     {
         *start = rp;
@@ -600,14 +594,6 @@ Rlist *RlistAppendAllTypes(Rlist **start, const void *item, RvalType type, bool
         lp->next = rp;
     }
 
-    rp->val = RvalCopy((Rval) {(void *) item, type});
-
-    ThreadLock(cft_lock);
-
-    rp->next = NULL;
-
-    ThreadUnlock(cft_lock);
-
     return rp;
 }
 
@@ -619,9 +605,9 @@ static Rlist *RlistPrependRval(Rlist **start, Rval rval)
 
     rp->next = *start;
     rp->val = rval;
-    ThreadLock(cft_lock);
+
     *start = rp;
-    ThreadUnlock(cft_lock);
+
     return rp;
 }
 
@@ -702,7 +688,7 @@ typedef enum
 } state;
 
 #define CLASS_BLANK(x)  (((x)==' ')||((x)=='\t'))
-#define CLASS_START1(x) (((x)=='\'')) 
+#define CLASS_START1(x) (((x)=='\''))
 #define CLASS_START2(x) (((x)=='"'))
 #define CLASS_END1(x)   ((CLASS_START1(x)))
 #define CLASS_END2(x)   ((CLASS_START2(x)))
@@ -722,7 +708,7 @@ typedef enum
 
 /**
  @brief parse elements in a list passed through use_module
- 
+
  @param[in] str: is the string to parse
  @param[out] newlist: rlist of elements found
 
@@ -751,7 +737,7 @@ static int LaunchParsingMachine(const char *str, Rlist **newlist)
                 {
                     current_state = ST_OPENED;
                 }
-                else if (CLASS_BRA1(*s)) 
+                else if (CLASS_BRA1(*s))
                 {
                     current_state = ST_IO;
                 }
@@ -930,9 +916,7 @@ void RvalDestroy(Rval rval)
     switch (rval.type)
     {
     case RVAL_TYPE_SCALAR:
-        ThreadLock(cft_lock);
         free(RvalScalarValue(rval));
-        ThreadUnlock(cft_lock);
         return;
 
     case RVAL_TYPE_LIST:
@@ -1318,6 +1302,13 @@ char *RvalToString(Rval rval)
     return StringWriterClose(w);
 }
 
+char *RlistToString(const Rlist *rlist)
+{
+    Writer *w = StringWriter();
+    RlistWrite(w, rlist);
+    return StringWriterClose(w);
+}
+
 unsigned RvalHash(Rval rval, unsigned seed, unsigned max)
 {
     switch (rval.type)
@@ -1406,7 +1397,7 @@ static JsonElement *RlistToJson(Rlist *list)
             break;
 
         default:
-            assert(false && "Unsupported item type in rlist");
+            ProgrammingError("Unsupported item type in rlist");
             break;
         }
     }
@@ -1416,7 +1407,8 @@ static JsonElement *RlistToJson(Rlist *list)
 
 JsonElement *RvalToJson(Rval rval)
 {
-    assert(rval.item);
+    /* Only empty Rlist can be NULL. */
+    assert(rval.item || rval.type == RVAL_TYPE_LIST);
 
     switch (rval.type)
     {
@@ -1443,66 +1435,66 @@ JsonElement *RvalToJson(Rval rval)
  */
 void RlistFlatten(EvalContext *ctx, Rlist **list)
 {
-    Rlist *prev = NULL, *next;
+    Rlist *next;
     for (Rlist *rp = *list; rp != NULL; rp = next)
     {
         next = rp->next;
-        if (rp->val.type != RVAL_TYPE_SCALAR)
-        {
-            prev = rp;
-            continue;
-        }
 
-        char naked[CF_BUFSIZE] = "";
-        if (IsNakedVar(RlistScalarValue(rp), '@'))
+        if (rp->val.type == RVAL_TYPE_SCALAR      &&
+            IsNakedVar(RlistScalarValue(rp), '@'))
         {
+            char naked[CF_MAXVARSIZE];
             GetNaked(naked, RlistScalarValue(rp));
 
+            /* Make sure there are no inner expansions to take place, like if
+             * rp was "@{blah_$(blue)}".  */
             if (!IsExpandable(naked))
             {
+                Log(LOG_LEVEL_DEBUG,
+                    "Flattening slist: %s", RlistScalarValue(rp));
+
                 VarRef *ref = VarRefParse(naked);
-                DataType value_type = CF_DATA_TYPE_NONE;
+                DataType value_type;
                 const void *value = EvalContextVariableGet(ctx, ref, &value_type);
                 VarRefDestroy(ref);
 
-                if (value)
+                if (value_type == CF_DATA_TYPE_NONE)
                 {
-                    switch (DataTypeToRvalType(value_type))
-                    {
-                    case RVAL_TYPE_LIST:
-                        {
-                            RlistDestroyEntry(list, rp);
-
-                            for (const Rlist *srp = value; srp != NULL; srp = srp->next)
-                            {
-                                Rlist *nrp = xmalloc(sizeof(Rlist));
-                                nrp->val = RvalCopy(srp->val);
-                                nrp->next = next;
-
-                                if (prev)
-                                {
-                                    prev->next = nrp;
-                                }
-                                else
-                                {
-                                    *list = nrp;
-                                }
-
-                                prev = nrp;
-                            }
-                        }
-                        continue;
-
-                    default:
-                        Log(LOG_LEVEL_WARNING, "Attempted to dereference variable '%s' using @ but variable did not resolve to a list",
-                            RlistScalarValue(rp));
-                        break;
-                    }
+                    assert(value == NULL);
+                    continue;                         /* undefined variable */
                 }
+
+                if (DataTypeToRvalType(value_type) != RVAL_TYPE_LIST)
+                {
+                    Log(LOG_LEVEL_WARNING,
+                        "'%s' failed - variable is not list but %s",
+                        RlistScalarValue(rp), DataTypeToString(value_type));
+                    continue;
+                }
+
+                /* NOTE: Remember that value can be NULL as an empty Rlist. */
+
+                /* at_node: just a mnemonic name for the
+                            list node with @{blah}. */
+                Rlist *at_node      = rp;
+                Rlist *insert_after = at_node;
+                for (const Rlist *rp2 = value; rp2 != NULL; rp2 = rp2->next)
+                {
+                    assert(insert_after != NULL);
+
+                    RlistInsertAfter(insert_after, RvalCopy(rp2->val));
+                    insert_after = insert_after->next;
+                }
+
+                /* Make sure we won't miss any element. */
+                assert(insert_after->next == next);
+                RlistDestroyEntry(list, at_node);   /* Delete @{blah} entry */
+
+                char *list_s = RlistToString(*list);
+                Log(LOG_LEVEL_DEBUG, "Flattened slist: %s", list_s);
+                free(list_s);
             }
         }
-
-        prev = rp;
     }
 }
 
@@ -1512,17 +1504,16 @@ bool RlistEqual(const Rlist *list1, const Rlist *list2)
 
     for (rp1 = list1, rp2 = list2; rp1 != NULL && rp2 != NULL; rp1 = rp1->next, rp2 = rp2->next)
     {
-        if (rp1->val.item && rp2->val.item)
+        if (rp1->val.item != NULL &&
+            rp2->val.item != NULL)
         {
-            const Rlist *rc1, *rc2;
-
             if (rp1->val.type == RVAL_TYPE_FNCALL || rp2->val.type == RVAL_TYPE_FNCALL)
             {
                 return false;      // inconclusive
             }
 
-            rc1 = rp1;
-            rc2 = rp2;
+            const Rlist *rc1 = rp1;
+            const Rlist *rc2 = rp2;
 
             // Check for list nesting with { fncall(), "x" ... }
 
@@ -1546,10 +1537,15 @@ bool RlistEqual(const Rlist *list1, const Rlist *list2)
                 return false;
             }
         }
-        else
+        else if ((rp1->val.item != NULL && rp2->val.item == NULL) ||
+                 (rp1->val.item == NULL && rp2->val.item != NULL))
         {
             return false;
         }
+        else
+        {
+            assert(rp1->val.item == NULL && rp2->val.item == NULL);
+        }
     }
 
     return true;
@@ -1560,3 +1556,65 @@ bool RlistEqual_untyped(const void *list1, const void *list2)
     return RlistEqual(list1, list2);
 }
 
+/*******************************************************************/
+
+static void RlistAppendContainerPrimitive(Rlist **list, const JsonElement *primitive)
+{
+    assert(JsonGetElementType(primitive) == JSON_ELEMENT_TYPE_PRIMITIVE);
+
+    switch (JsonGetPrimitiveType(primitive))
+    {
+    case JSON_PRIMITIVE_TYPE_BOOL:
+        RlistAppendScalar(list, JsonPrimitiveGetAsBool(primitive) ? "true" : "false");
+        break;
+    case JSON_PRIMITIVE_TYPE_INTEGER:
+        {
+            char *str = StringFromLong(JsonPrimitiveGetAsInteger(primitive));
+            RlistAppendScalar(list, str);
+            free(str);
+        }
+        break;
+    case JSON_PRIMITIVE_TYPE_REAL:
+        {
+            char *str = StringFromDouble(JsonPrimitiveGetAsReal(primitive));
+            RlistAppendScalar(list, str);
+            free(str);
+        }
+        break;
+    case JSON_PRIMITIVE_TYPE_STRING:
+        RlistAppendScalar(list, JsonPrimitiveGetAsString(primitive));
+        break;
+
+    case JSON_PRIMITIVE_TYPE_NULL:
+        break;
+    }
+}
+
+Rlist *RlistFromContainer(const JsonElement *container)
+{
+    Rlist *list = NULL;
+
+    switch (JsonGetElementType(container))
+    {
+    case JSON_ELEMENT_TYPE_PRIMITIVE:
+        RlistAppendContainerPrimitive(&list, container);
+        break;
+
+    case JSON_ELEMENT_TYPE_CONTAINER:
+        {
+            JsonIterator iter = JsonIteratorInit(container);
+            const JsonElement *child;
+
+            while (NULL != (child = JsonIteratorNextValue(&iter)))
+            {
+                if (JsonGetElementType(child) == JSON_ELEMENT_TYPE_PRIMITIVE)
+                {
+                    RlistAppendContainerPrimitive(&list, child);
+                }
+            }
+        }
+        break;
+    }
+
+    return list;
+}
