@@ -1,7 +1,7 @@
 /*
-   Copyright (C) CFEngine AS
+   Copyright 2017 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by CFEngine AS.
+   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -29,6 +29,7 @@
 #include <writer.h>
 #include <json.h>
 
+/* NOTE: an empty Rlist is simply NULL. */
 struct Rlist_
 {
     Rval val;
@@ -52,6 +53,7 @@ Rval RvalCopyRewriter(Rval rval, JsonElement *map);
 void RvalDestroy(Rval rval);
 JsonElement *RvalToJson(Rval rval);
 char *RvalToString(Rval rval);
+char *RlistToString(const Rlist *rlist);
 void RvalWrite(Writer *writer, Rval rval);
 void RvalWriteQuoted(Writer *writer, Rval rval);
 unsigned RvalHash(Rval rval, unsigned seed, unsigned max);
@@ -89,6 +91,8 @@ Rlist *RlistAppendAllTypes(Rlist **start, const void *item, RvalType type, bool
 Rlist *RlistFromSplitString(const char *string, char sep);
 Rlist *RlistFromSplitRegex(const char *string, const char *regex, size_t max_entries, bool allow_blanks);
 Rlist *RlistFromRegexSplitNoOverflow(const char *string, const char *regex, int max);
+Rlist *RlistFromContainer(const JsonElement *container);
+
 void RlistWrite(Writer *writer, const Rlist *list);
 Rlist *RlistLast(Rlist *start);
 void RlistFilter(Rlist **list, bool (*KeepPredicate)(void *item, void *predicate_data), void *predicate_user_data, void (*DestroyItem)(void *item));
