@@ -1,7 +1,7 @@
 /*
-   Copyright (C) CFEngine AS
+   Copyright 2017 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by CFEngine AS.
+   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -207,7 +207,7 @@ bool ShellCommandReturnsZero(const char *command, ShellType shell)
 
             if (LogGetGlobalLevel() < LOG_LEVEL_INFO)
             {
-                if ((devnull = safe_open("/dev/null", O_WRONLY)) == -1)
+                if ((devnull = open("/dev/null", O_WRONLY)) == -1)
                 {
                     Log(LOG_LEVEL_ERR, "Command '%s' failed. (open: %s)", command, GetErrorStr());
                     exit(EXIT_FAILURE);
