@@ -1,7 +1,7 @@
 /*
-   Copyright (C) CFEngine AS
+   Copyright 2017 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by CFEngine AS.
+   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -26,6 +26,7 @@
 #include <map.h>
 #include <rlist.h>
 #include <writer.h>
+#include <conversion.h>                                 /* DataTypeToString */
 
 
 static void VariableDestroy(Variable *var);                 /* forward declaration */
@@ -37,13 +38,11 @@ static void VariableDestroy_untyped(void *var)
 
 
 /**
-   Define VarMap.
-   Key:   VarRef
-   Value: Variable
+   Define "VarMap" hash table.
+       Key:   VarRef
+       Value: Variable
 */
-
 TYPED_MAP_DECLARE(Var, VarRef *, Variable *)
-
 TYPED_MAP_DEFINE(Var, VarRef *, Variable *,
                  VarRefHash_untyped,
                  VarRefEqual_untyped,
@@ -95,9 +94,51 @@ void VariableTableDestroy(VariableTable *table)
     }
 }
 
+/* NULL return value means variable not found. */
 Variable *VariableTableGet(const VariableTable *table, const VarRef *ref)
 {
-    return VarMapGet(table->vars, ref);
+    Variable *v = VarMapGet(table->vars, ref);
+
+    char *ref_s = VarRefToString(ref, true);              /* TODO optimise */
+
+    if (v != NULL)
+    {
+        CF_ASSERT(v->rval.item != NULL || DataTypeIsIterable(v->type),
+                  "VariableTableGet(%s): "
+                  "Only iterables (Rlists) are allowed to be NULL",
+                  ref_s);
+    }
+
+    if (LogModuleEnabled(LOG_MOD_VARTABLE))
+    {
+        Buffer *buf = BufferNew();
+        BufferPrintf(buf, "VariableTableGet(%s): %s", ref_s,
+                     v ? DataTypeToString(v->type) : "NOT FOUND");
+        if (v != NULL)
+        {
+            char *value_s;
+            BufferAppendString(buf, "  => ");
+            if (DataTypeIsIterable(v->type) &&
+                v->rval.item == NULL)
+            {
+                value_s = xstrdup("EMPTY");
+            }
+            else
+            {
+                value_s = RvalToString(v->rval);
+            }
+
+            BufferAppendString(buf, value_s);
+            free(value_s);
+        }
+
+        LogDebug(LOG_MOD_VARTABLE, "%s", BufferGet(buf));
+
+        BufferDestroy(buf);
+    }
+
+    free(ref_s);
+    return v;
 }
 
 bool VariableTableRemove(VariableTable *table, const VarRef *ref)
@@ -132,6 +173,21 @@ bool VariableTablePut(VariableTable *table, const VarRef *ref,
 {
     assert(VarRefIsQualified(ref));
 
+    /* TODO assert there are no CF_NS or '.' in the variable name? */
+
+    if (LogModuleEnabled(LOG_MOD_VARTABLE))
+    {
+        char *value_s = RvalToString(*rval);
+        LogDebug(LOG_MOD_VARTABLE, "VariableTablePut(%s): %s  => %s",
+            ref->lval, DataTypeToString(type),
+            rval->item ? value_s : "EMPTY");
+        free(value_s);
+    }
+
+    CF_ASSERT(rval != NULL || DataTypeIsIterable(type),
+              "VariableTablePut(): "
+              "Only iterables (Rlists) are allowed to be NULL");
+
     Variable *var = VariableNew(VarRefCopy(ref), RvalCopy(*rval), type,
                                 StringSetFromString(tags, ','), promise);
     return VarMapInsert(table->vars, var->ref, var);
