@@ -1,7 +1,7 @@
 /*
-   Copyright (C) CFEngine AS
+   Copyright 2017 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by CFEngine AS.
+   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -277,6 +277,7 @@ static int IsCf3Scalar(char *str)
     return vars;
 }
 
+/* Extract everything up to the dollar sign. */
 size_t ExtractScalarPrefix(Buffer *out, const char *str, size_t len)
 {
     assert(str);
@@ -346,10 +347,8 @@ static const char *ReferenceEnd(const char *str, size_t len)
         case '}':
             if (stack[level] != '{')
             {
-                Writer *w = StringWriter();
-                WriterWriteLen(w, str, len);
-                Log(LOG_LEVEL_ERR, "Variable reference bracket mismatch '%s'", StringWriterData(w));
-                WriterClose(w);
+                Log(LOG_LEVEL_ERR, "Variable reference bracket mismatch '%.*s'",
+                    (int) len, str);
                 return NULL;
             }
             level--;
@@ -357,10 +356,8 @@ static const char *ReferenceEnd(const char *str, size_t len)
         case ')':
             if (stack[level] != '(')
             {
-                Writer *w = StringWriter();
-                WriterWriteLen(w, str, len);
-                Log(LOG_LEVEL_ERR, "Variable reference bracket mismatch '%s'", StringWriterData(w));
-                WriterClose(w);
+                Log(LOG_LEVEL_ERR, "Variable reference bracket mismatch '%.*s'",
+                    (int) len, str);
                 return NULL;
             }
             level--;
@@ -376,6 +373,10 @@ static const char *ReferenceEnd(const char *str, size_t len)
     return NULL;
 }
 
+/**
+ * Extract variable inside dollar-paren.
+ * @param extract_inner ignore opening dollar-paren and closing paren.
+ */
 bool ExtractScalarReference(Buffer *out, const char *str, size_t len, bool extract_inner)
 {
     if (len <= 1)
@@ -399,26 +400,23 @@ bool ExtractScalarReference(Buffer *out, const char *str, size_t len, bool extra
             }
             else
             {
-                return ExtractScalarReference(out, dollar_point + 1, remaining - 1, extract_inner);
+                return ExtractScalarReference(out, dollar_point + 1,
+                                              remaining - 1, extract_inner);
             }
         }
 
         if (!close_point)
         {
-            Writer *w = StringWriter();
-            WriterWriteLen(w, str, len);
-            Log(LOG_LEVEL_ERR, "Variable reference close mismatch '%s'", StringWriterData(w));
-            WriterClose(w);
+            Log(LOG_LEVEL_ERR, "Variable reference close mismatch '%.*s'",
+                (int) len, str);
             return false;
         }
 
         size_t outer_len = close_point - dollar_point + 1;
         if (outer_len <= 3)
         {
-            Writer *w = StringWriter();
-            WriterWriteLen(w, str, len);
-            Log(LOG_LEVEL_ERR, "Empty variable reference close mismatch '%s'", StringWriterData(w));
-            WriterClose(w);
+            Log(LOG_LEVEL_ERR, "Empty variable reference close mismatch '%.*s'",
+                (int) len, str);
             return false;
         }
 
