@@ -1,7 +1,7 @@
 /*
-   Copyright (C) CFEngine AS
+   Copyright 2017 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by CFEngine AS.
+   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -123,12 +123,11 @@ PromiseResult VerifyReportPromise(EvalContext *ctx, const Promise *pp)
 
 static void ReportToLog(const char *message)
 {
-    int report_size = strlen(message) + 4;
-    char *report_message = malloc(report_size);
+    char *report_message;
+    xasprintf(&report_message, "R: %s", message);
 
-    xsnprintf(report_message, report_size, "R: %s", message);
-
-    fprintf(stdout, "%s\n", report_message);
+    fputs(report_message, stdout);
+    fputc('\n', stdout);
     LogToSystemLog(report_message, LOG_LEVEL_NOTICE);
 
     free(report_message);
