@@ -1,7 +1,7 @@
 /*
-   Copyright (C) CFEngine AS
+   Copyright 2017 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by CFEngine AS.
+   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -25,9 +25,12 @@
 #ifndef CFENGINE_VERIFY_VARS_H
 #define CFENGINE_VERIFY_VARS_H
 
+
 #include <eval_context.h>
 #include <policy.h>
 
-PromiseResult VerifyVarPromise(EvalContext *ctx, const Promise *pp, bool allow_duplicates);
+
+PromiseResult VerifyVarPromise(EvalContext *ctx, const Promise *pp, void *param);
+
 
 #endif
