@@ -1,7 +1,7 @@
 /*
-   Copyright (C) CFEngine AS
+   Copyright 2017 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by CFEngine AS.
+   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -47,11 +47,11 @@ bool FileCanOpen(const char *path, const char *modes);
 
 /* Write LEN bytes at PTR to descriptor DESC, retrying if interrupted.
    Return LEN upon success, write's (negative) error code otherwise.  */
-int FullWrite(int desc, const char *ptr, size_t len);
+ssize_t FullWrite(int desc, const char *ptr, size_t len);
 
 /* Read up to LEN bytes (or EOF) to PTR from descriptor DESC, retrying if interrupted.
    Return amount of bytes read upon success, -1 otherwise */
-int FullRead(int desc, char *ptr, size_t len);
+ssize_t FullRead(int desc, char *ptr, size_t len);
 
 int IsDirReal(const char *path);
 
@@ -113,4 +113,22 @@ bool SetCloseOnExec(int fd, bool enable);
  */
 bool DeleteDirectoryTree(const char *path);
 
+bool FileSparseWrite(int fd, const void *buf, size_t count,
+                     bool *wrote_hole);
+bool FileSparseCopy(int sd, const char *src_name,
+                    int dd, const char *dst_name,
+                    size_t blk_size,
+                    size_t *total_bytes_written,
+                    bool   *last_write_was_a_hole);
+bool FileSparseClose(int fd, const char *filename,
+                     bool do_sync,
+                     size_t total_bytes_written,
+                     bool last_write_was_hole);
+
+/**
+ * @brief Works exactly like posix 'getline', EXCEPT it does not include carriage return at the end.
+ * @return -1 on error OR EOF, so check. Or bytes in buff without excluding terminator.
+ */
+ssize_t CfReadLine(char **buff, size_t *size, FILE *fp);
+
 #endif
