@@ -1,7 +1,7 @@
 /*
-   Copyright (C) CFEngine AS
+   Copyright 2017 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by CFEngine AS.
+   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -85,7 +85,7 @@ static int IPV4_hex_parser(const char *source, struct IPV4Address *address)
         // shortcut for the 0a0b0c0d format
         unsigned int a, b, c, d, pport = 0;
         if (strlen(source) == 8 &&
-            4 == sscanf(source, "%2x%2x%2x%2x", &a, &b, &c, &d))
+            sscanf(source, "%2x%2x%2x%2x", &a, &b, &c, &d) == 4)
         {
             address->octets[3] = a;
             address->octets[2] = b;
@@ -97,7 +97,7 @@ static int IPV4_hex_parser(const char *source, struct IPV4Address *address)
 
         // shortcut for the 0a0b0c0d:0e0f format
         if (strlen(source) == 8+1+4 &&
-            5 == sscanf(source, "%2x%2x%2x%2x:%4x", &a, &b, &c, &d, &pport))
+            sscanf(source, "%2x%2x%2x%2x:%4x", &a, &b, &c, &d, &pport) == 5)
         {
             address->octets[3] = a;
             address->octets[2] = b;
@@ -378,30 +378,30 @@ static int IPV6_hex_parser(const char *source, struct IPV6Address *address)
         unsigned int a, b, c, d, e, f, g, h, pport = 0;
 
         if (strlen(source) == 32 &&
-            8 == sscanf(source, "%4x%4x%4x%4x%4x%4x%4x%4x", &a, &b, &c, &d, &e, &f, &g, &h))
+            sscanf(source, "%4x%4x%4x%4x%4x%4x%4x%4x", &a, &b, &c, &d, &e, &f, &g, &h) == 8)
         {
-            address->sixteen[7] = a;
-            address->sixteen[6] = b;
-            address->sixteen[5] = c;
-            address->sixteen[4] = d;
-            address->sixteen[3] = e;
-            address->sixteen[2] = f;
-            address->sixteen[1] = g;
-            address->sixteen[0] = h;
+            address->sixteen[0] = a;
+            address->sixteen[1] = b;
+            address->sixteen[2] = c;
+            address->sixteen[3] = d;
+            address->sixteen[4] = e;
+            address->sixteen[5] = f;
+            address->sixteen[6] = g;
+            address->sixteen[7] = h;
             return 0;
         }
 
         if (strlen(source) == 32+1+4 &&
-            9 == sscanf(source, "%4x%4x%4x%4x%4x%4x%4x%4x:%4x", &a, &b, &c, &d, &e, &f, &g, &h, &pport))
+            sscanf(source, "%4x%4x%4x%4x%4x%4x%4x%4x:%4x", &a, &b, &c, &d, &e, &f, &g, &h, &pport) == 9)
         {
-            address->sixteen[7] = a;
-            address->sixteen[6] = b;
-            address->sixteen[5] = c;
-            address->sixteen[4] = d;
-            address->sixteen[3] = e;
-            address->sixteen[2] = f;
-            address->sixteen[1] = g;
-            address->sixteen[0] = h;
+            address->sixteen[0] = a;
+            address->sixteen[1] = b;
+            address->sixteen[2] = c;
+            address->sixteen[3] = d;
+            address->sixteen[4] = e;
+            address->sixteen[5] = f;
+            address->sixteen[6] = g;
+            address->sixteen[7] = h;
             address->port = pport;
             return 0;
         }
