@@ -1,7 +1,7 @@
 /*
-   Copyright (C) CFEngine AS
+   Copyright 2017 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by CFEngine AS.
+   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -829,7 +829,7 @@ static char *JsonEncodeString(const char *unescaped_string)
     return StringWriterClose(writer);
 }
 
-static char *JsonDecodeString(const char *encoded_string)
+char *JsonDecodeString(const char *encoded_string)
 {
     assert(encoded_string);
 
@@ -2177,7 +2177,7 @@ static JsonParseError JsonParseAsObject(void *lookup_context, JsonLookup *lookup
         default:
             // Note the character class excludes ':'.
             // This will match the key from { foo : 2 } but not { -foo: 2 }
-            if (NULL == property_name &&
+            if (property_name == NULL &&
                 StringMatch("^\\w[-\\w]*\\s*:", *data, NULL, NULL))
             {
                 char *colon = strchr(*data, ':');
@@ -2345,7 +2345,7 @@ JsonElement* StringCaptureData(pcre *pattern, const char* regex, const char* dat
 
     Seq *s;
 
-    if (NULL != pattern)
+    if (pattern != NULL)
     {
         s = StringMatchCapturesWithPrecompiledRegex(pattern, data, true);
     }
