@@ -1,7 +1,7 @@
 /*
-   Copyright (C) CFEngine AS
+   Copyright 2017 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by CFEngine AS.
+   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -335,6 +335,10 @@ void globfree(glob_t *pglob);
 # include <sys/time.h>
 #endif
 
+#ifdef HAVE_SYS_RESOURCE_H
+# include <sys/resource.h>
+#endif
+
 #ifndef __MINGW32__
 # include <pwd.h>
 # include <grp.h>
@@ -794,49 +798,69 @@ struct timespec
 /* Copy file defines                                               */
 /*******************************************************************/
 
-            /* Based heavily on cp.c in GNU-fileutils */
-
-#ifndef DEV_BSIZE
+/**
+ * DEV_BSIZE is 512 for most common platforms
+ *           (Linux, AIX on Power, Solaris etc).
+ *
+ * Exceptions:
+ *             HP-UX:       1024
+ *             AIX on PS/2: 4096
+ *             Windows:     undefined
+ */
+#ifndef DEV_BSIZE                       /* usually defined in <sys/param.h> */
 # ifdef BSIZE
 #  define DEV_BSIZE BSIZE
-# else/* !BSIZE */
-#  define DEV_BSIZE 4096
-# endif/* !BSIZE */
-#endif /* !DEV_BSIZE */
+# else
+#  define DEV_BSIZE 512
+# endif
+#endif
 
-/* Extract or fake data from a `struct stat'.
+/**
+   Extract or fake data from a `struct stat'.
    ST_BLKSIZE: Optimal I/O blocksize for the file, in bytes.
-   ST_NBLOCKS: Number of 512-byte blocks in the file
-   (including indirect blocks). */
+               This is tightly coupled to ST_NBLOCKS, i.e it must stand that
+               (s.st_size <= ST_NBLOCKS(s) * ST_BLKSIZE(s))
+   ST_NBLOCKS: Number of blocks in the file, **in ST_BLKSIZE units**
+               WARNING this is different than "stat.st_nblocks" on most systems
+   ST_NBYTES : "disk usage" of the file on the disk, **in bytes**.
 
-#define SMALL_BLOCK_BUF_SIZE 512
+   TODO on Windows here is how to get the "cluster size" i.e. the block size:
+   - send IOCTL_DISK_GET_DRIVE_GEOMETRY_EX and use Geometry.BytesPerSector
+     from DISK_GEOMETRY_EX structure
+   - To check if a file is sparse: File.GetAttributes().SparseFile
+*/
 
-#ifndef HAVE_ST_BLOCKS
-# define ST_BLKSIZE(statbuf) DEV_BSIZE
-# if defined(_POSIX_SOURCE) || !defined(BSIZE)  /* fileblocks.c uses BSIZE.  */
-#  define ST_NBLOCKS(statbuf) (((statbuf).st_size + 512 - 1) / 512)
-# else/* !_POSIX_SOURCE && BSIZE */
-#  define ST_NBLOCKS(statbuf) (st_blocks ((statbuf).st_size))
-# endif/* !_POSIX_SOURCE && BSIZE */
-#else /* HAVE_ST_BLOCKS */
-/* Some systems, like Sequents, return st_blksize of 0 on pipes. */
+/*
+ * Known platforms that don't have stat.st_blocks:
+ *   Windows (MinGW)
+ */
+#ifndef HAVE_STRUCT_STAT_ST_BLOCKS
+
+
+# define ST_BLKSIZE(statbuf)    DEV_BSIZE
+# define ST_NBLOCKS(statbuf)    (((statbuf).st_size + DEV_BSIZE - 1) / DEV_BSIZE)
+# define ST_NBYTES(statbuf)     (ST_NBLOCKS(statbuf) * DEV_BSIZE)
+
+
+#else  /* HAVE_STRUCT_STAT_ST_BLOCKS */
+  /* Some systems, like Sequents, return st_blksize of 0 on pipes. */
 # define ST_BLKSIZE(statbuf) ((statbuf).st_blksize > 0 \
                                ? (statbuf).st_blksize : DEV_BSIZE)
-# if defined(__hpux)
-/* HP-UX counts st_blocks in 1024-byte units.
-   This loses when mixing HP-UX and BSD filesystems with NFS.  */
-#  define ST_NBLOCKS(statbuf) ((statbuf).st_blocks * 2)
-# else/* !hpux */
-#  if defined(_AIX) && defined(_I386)
-/* AIX PS/2 counts st_blocks in 4K units.  */
-#   define ST_NBLOCKS(statbuf) ((statbuf).st_blocks * 8)
-#  else
-      /* not AIX PS/2 */
-#   define ST_NBLOCKS(statbuf) ((statbuf).st_blocks)
-#  endif
-       /* not AIX PS/2 */
-# endif/* !hpux */
-#endif /* HAVE_ST_BLOCKS */
+# define ST_NBLOCKS(statbuf) ((ST_NBYTES(statbuf) + ST_BLKSIZE(statbuf) - 1) \
+                              / ST_BLKSIZE(statbuf))
+
+# if defined(_CRAY)
+#  define ST_NBYTES(statbuf) ((statbuf).st_blocks * ST_BLKSIZE(statbuf))
+# else
+   /* ======= DEFAULT ============================== */
+   /* Most OS give stat.st_blocks in DEV_BSIZE units */
+#  define ST_NBYTES(statbuf) ((statbuf).st_blocks * DEV_BSIZE)
+   /* ============================================== */
+# endif /* CRAY */
+
+
+#endif /* HAVE_STRUCT_STAT_ST_BLOCKS */
+
 
 #ifndef SEEK_CUR
 # define SEEK_CUR 1
@@ -967,6 +991,23 @@ struct timespec
 # include <direct.h>
 #endif
 
+
+/* Some shorthands. */
+
+# if defined(__sun) && !defined(sun)
+#  define sun
+# endif
+
+# if defined(__SVR4) && !defined(SVR4)
+#  define SVR4
+# endif
+
+# if (defined(sun) && defined(SVR4)) || defined (SOLARIS2)
+#  define SUNOS_5
+# endif
+
+
+
 /* Must be always the last one! */
 #include <deprecated.h>
 #include <config.post.h>
