@@ -1,7 +1,7 @@
 /*
-   Copyright (C) CFEngine AS
+   Copyright 2017 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by CFEngine AS.
+   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -857,6 +857,33 @@ int Chop(char *str, size_t max_length)
     return 0;
 }
 
+char *TrimWhitespace(char *s)
+{
+    assert(s);
+
+    // Leading whitespace:
+    while (isspace(s[0]))
+    {
+        ++s;
+    }
+
+    // Empty string (only whitespace):
+    if (s[0] == '\0')
+    {
+        return s;
+    }
+
+    // Trailing whitespace:
+    char *end = s + strlen(s) - 1; // Last byte before '\0'
+    while ( isspace(end[0]) )
+    {
+        --end;
+    }
+    end[1] = '\0';   // Null terminate string after last non space char
+
+    return s;
+}
+
 bool StringEndsWithCase(const char *str, const char *suffix, const bool case_fold)
 {
     size_t str_len = strlen(str);
@@ -911,33 +938,28 @@ bool StringStartsWith(const char *str, const char *prefix)
     return true;
 }
 
-void *MemSpan(const void *mem, char c, size_t n)
+
+/**
+ * Returns pointer to the first byte in #buf that is not #c. Returns NULL if
+ * all of #buf contains only bytes of value #c.
+ *
+ * @NOTE this functions complements memchr() from POSIX.
+ *
+ * @TODO move to libcompat, it appears to be available in some systems.
+ */
+void *memcchr(const void *buf, int c, size_t buf_size)
 {
-    const char *end = mem + n;
-    for (; (char*)mem < end; ++mem)
+    const char *cbuf = buf;
+    for (size_t i = 0; i < buf_size; i++)
     {
-        if (*((char *)mem) != c)
+        if (cbuf[i] != c)
         {
-            return (char *)mem;
+            return (void *) &cbuf[i];                    /* cast-away const */
         }
     }
-
-    return (char *)mem;
+    return NULL;
 }
 
-void *MemSpanInverse(const void *mem, char c, size_t n)
-{
-    const char *end = mem + n;
-    for (; (char*)mem < end; ++mem)
-    {
-        if (*((char*)mem) == c)
-        {
-            return (char *)mem;
-        }
-    }
-
-    return (char *)mem;
-}
 
 /*
  * @brief extract info from input string given two types of constraints:
@@ -984,63 +1006,6 @@ bool StringAppend(char *dst, const char *src, size_t n)
     return (i < n || !src[j]);
 }
 
-bool StringAppendPromise(char *dst, const char *src, size_t n)
-{
-    int i, j;
-    n--;
-    for (i = 0; i < n && dst[i]; i++)
-    {
-    }
-    for (j = 0; i < n && src[j]; i++, j++)
-    {
-        const char ch = src[j];
-        switch (ch)
-        {
-        case '*':
-            dst[i] = ':';
-            break;
-
-        case '#':
-            dst[i] = '.';
-            break;
-
-        default:
-            dst[i] = ch;
-            break;
-        }
-    }
-    dst[i] = '\0';
-    return (i < n || !src[j]);
-}
-
-bool StringAppendAbbreviatedPromise(char *dst, const char *src, size_t n, const size_t max_fragment)
-{
-    /* check if `src` contains a new line (may happen for "insert_lines") */
-    const char *const nl = strchr(src, '\n');
-    if (NULL == nl)
-    {
-        return StringAppendPromise(dst, src, n);
-    }
-    else
-    {
-        /* `src` contains a newline: abbreviate it by taking the first and last few characters */
-        static const char sep[] = "...";
-        char abbr[sizeof(sep) + 2 * max_fragment];
-        const int head = (nl > src + max_fragment) ? max_fragment : (nl - src);
-        const char * last_line = strrchr(src, '\n') + 1;
-        assert(last_line); /* not max_fragmentULL, we know we have at least one '\n' */
-        const int tail = strlen(last_line);
-        if (tail > max_fragment)
-        {
-            last_line += tail - max_fragment;
-        }
-        memcpy(abbr, src, head);
-        strcpy(abbr + head, sep);
-        strcat(abbr, last_line);
-        return StringAppendPromise(dst, abbr, n);
-    }
-}
-
 /**
  *  Canonify #src into #dst.
  *
